Package: okxr
Title: R Interface to the 'OKX' REST API
Version: 0.4.8
Authors@R: c(
    person('Oliver', 'Zhou', email = 'oliver.yxzhou@gmail.com', role = c('aut', 'cre')),
    person('Lily', 'Li', email = 'lilyli.qc.hk@gmail.com', role = 'aut')
    )
Author: Oliver Zhou [aut, cre],
  Lily Li [aut]
Maintainer: Oliver Zhou <oliver.yxzhou@gmail.com>
Description: Provides lightweight R wrappers for the 'OKX' REST API, covering 
    endpoints for market data, trading, account management, asset balances, 
    and copy trading. The upstream API reference is available at
    <https://www.okx.com/docs-v5/en/>.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
Imports: rlang, httr, jsonlite, digest, base64enc, data.table
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Collate: okxr-package.R def_constants.R utils_request_helpers.R
        utils_auth.R utils_parser.R utils_get_generator.R
        utils_post_generator.R utils_labels.R wrappers_get_market.R
        wrappers_get_asset.R wrappers_get_account.R
        wrappers_get_trade.R wrappers_get_copy_trade.R
        wrappers_post_asset.R wrappers_post_configure.R
        wrappers_post_trade.R
URL: https://github.com/OliverLDS/okxr
BugReports: https://github.com/OliverLDS/okxr/issues
RoxygenNote: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-01 13:38:39 UTC; oliver
Repository: CRAN
Date/Publication: 2026-08-01 13:50:02 UTC
Built: R 4.5.2; ; 2026-08-01 17:10:34 UTC; unix

Package: okxr
Title: R Interface to the 'OKX' REST API
Version: 0.4.5
Authors@R: c(
    person('Oliver', 'Zhou', email = 'oliver.yxzhou@gmail.com', role = c('aut', 'cre')),
    person('Lily', 'Li', email = 'lilyli.qc.hk@gmail.com', role = 'aut')
    )
Author: Oliver Zhou [aut, cre],
  Lily Li [aut]
Maintainer: Oliver Zhou <oliver.yxzhou@gmail.com>
Description: Provides lightweight R wrappers for the 'OKX' REST API, covering 
    endpoints for market data, trading, account management, asset balances, 
    and copy trading. The upstream API reference is available at
    <https://www.okx.com/docs-v5/en/>.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
Imports: rlang, httr, jsonlite, digest, base64enc, data.table
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Collate: okxr-package.R def_constants.R utils_request_helpers.R
        utils_auth.R utils_parser.R utils_get_generator.R
        utils_post_generator.R utils_labels.R wrappers_get_market.R
        wrappers_get_asset.R wrappers_get_account.R
        wrappers_get_trade.R wrappers_get_copy_trade.R
        wrappers_post_asset.R wrappers_post_configure.R
        wrappers_post_trade.R
URL: https://github.com/OliverLDS/okxr
BugReports: https://github.com/OliverLDS/okxr/issues
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2026-05-08 08:16:44 UTC; oliver
Repository: CRAN
Date/Publication: 2026-05-08 11:00:02 UTC
Built: R 4.6.0; ; 2026-05-08 13:20:07 UTC; unix

# Generated by roxygen2: do not edit by hand

export(get_account_adjust_leverage_info)
export(get_account_balance)
export(get_account_bill_types)
export(get_account_bills)
export(get_account_bills_archive)
export(get_account_bills_history_archive)
export(get_account_collateral_assets)
export(get_account_config)
export(get_account_greeks)
export(get_account_instruments)
export(get_account_interest_accrued)
export(get_account_interest_limits)
export(get_account_interest_rate)
export(get_account_leverage_info)
export(get_account_max_avail_size)
export(get_account_max_loan)
export(get_account_max_size)
export(get_account_max_withdrawal)
export(get_account_mmp_config)
export(get_account_move_positions_history)
export(get_account_position_risk)
export(get_account_position_tiers)
export(get_account_positions)
export(get_account_positions_history)
export(get_account_precheck_set_delta_neutral)
export(get_account_risk_state)
export(get_account_set_account_switch_precheck)
export(get_account_spot_borrow_repay_history)
export(get_account_subaccount_balances)
export(get_account_subaccount_max_withdrawal)
export(get_account_subtypes)
export(get_account_trade_fee)
export(get_asset_asset_valuation)
export(get_asset_balances)
export(get_asset_bills)
export(get_asset_bills_history)
export(get_asset_convert_currencies)
export(get_asset_convert_currency_pair)
export(get_asset_convert_history)
export(get_asset_currencies)
export(get_asset_deposit_address)
export(get_asset_deposit_history)
export(get_asset_deposit_withdraw_status)
export(get_asset_exchange_list)
export(get_asset_non_tradable_assets)
export(get_asset_transfer_state)
export(get_asset_withdrawal_history)
export(get_copy_trade_config)
export(get_copy_trade_current_subpos)
export(get_copy_trade_historical_subpos)
export(get_copy_trade_instruments)
export(get_copy_trade_my_leaders)
export(get_copy_trade_profit_sharing_details)
export(get_copy_trade_public_config)
export(get_copy_trade_public_copy_traders)
export(get_copy_trade_public_current_subpositions)
export(get_copy_trade_public_lead_traders)
export(get_copy_trade_public_pnl)
export(get_copy_trade_public_preference_currency)
export(get_copy_trade_public_stats)
export(get_copy_trade_public_subpositions_history)
export(get_copy_trade_public_weekly_pnl)
export(get_copy_trade_settings)
export(get_copy_trade_total_profit_sharing)
export(get_copy_trade_total_unrealized_profit_sharing)
export(get_copy_trade_unrealized_profit_sharing_details)
export(get_market_block_ticker)
export(get_market_block_tickers)
export(get_market_books)
export(get_market_books_rpi)
export(get_market_candles)
export(get_market_exchange_rate)
export(get_market_history_candles)
export(get_market_history_index_candles)
export(get_market_history_mark_price_candles)
export(get_market_history_trades)
export(get_market_index_candles)
export(get_market_index_components)
export(get_market_index_tickers)
export(get_market_mark_price_candles)
export(get_market_option_instrument_family_trades)
export(get_market_platform_24_volume)
export(get_market_ticker)
export(get_market_tickers)
export(get_market_trades)
export(get_public_block_trades)
export(get_public_convert_contract_coin)
export(get_public_delivery_exercise_history)
export(get_public_discount_rate_interest_free_quota)
export(get_public_economic_calendar)
export(get_public_estimated_price)
export(get_public_estimated_settlement_info)
export(get_public_event_contract_events)
export(get_public_event_contract_markets)
export(get_public_event_contract_series)
export(get_public_funding_rate)
export(get_public_funding_rate_history)
export(get_public_instrument_tick_bands)
export(get_public_instruments)
export(get_public_insurance_fund)
export(get_public_interest_rate_loan_quota)
export(get_public_mark_price)
export(get_public_mm_instrument_types)
export(get_public_open_interest)
export(get_public_opt_summary)
export(get_public_option_trades)
export(get_public_position_tiers)
export(get_public_premium_history)
export(get_public_price_limit)
export(get_public_settlement_history)
export(get_public_time)
export(get_public_underlying)
export(get_trade_account_rate_limit)
export(get_trade_easy_convert_currency_list)
export(get_trade_easy_convert_history)
export(get_trade_fills)
export(get_trade_fills_history)
export(get_trade_one_click_repay_currency_list)
export(get_trade_one_click_repay_currency_list_v2)
export(get_trade_one_click_repay_history)
export(get_trade_one_click_repay_history_v2)
export(get_trade_order)
export(get_trade_order_algo)
export(get_trade_orders_algo_history)
export(get_trade_orders_algo_pending)
export(get_trade_orders_history)
export(get_trade_orders_history_7d)
export(get_trade_orders_history_archive)
export(get_trade_orders_pending)
export(get_user_glp_historical_performance)
export(get_user_glp_today_performance)
export(get_var_label)
export(post_account_account_level_switch_preset)
export(post_account_mmp_config)
export(post_account_mmp_reset)
export(post_account_move_positions)
export(post_account_position_margin_balance)
export(post_account_set_account_level)
export(post_account_set_auto_loan)
export(post_account_set_auto_repay)
export(post_account_set_collateral_assets)
export(post_account_set_fee_type)
export(post_account_set_greeks)
export(post_account_set_leverage)
export(post_account_set_position_mode)
export(post_account_spot_manual_borrow_repay)
export(post_asset_cancel_withdrawal)
export(post_asset_convert_estimate_quote)
export(post_asset_convert_trade)
export(post_asset_transfer)
export(post_asset_withdrawal)
export(post_trade_amend_algos)
export(post_trade_amend_batch_orders)
export(post_trade_amend_order)
export(post_trade_batch_orders)
export(post_trade_cancel_algos)
export(post_trade_cancel_all_after)
export(post_trade_cancel_batch_orders)
export(post_trade_cancel_order)
export(post_trade_close_position)
export(post_trade_mass_cancel)
export(post_trade_order)
export(post_trade_order_algo)
export(post_trade_order_precheck)
export(set_okxr_options)
importFrom(data.table,as.data.table)
importFrom(httr,content)
importFrom(rlang,"%||%")

# okxr: R Interface to the OKX API

`okxr` is an R package for working with the OKX REST API from R. It provides
typed wrappers for market data, account endpoints, asset history, trading, and
copy trading, with shared request signing and schema-based response parsing.

## Status

`okxr` is available on CRAN. Install the stable release from CRAN or use the
GitHub repository for development versions between CRAN releases.

Current development release: `v0.4.5`

## Installation

Stable CRAN release:

```r
install.packages("okxr")
```

Development release from GitHub:

```r
# install.packages("devtools")
devtools::install_github("OliverLDS/okxr")
```

## Features

* Signed OKX REST requests with HMAC authentication
* Typed parsing into tabular R objects with variable labels
* Shared internal GET/POST generators backed by endpoint specs
* User-facing wrappers for account, asset, market, trade, and copy-trading APIs
* Configurable raw vs parsed return mode via `set_okxr_options()`

## Setup

```r
config <- list(
  api_key = "your_api_key",
  secret_key = "your_secret_key",
  passphrase = "your_passphrase"
)
```

Public market and public reference endpoints can be called without credentials.
Private account, asset, trade, and copy-trading endpoints require signed OKX
API credentials.

Do not store real credentials in committed R scripts, examples, or tests. For
interactive use, load them from environment variables or another local secret
store. If you use OKX simulated trading, set `demo = TRUE` in `config`.

```r
config <- list(
  api_key = Sys.getenv("OKX_API_KEY"),
  secret_key = Sys.getenv("OKX_SECRET_KEY"),
  passphrase = Sys.getenv("OKX_PASSPHRASE"),
  demo = TRUE,
  timeout = 10
)
```

## Return values

By default, wrappers return parsed `data.table` objects with typed columns and
variable labels where schemas are defined. Use `set_okxr_options(raw_data = TRUE)`
to return the raw OKX `data` payload instead.

Network failures, request timeouts, OKX error responses, or empty API `data`
payloads may return `NULL` with a warning. Request timeout defaults to 10
seconds and can be set globally with `set_okxr_options(timeout = 15)` or per
request with `config$timeout`.

## Examples

Public market examples require network access but no OKX credentials. Private
account, trading, asset, and copy-trading examples require valid OKX credentials;
trading examples may have account side effects.

### Market data

```r
get_market_candles(
  inst_id = "BTC-USDT",
  bar = "1m",
  limit = 100
)
```

### Account data

```r
get_account_balance(config = config)
get_account_positions(config = config)
```

### Place an order

```r
post_trade_order(
  inst_id = "BTC-USDT",
  td_mode = "cross",
  side = "buy",
  ord_type = "market",
  sz = "0.01",
  config = config
)
```

### Copy trading

```r
get_copy_trade_my_leaders(config = config)
get_copy_trade_current_subpos(config = config)
```

## Wrapper categories

| Category | Method | Example function |
| --- | --- | --- |
| market | GET | `get_market_candles()` |
| market | GET | `get_market_tickers()` |
| market | GET | `get_market_trades()` |
| market | GET | `get_market_mark_price_candles()` |
| market | GET | `get_market_index_tickers()` |
| market | GET | `get_market_index_candles()` |
| market | GET | `get_market_index_components()` |
| public | GET | `get_public_time()` |
| public | GET | `get_public_underlying()` |
| public | GET | `get_public_estimated_price()` |
| public | GET | `get_public_estimated_settlement_info()` |
| public | GET | `get_public_opt_summary()` |
| public | GET | `get_public_position_tiers()` |
| public | GET | `get_public_discount_rate_interest_free_quota()` |
| public | GET | `get_public_interest_rate_loan_quota()` |
| public | GET | `get_public_insurance_fund()` |
| public | GET | `get_public_block_trades()` |
| public | GET | `get_public_convert_contract_coin()` |
| public | GET | `get_public_instrument_tick_bands()` |
| public | GET | `get_public_premium_history()` |
| public | GET | `get_public_option_trades()` |
| account | GET | `get_account_balance()` |
| account | GET | `get_account_instruments()` |
| account | GET | `get_account_subtypes()` |
| account | GET | `get_account_adjust_leverage_info()` |
| account | GET | `get_account_position_risk()` |
| account | GET | `get_account_max_loan()` |
| account | GET | `get_account_max_size()` |
| account | GET | `get_account_max_avail_size()` |
| account | GET | `get_account_trade_fee()` |
| account | GET | `get_account_interest_rate()` |
| account | GET | `get_account_interest_accrued()` |
| account | GET | `get_account_max_withdrawal()` |
| account | GET | `get_account_risk_state()` |
| account | GET | `get_account_interest_limits()` |
| account | GET | `get_account_greeks()` |
| account | GET | `get_account_position_tiers()` |
| account | GET | `get_account_collateral_assets()` |
| account | GET | `get_account_mmp_config()` |
| account | GET | `get_account_move_positions_history()` |
| account | GET | `get_account_precheck_set_delta_neutral()` |
| account | GET | `get_account_bills_history_archive()` |
| account | GET | `get_account_subaccount_balances()` |
| account | GET | `get_account_subaccount_max_withdrawal()` |
| account | GET | `get_account_set_account_switch_precheck()` |
| account | GET | `get_account_spot_borrow_repay_history()` |
| account | GET | `get_account_bills()` |
| account | POST | `post_account_set_leverage()` |
| account | POST | `post_account_set_position_mode()` |
| account | POST | `post_account_set_fee_type()` |
| account | POST | `post_account_set_greeks()` |
| account | POST | `post_account_set_auto_repay()` |
| account | POST | `post_account_set_auto_loan()` |
| account | POST | `post_account_set_account_level()` |
| account | POST | `post_account_set_collateral_assets()` |
| account | POST | `post_account_position_margin_balance()` |
| account | POST | `post_account_spot_manual_borrow_repay()` |
| account | POST | `post_account_account_level_switch_preset()` |
| account | POST | `post_account_mmp_reset()` |
| account | POST | `post_account_mmp_config()` |
| account | POST | `post_account_move_positions()` |
| asset | GET | `get_asset_balances()` |
| asset | GET | `get_asset_deposit_history()` |
| asset | GET | `get_asset_withdrawal_history()` |
| asset | GET | `get_asset_currencies()` |
| asset | GET | `get_asset_non_tradable_assets()` |
| asset | GET | `get_asset_asset_valuation()` |
| asset | GET | `get_asset_bills()` |
| asset | GET | `get_asset_transfer_state()` |
| asset | GET | `get_asset_convert_history()` |
| asset | POST | `post_asset_transfer()` |
| asset | POST | `post_asset_withdrawal()` |
| asset | POST | `post_asset_cancel_withdrawal()` |
| asset | POST | `post_asset_convert_estimate_quote()` |
| asset | POST | `post_asset_convert_trade()` |
| trade | GET | `get_trade_order()` |
| trade | GET | `get_trade_orders_history()` |
| trade | GET | `get_trade_orders_algo_pending()` |
| trade | GET | `get_trade_easy_convert_history()` |
| trade | GET | `get_trade_one_click_repay_history_v2()` |
| trade | GET | `get_trade_account_rate_limit()` |
| trade | GET | `get_trade_fills()` |
| trade | POST | `post_trade_order()` |
| trade | POST | `post_trade_cancel_order()` |
| trade | POST | `post_trade_batch_orders()` |
| trade | POST | `post_trade_cancel_batch_orders()` |
| trade | POST | `post_trade_amend_order()` |
| trade | POST | `post_trade_amend_batch_orders()` |
| trade | POST | `post_trade_order_precheck()` |
| trade | POST | `post_trade_cancel_all_after()` |
| trade | POST | `post_trade_cancel_algos()` |
| trade | POST | `post_trade_amend_algos()` |
| trade | POST | `post_trade_mass_cancel()` |
| copy trading | GET | `get_copy_trade_my_leaders()` |
| copy trading | GET | `get_copy_trade_settings()` |
| copy trading | GET | `get_copy_trade_current_subpos()` |
| copy trading | GET | `get_copy_trade_historical_subpos()` |
| copy trading | GET | `get_copy_trade_instruments()` |
| copy trading | GET | `get_copy_trade_public_config()` |
| copy trading | GET | `get_copy_trade_public_lead_traders()` |
| copy trading | GET | `get_copy_trade_public_copy_traders()` |
| copy trading | GET | `get_copy_trade_public_preference_currency()` |
| copy trading | GET | `get_copy_trade_public_current_subpositions()` |
| copy trading | GET | `get_copy_trade_public_subpositions_history()` |
| copy trading | GET | `get_copy_trade_public_pnl()` |
| copy trading | GET | `get_copy_trade_public_stats()` |
| copy trading | GET | `get_copy_trade_public_weekly_pnl()` |
| copy trading | GET | `get_copy_trade_profit_sharing_details()` |
| copy trading | GET | `get_copy_trade_total_profit_sharing()` |

## Release notes

See [NEWS.md](NEWS.md) for release history.

## Development status

* [x] GET support for major endpoints
* [x] POST support for order, cancel, leverage, close position
* [x] Copy trading wrappers
* [x] Package metadata and generated documentation aligned for GitHub release
* [x] Automated test suite foundation
* [x] GitHub Actions package check workflow
* [x] CRAN-safe package examples and package-level help
* [x] Unsigned public endpoints and request timeout handling
* [x] CRAN policy hygiene files and release metadata cleanup
* [x] Mocked request/parser robustness tests
* [x] CRAN release-candidate documentation alignment
* [x] Manual CRAN preflight workflow for release checks
* [x] CRAN submission checklist for final manual steps
* [x] Final CRAN submission metadata update
* [x] CRAN reviewer-requested DESCRIPTION and Rd fixes
* [x] First CRAN release published
* [x] First post-CRAN endpoint expansion batch
* [x] Expanded public reference and index market coverage
* [x] Broader account inspection and sub-account GET coverage for v0.3.2
* [x] Broader trade history, algo-order, asset billing, and convert GET coverage for v0.3.3
* [x] Broader copy-trading coverage plus expanded asset history filter support for v0.3.4
* [x] Remaining copy-trading GET endpoints and copy-trading parameter completeness fixes for v0.3.5
* [x] First trade action expansion batch for v0.4.0
* [x] Advanced trade algo and mass-cancel action coverage for v0.4.1
* [x] Account configuration action coverage for v0.4.2
* [x] Account operational action coverage for v0.4.3
* [x] Asset transfer, withdrawal, and convert action coverage for v0.4.4
* [x] POST wrapper validation and cancellation parameter completeness for v0.4.5
* [ ] Websocket support

## License

MIT

## Author

Oliver Zhou
Lily Li

# okxr: R Interface to the OKX API

`okxr` is an R package for working with the OKX REST API from R. It provides
typed wrappers for market data, account endpoints, asset history, trading, and
copy trading, with shared request signing and schema-based response parsing.

## Status

`okxr` is available on CRAN. Install the stable release from CRAN or use the
GitHub repository for development versions between CRAN releases.

Current development release: `v0.4.7`

## Installation

Stable CRAN release:

```r
install.packages("okxr")
```

Development release from GitHub:

```r
# install.packages("devtools")
devtools::install_github("OliverLDS/okxr")
```

## Features

* Signed OKX REST requests with HMAC authentication
* Typed parsing into tabular R objects with variable labels
* Shared internal GET/POST generators backed by endpoint specs
* User-facing wrappers for account, asset, market, trade, and copy-trading APIs
* Configurable raw vs parsed return mode via `set_okxr_options()`

## Setup

```r
config <- list(
  api_key = "your_api_key",
  secret_key = "your_secret_key",
  passphrase = "your_passphrase"
)
```

Public market and public reference endpoints can be called without credentials.
Private account, asset, trade, and copy-trading endpoints require signed OKX
API credentials.

Do not store real credentials in committed R scripts, examples, or tests. For
interactive use, load them from environment variables or another local secret
store. If you use OKX simulated trading, set `demo = TRUE` in `config`.

```r
config <- list(
  api_key = Sys.getenv("OKX_API_KEY"),
  secret_key = Sys.getenv("OKX_SECRET_KEY"),
  passphrase = Sys.getenv("OKX_PASSPHRASE"),
  demo = TRUE,
  timeout = 10
)
```

## Return values

By default, wrappers return parsed `data.table` objects with typed columns and
variable labels where schemas are defined. Use `set_okxr_options(raw_data = TRUE)`
to return the raw OKX `data` payload instead.

Network failures, request timeouts, OKX error responses, or empty API `data`
payloads may return `NULL` with a warning. Request timeout defaults to 10
seconds and can be set globally with `set_okxr_options(timeout = 15)` or per
request with `config$timeout`.

## Examples

Public market examples require network access but no OKX credentials. Private
account, trading, asset, and copy-trading examples require valid OKX credentials;
trading examples may have account side effects.

### Market data

```r
get_market_candles(
  inst_id = "BTC-USDT",
  bar = "1m",
  limit = 100
)
```

### Account data

```r
get_account_balance(config = config)
get_account_positions(config = config)
```

### Place an order

```r
post_trade_order(
  inst_id = "BTC-USDT",
  td_mode = "cross",
  side = "buy",
  ord_type = "market",
  sz = "0.01",
  config = config
)
```

### Copy trading

```r
get_copy_trade_my_leaders(config = config)
get_copy_trade_current_subpos(config = config)
```

## Wrapper categories

| Category | Method | Example function |
| --- | --- | --- |
| market | GET | `get_market_candles()` |
| market | GET | `get_market_tickers()` |
| market | GET | `get_market_books_rpi()` |
| market | GET | `get_market_trades()` |
| market | GET | `get_market_mark_price_candles()` |
| market | GET | `get_market_index_tickers()` |
| market | GET | `get_market_index_candles()` |
| market | GET | `get_market_index_components()` |
| public | GET | `get_public_time()` |
| public | GET | `get_public_event_contract_series()` |
| public | GET | `get_public_event_contract_events()` |
| public | GET | `get_public_event_contract_markets()` |
| public | GET | `get_public_mm_instrument_types()` |
| public | GET | `get_public_underlying()` |
| public | GET | `get_public_estimated_price()` |
| public | GET | `get_public_estimated_settlement_info()` |
| public | GET | `get_public_opt_summary()` |
| public | GET | `get_public_position_tiers()` |
| public | GET | `get_public_discount_rate_interest_free_quota()` |
| public | GET | `get_public_interest_rate_loan_quota()` |
| public | GET | `get_public_insurance_fund()` |
| public | GET | `get_public_block_trades()` |
| public | GET | `get_public_convert_contract_coin()` |
| public | GET | `get_public_instrument_tick_bands()` |
| public | GET | `get_public_premium_history()` |
| public | GET | `get_public_option_trades()` |
| account | GET | `get_account_balance()` |
| account | GET | `get_account_instruments()` |
| account | GET | `get_account_bill_types()` |
| account | GET | `get_account_subtypes()` |
| account | GET | `get_account_adjust_leverage_info()` |
| account | GET | `get_account_position_risk()` |
| account | GET | `get_account_max_loan()` |
| account | GET | `get_account_max_size()` |
| account | GET | `get_account_max_avail_size()` |
| account | GET | `get_account_trade_fee()` |
| account | GET | `get_account_interest_rate()` |
| account | GET | `get_account_interest_accrued()` |
| account | GET | `get_account_max_withdrawal()` |
| account | GET | `get_account_risk_state()` |
| account | GET | `get_account_interest_limits()` |
| account | GET | `get_account_greeks()` |
| account | GET | `get_account_position_tiers()` |
| account | GET | `get_account_collateral_assets()` |
| account | GET | `get_account_mmp_config()` |
| account | GET | `get_account_move_positions_history()` |
| account | GET | `get_account_precheck_set_delta_neutral()` |
| account | GET | `get_account_bills_history_archive()` |
| account | GET | `get_account_subaccount_balances()` |
| account | GET | `get_account_subaccount_max_withdrawal()` |
| account | GET | `get_account_set_account_switch_precheck()` |
| account | GET | `get_account_spot_borrow_repay_history()` |
| account | GET | `get_account_bills()` |
| account | POST | `post_account_set_leverage()` |
| account | POST | `post_account_set_position_mode()` |
| account | POST | `post_account_set_fee_type()` |
| account | POST | `post_account_set_greeks()` |
| account | POST | `post_account_set_auto_repay()` |
| account | POST | `post_account_set_auto_loan()` |
| account | POST | `post_account_set_account_level()` |
| account | POST | `post_account_set_collateral_assets()` |
| account | POST | `post_account_position_margin_balance()` |
| account | POST | `post_account_spot_manual_borrow_repay()` |
| account | POST | `post_account_account_level_switch_preset()` |
| account | POST | `post_account_mmp_reset()` |
| account | POST | `post_account_mmp_config()` |
| account | POST | `post_account_move_positions()` |
| asset | GET | `get_asset_balances()` |
| asset | GET | `get_asset_deposit_history()` |
| asset | GET | `get_asset_withdrawal_history()` |
| asset | GET | `get_asset_currencies()` |
| asset | GET | `get_asset_non_tradable_assets()` |
| asset | GET | `get_asset_asset_valuation()` |
| asset | GET | `get_asset_bills()` |
| asset | GET | `get_asset_transfer_state()` |
| asset | GET | `get_asset_convert_history()` |
| asset | POST | `post_asset_transfer()` |
| asset | POST | `post_asset_withdrawal()` |
| asset | POST | `post_asset_cancel_withdrawal()` |
| asset | POST | `post_asset_convert_estimate_quote()` |
| asset | POST | `post_asset_convert_trade()` |
| trade | GET | `get_trade_order()` |
| trade | GET | `get_trade_orders_history()` |
| trade | GET | `get_trade_orders_algo_pending()` |
| trade | GET | `get_trade_easy_convert_history()` |
| trade | GET | `get_trade_one_click_repay_history_v2()` |
| trade | GET | `get_trade_account_rate_limit()` |
| trade | GET | `get_trade_fills()` |
| trade | POST | `post_trade_order()` |
| trade | POST | `post_trade_cancel_order()` |
| trade | POST | `post_trade_batch_orders()` |
| trade | POST | `post_trade_cancel_batch_orders()` |
| trade | POST | `post_trade_amend_order()` |
| trade | POST | `post_trade_amend_batch_orders()` |
| trade | POST | `post_trade_order_precheck()` |
| trade | POST | `post_trade_order_algo()` |
| trade | POST | `post_trade_cancel_all_after()` |
| trade | POST | `post_trade_cancel_algos()` |
| trade | POST | `post_trade_amend_algos()` |
| trade | POST | `post_trade_mass_cancel()` |
| copy trading | GET | `get_copy_trade_my_leaders()` |
| copy trading | GET | `get_copy_trade_settings()` |
| copy trading | GET | `get_copy_trade_current_subpos()` |
| copy trading | GET | `get_copy_trade_historical_subpos()` |
| copy trading | GET | `get_copy_trade_instruments()` |
| copy trading | GET | `get_copy_trade_public_config()` |
| copy trading | GET | `get_copy_trade_public_lead_traders()` |
| copy trading | GET | `get_copy_trade_public_copy_traders()` |
| copy trading | GET | `get_copy_trade_public_preference_currency()` |
| copy trading | GET | `get_copy_trade_public_current_subpositions()` |
| copy trading | GET | `get_copy_trade_public_subpositions_history()` |
| copy trading | GET | `get_copy_trade_public_pnl()` |
| copy trading | GET | `get_copy_trade_public_stats()` |
| copy trading | GET | `get_copy_trade_public_weekly_pnl()` |
| copy trading | GET | `get_copy_trade_profit_sharing_details()` |
| copy trading | GET | `get_copy_trade_total_profit_sharing()` |

## Release notes

See [NEWS.md](NEWS.md) for release history.

## Development status

* [x] GET support for major endpoints
* [x] POST support for order, cancel, leverage, close position
* [x] Copy trading wrappers
* [x] Package metadata and generated documentation aligned for GitHub release
* [x] Automated test suite foundation
* [x] GitHub Actions package check workflow
* [x] CRAN-safe package examples and package-level help
* [x] Unsigned public endpoints and request timeout handling
* [x] CRAN policy hygiene files and release metadata cleanup
* [x] Mocked request/parser robustness tests
* [x] CRAN release-candidate documentation alignment
* [x] Manual CRAN preflight workflow for release checks
* [x] CRAN submission checklist for final manual steps
* [x] Final CRAN submission metadata update
* [x] CRAN reviewer-requested DESCRIPTION and Rd fixes
* [x] First CRAN release published
* [x] First post-CRAN endpoint expansion batch
* [x] Expanded public reference and index market coverage
* [x] Broader account inspection and sub-account GET coverage for v0.3.2
* [x] Broader trade history, algo-order, asset billing, and convert GET coverage for v0.3.3
* [x] Broader copy-trading coverage plus expanded asset history filter support for v0.3.4
* [x] Remaining copy-trading GET endpoints and copy-trading parameter completeness fixes for v0.3.5
* [x] First trade action expansion batch for v0.4.0
* [x] Advanced trade algo and mass-cancel action coverage for v0.4.1
* [x] Account configuration action coverage for v0.4.2
* [x] Account operational action coverage for v0.4.3
* [x] Asset transfer, withdrawal, and convert action coverage for v0.4.4
* [x] POST wrapper validation and cancellation parameter completeness for v0.4.5
* [x] OKX compatibility patch for base URL, RPI migration, chase algo orders, asset bill filters, and parser fields for v0.4.6
* [x] 2026 compatibility-completion patch for event contracts, MM instrument types, instrument metadata, ELP, slippage, bill types, and parser fields for v0.4.7
* [ ] Websocket support

## License

MIT

## Author

Oliver Zhou
Lily Li

#' Retrieve variable labels from OKX data frames
#'
#' Returns human-readable labels attached to a data frame produced by OKX API parsers.
#' You can retrieve all labels or a label for a specific variable by name or index.
#'
#' @param df A \code{data.frame} with \code{"var_labels"} attribute (as returned by OKX API parsers).
#' @param var Optional variable name (character) or index (numeric). If \code{NULL}, all labels are returned.
#' @param default Value to return if the variable has no label (default: \code{NA_character_}).
#'
#' @return A character label if \code{var} is provided, or a named character vector of all labels if \code{var = NULL}.
#'
#' @examples
#' df <- data.frame(ordId = "123", px = 10)
#' attr(df, "var_labels") <- c(ordId = "Order ID", px = "Price")
#'
#' get_var_label(df, "ordId")
#' get_var_label(df, 2)
#' get_var_label(df)
#'
#' @export
get_var_label <- function(df, var = NULL, default = NA_character_) {
  labels <- attr(df, "var_labels")
  if (is.null(labels)) return(default)

  if (is.null(var)) {
    return(labels)  # Return all labels
  }

  var_name <- if (is.character(var)) var else names(df)[var]
  if (!var_name %in% names(labels)) return(default)

  labels[[var_name]]
}

#' okxr: R Interface to the OKX REST API
#'
#' `okxr` provides lightweight wrappers for selected OKX REST API endpoints,
#' including market data, account information, asset metadata, order-book trade
#' queries, trading actions, and copy-trading endpoints.
#'
#' @details
#' Public market and public reference endpoints can be called without
#' credentials. Private account, asset, trade, and copy-trading endpoints require
#' an OKX API credential list with `api_key`, `secret_key`, and `passphrase`
#' entries.
#'
#' If `config$demo` is `TRUE`, signed requests include OKX's simulated trading
#' header. Request timeout defaults to 10 seconds and can be set globally with
#' `set_okxr_options(timeout = 15)` or per request with `config$timeout`.
#'
#' By default, wrappers return parsed `data.table` objects. Use
#' `set_okxr_options(raw_data = TRUE)` to return raw API `data` payloads instead.
#'
#' Network failures, request timeouts, OKX error responses, or empty API `data`
#' payloads may return `NULL` with a warning. Live API examples are intentionally
#' non-running because they require credentials, network access, and may have
#' account-specific side effects.
#'
#' @seealso
#' [set_okxr_options()], [get_market_candles()], [post_trade_order()]
#'
#' @keywords package
"_PACKAGE"

#' Create OKX API request headers
#'
#' Generate the required signed headers for an OKX REST request.
#'
#' @param config List with `api_key`, `secret_key`, and `passphrase`.
#' @param httr_method HTTP method as a string (e.g., "GET" or "POST").
#' @param httr_path Path portion of the API endpoint.
#' @param body_json Optional JSON string of the request body.
#'
#' @return A `httr::add_headers` object containing the signed headers.
.get_headers <- function(config, httr_method, httr_path, body_json = "") {
  .okx_validate_config(config)

  timestamp <- format(Sys.time(), "%Y-%m-%dT%H:%M:%S.000Z", tz = "UTC")
  prehash <- paste0(timestamp, toupper(httr_method), httr_path, body_json)
  hmac_sha256 <- digest::hmac(
    key = config$secret_key,
    object = charToRaw(prehash),
    algo = "sha256",
    raw = TRUE
  )
  signature <- base64enc::base64encode(hmac_sha256)

  if (identical(config$demo, TRUE)) {
    httr::add_headers(
      "OK-ACCESS-KEY" = config$api_key,
      "OK-ACCESS-SIGN" = signature,
      "OK-ACCESS-TIMESTAMP" = timestamp,
      "OK-ACCESS-PASSPHRASE" = config$passphrase,
      "Content-Type" = "application/json",
      "x-simulated-trading" = "1"
    )
  } else {
    httr::add_headers(
      "OK-ACCESS-KEY" = config$api_key,
      "OK-ACCESS-SIGN" = signature,
      "OK-ACCESS-TIMESTAMP" = timestamp,
      "OK-ACCESS-PASSPHRASE" = config$passphrase,
      "Content-Type" = "application/json"
    )
  }
}

#' Build a full OKX request object
#'
#' Assemble URL, headers, and body for an OKX API call.
#'
#' @param httr_method HTTP method ("GET" or "POST").
#' @param base_url Base URL of the OKX API.
#' @param api_path API path (e.g., "/api/v5/account/balance").
#' @param query_string Query string starting with "?" or empty.
#' @param config List with API credentials.
#' @param body_json Optional JSON string for POST body.
#' @param auth Logical. Whether to sign the request with OKX credentials.
#'
#' @return A list with elements: `method`, `url`, `full_path`, `headers`, and `body_json`.
.build_request <- function(httr_method, base_url, api_path, query_string, config = NULL, body_json = "", auth = TRUE) {
  full_path <- paste0(api_path, query_string)
  full_url  <- paste0(base_url, full_path)
  headers   <- if (auth) .get_headers(config, httr_method, full_path, body_json) else NULL

  list(
    method     = httr_method,
    url        = full_url,
    full_path  = full_path,
    headers    = headers,
    body_json  = body_json
  )
}

#' Execute a GET request to OKX
#'
#' Perform a signed GET call to the OKX API.
#'
#' @param api_path API path (e.g., "/api/v5/account/balance").
#' @param query_string Query string starting with "?" or empty.
#' @param config List with API credentials.
#' @param auth Logical. Whether to sign the request with OKX credentials.
#'
#' @return An `httr` response object, or `NULL` if the request fails.
.execute_get_action <- function(api_path, query_string, config = NULL, auth = TRUE) {
  base_url <- .okx_base_url
  httr_method <- "GET"
  req <- .build_request(httr_method, base_url, api_path, query_string, config, auth = auth)
  timeout <- .okx_request_timeout(config)
  res <- tryCatch(
    {
      if (is.null(req$headers)) {
        httr::GET(req$url, timeout)
      } else {
        httr::GET(req$url, req$headers, timeout)
      }
    },
    error = function(err) {
      warning("Request failed: ", conditionMessage(err), call. = FALSE)
      NULL
    }
  )
  if (is.null(res)) {
    return(NULL)
  }
  if (httr::http_error(res)) {
    warning("Request failed: ", httr::status_code(res), call. = FALSE)
    return(NULL)
  }
  res
}

#' Execute a POST request to OKX
#'
#' Perform a signed POST call to the OKX API.
#'
#' @param api_path API path (e.g., "/api/v5/trade/order").
#' @param body_list List to be converted to JSON for the request body.
#' @param config List with API credentials.
#'
#' @return An `httr` response object, or `NULL` if the request fails.
.execute_post_action <- function(api_path, body_list, config) {
  base_url     <- .okx_base_url
  httr_method  <- "POST"

  body_json <- jsonlite::toJSON(body_list, auto_unbox = TRUE, pretty = FALSE)

  req <- .build_request(
    httr_method   = httr_method,
    base_url      = base_url,
    api_path      = api_path,
    query_string  = "",
    config        = config,
    body_json     = body_json,
    auth          = TRUE
  )

  timeout <- .okx_request_timeout(config)
  res <- tryCatch(
    httr::POST(req$url, req$headers, timeout, body = req$body_json, encode = "raw"),
    error = function(err) {
      warning("Request failed: ", conditionMessage(err), call. = FALSE)
      NULL
    }
  )

  if (is.null(res)) {
    return(NULL)
  }
  if (httr::http_error(res)) {
    warning("Request failed: ", httr::status_code(res), call. = FALSE)
    return(NULL)
  }
  res
}

#' Create a GET request function from an API spec
#'
#' Converts a single entry from \code{.api_GET_specs} into a ready-to-use function
#' that executes the HTTP GET request, parses the response, and returns a typed data frame.
#'
#' @param api A list containing API spec fields: \code{okx_path}, \code{query} (string or function),
#'   \code{schema} (data.frame), and optional \code{mode} ("named" or "positional").
#'
#' @return A function with signature \code{(tz = "Asia/Hong_Kong", config, ...)} that:
#' \itemize{
#'   \item builds the query string using \code{api$query} (if a function),
#'   \item signs and sends the GET request,
#'   \item parses and returns the data using the schema-aware parser.
#' }
#' @keywords internal
.make_get_function <- function(api) {
  parser <- .make_parser(api$parser_schema, mode = api$parser_mode %||% "named")
  auth <- api$auth %||% TRUE

  function(query_string, tz, config, raw_data = getOption("okxr.raw_data", FALSE)) {
    res <- .execute_get_action(api$okx_path, query_string, config, auth = auth)
    if (is.null(res)) {
      return(NULL)
    }

    parsed_res <- parser(res = res, tz = tz)
    .okx_extract_result(parsed_res, raw_data = raw_data)
  }
}

#' @title GET request function list
#' @description A list of endpoint functions automatically generated from \code{.api_GET_specs}.
#' Each function supports \code{tz}, \code{config}, and additional endpoint-specific arguments.
#' @keywords internal
.gets <- lapply(.api_GET_specs, .make_get_function)

#---- Asset: GET Wrappers ----

#' Get asset balances
#'
#' Retrieves the available, total, and frozen balance for each asset in the account.
#'
#' @param config API credentials as a list with \code{api_key}, \code{secret_key}, and \code{passphrase}.
#' @param tz Timezone string (default: \code{"Asia/Hong_Kong"}).
#'
#' @return A \code{data.frame} with balances per currency.
#'
#' @export
get_asset_balances <- function(config, tz = .okx_default_tz) {
  .gets$asset_balances(query_string = "", tz = tz, config = config)
}

#' Get asset deposit history
#'
#' Retrieves a record of all asset deposits made to your account.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param dep_id Character or `NULL`. Deposit ID filter.
#' @param from_wd_id Character or `NULL`. Source withdrawal ID filter.
#' @param tx_id Character or `NULL`. Transaction hash filter.
#' @param type Character or `NULL`. Deposit type filter.
#' @param state Character or `NULL`. Deposit state filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key}, and \code{passphrase}.
#' @param tz Timezone string (default: \code{"Asia/Hong_Kong"}).
#'
#' @return A \code{data.frame} with deposit history rows and detailed transfer metadata.
#'
#' @export
get_asset_deposit_history <- function(ccy = NULL, dep_id = NULL, from_wd_id = NULL, tx_id = NULL, type = NULL, state = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ccy = ccy,
    depId = dep_id,
    fromWdId = from_wd_id,
    txId = tx_id,
    type = type,
    state = state,
    after = after,
    before = before,
    limit = limit
  )
  .gets$asset_deposit_history(query_string = query_string, tz = tz, config = config)
}

#' Get asset withdrawal history
#'
#' Retrieves a record of all asset withdrawals from your account.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param wd_id Character or `NULL`. Withdrawal ID filter.
#' @param client_id Character or `NULL`. Client withdrawal ID filter.
#' @param tx_id Character or `NULL`. Transaction hash filter.
#' @param type Character or `NULL`. Withdrawal type filter.
#' @param state Character or `NULL`. Withdrawal state filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key}, and \code{passphrase}.
#' @param tz Timezone string (default: \code{"Asia/Hong_Kong"}).
#'
#' @return A \code{data.frame} with withdrawal history rows and detailed transfer metadata.
#'
#' @export
get_asset_withdrawal_history <- function(ccy = NULL, wd_id = NULL, client_id = NULL, tx_id = NULL, type = NULL, state = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ccy = ccy,
    wdId = wd_id,
    clientId = client_id,
    txId = tx_id,
    type = type,
    state = state,
    after = after,
    before = before,
    limit = limit
  )
  .gets$asset_withdrawal_history(query_string = query_string, tz = tz, config = config)
}

#' Get funding currencies
#'
#' Retrieve currencies available to the current account.
#'
#' @param ccy Character or `NULL`. Single currency or comma-separated currencies.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string (default: \code{"Asia/Hong_Kong"}).
#'
#' @return A \code{data.frame} with currency and chain metadata.
#'
#' @export
get_asset_currencies <- function(ccy = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(ccy = ccy)
  .gets$asset_currencies(query_string = query_string, tz = tz, config = config)
}

#' Get deposit address
#'
#' Retrieve deposit addresses for a currency.
#'
#' @param ccy Character. Currency, e.g. \code{"BTC"} or \code{"USDT"}.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string (default: \code{"Asia/Hong_Kong"}).
#'
#' @return A \code{data.frame} with deposit address rows.
#'
#' @export
get_asset_deposit_address <- function(ccy, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(ccy = ccy)
  .gets$asset_deposit_address(query_string = query_string, tz = tz, config = config)
}

#' Get non-tradable assets
#'
#' Retrieve balances and withdrawal metadata for non-tradable assets.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with non-tradable asset rows.
#' @export
get_asset_non_tradable_assets <- function(
  ccy = NULL,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(ccy = ccy)
  .gets$asset_non_tradable_assets(query_string = query_string, tz = tz, config = config)
}

#' Get asset valuation
#'
#' Retrieve total account valuation for funding assets.
#'
#' @param ccy Character or `NULL`. Valuation currency filter.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with valuation summary rows.
#' @export
get_asset_asset_valuation <- function(
  ccy = NULL,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(ccy = ccy)
  .gets$asset_asset_valuation(query_string = query_string, tz = tz, config = config)
}

#' Get asset transfer state
#'
#' Retrieve the state of a funding transfer.
#'
#' @param trans_id Character or `NULL`. Transfer ID.
#' @param client_id Character or `NULL`. Client-supplied transfer ID.
#' @param type Character or `NULL`. Transfer type.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with transfer state rows.
#' @export
get_asset_transfer_state <- function(trans_id = NULL, client_id = NULL, type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(transId = trans_id, clientId = client_id, type = type)
  .gets$asset_transfer_state(query_string = query_string, tz = tz, config = config)
}

#' Get asset bills
#'
#' Retrieve recent funding account bills.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param type Character or `NULL`. Bill type filter.
#' @param client_id Character or `NULL`. Client-supplied transfer or withdrawal ID.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with asset bill rows.
#' @export
get_asset_bills <- function(ccy = NULL, type = NULL, client_id = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ccy = ccy,
    type = type,
    clientId = client_id,
    after = after,
    before = before,
    limit = limit
  )
  .gets$asset_bills(query_string = query_string, tz = tz, config = config)
}

#' Get asset bills history
#'
#' Retrieve historical funding account bills.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param type Character or `NULL`. Bill type filter.
#' @param client_id Character or `NULL`. Client-supplied transfer or withdrawal ID.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param paging_type Character or `NULL`. Paging type selector.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with historical asset bill rows.
#' @export
get_asset_bills_history <- function(ccy = NULL, type = NULL, client_id = NULL, after = NULL, before = NULL, limit = NULL, paging_type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ccy = ccy,
    type = type,
    clientId = client_id,
    after = after,
    before = before,
    limit = limit,
    pagingType = paging_type
  )
  .gets$asset_bills_history(query_string = query_string, tz = tz, config = config)
}

#' Get deposit or withdrawal status
#'
#' Retrieve detailed status for a deposit or withdrawal.
#'
#' @param wd_id Character or `NULL`. Withdrawal ID.
#' @param tx_id Character or `NULL`. Deposit transaction hash.
#' @param ccy Character or `NULL`. Currency filter used with \code{tx_id}.
#' @param to Character or `NULL`. Destination address used with \code{tx_id}.
#' @param chain Character or `NULL`. Chain identifier used with \code{tx_id}.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with detailed deposit/withdraw status rows.
#' @export
get_asset_deposit_withdraw_status <- function(wd_id = NULL, tx_id = NULL, ccy = NULL, to = NULL, chain = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    wdId = wd_id,
    txId = tx_id,
    ccy = ccy,
    to = to,
    chain = chain
  )
  .gets$asset_deposit_withdraw_status(query_string = query_string, tz = tz, config = config)
}

#' Get exchange list
#'
#' Retrieve the public exchange list used by asset withdrawal metadata.
#'
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with exchange identifiers and names.
#' @export
get_asset_exchange_list <- function(tz = .okx_default_tz) {
  .gets$asset_exchange_list(query_string = "", tz = tz, config = NULL)
}

#' Get convert currencies
#'
#' Retrieve currencies supported by the asset convert API.
#'
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with supported convert currencies.
#' @export
get_asset_convert_currencies <- function(config, tz = .okx_default_tz) {
  .gets$asset_convert_currencies(query_string = "", tz = tz, config = config)
}

#' Get convert currency pair
#'
#' Retrieve convert metadata for a currency pair.
#'
#' @param from_ccy Character. Currency to convert from.
#' @param to_ccy Character. Currency to convert to.
#' @param convert_mode Character or `NULL`. Convert mode selector.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with currency-pair convert metadata.
#' @export
get_asset_convert_currency_pair <- function(from_ccy, to_ccy, convert_mode = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(fromCcy = from_ccy, toCcy = to_ccy, convertMode = convert_mode)
  .gets$asset_convert_currency_pair(query_string = query_string, tz = tz, config = config)
}

#' Get convert history
#'
#' Retrieve historical asset convert trades.
#'
#' @param cl_t_req_id Character or `NULL`. Client trade request ID.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param tag Character or `NULL`. Broker tag filter.
#' @param config API credentials as a list with \code{api_key}, \code{secret_key},
#'   and \code{passphrase}.
#' @param tz Timezone string.
#'
#' @return A \code{data.frame} with convert history rows.
#' @export
get_asset_convert_history <- function(cl_t_req_id = NULL, after = NULL, before = NULL, limit = NULL, tag = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    clTReqId = cl_t_req_id,
    after = after,
    before = before,
    limit = limit,
    tag = tag
  )
  .gets$asset_convert_history(query_string = query_string, tz = tz, config = config)
}

#---- Asset: POST Wrappers ----

.okx_asset_compact_body <- function(body_list) {
  body_list[!vapply(body_list, is.null, logical(1))]
}

#' Transfer Assets
#'
#' Transfer assets between funding, trading, and related accounts.
#'
#' @param ccy Currency to transfer.
#' @param amt Transfer amount.
#' @param from Source account code, such as `"6"` for funding or `"18"` for
#'   trading.
#' @param to Destination account code.
#' @param type Optional transfer type code. Defaults to `"0"` for an internal
#'   transfer within the same account.
#' @param sub_acct Optional sub-account name when the transfer type requires it.
#' @param loan_trans Optional logical. Whether the transfer should be treated
#'   as a loan transfer.
#' @param omit_pos_risk Optional logical. Whether to omit position risk checks
#'   where supported by OKX.
#' @param client_id Optional client-supplied transfer request ID.
#' @param tz Timezone used for any timestamp parsing.
#' @param config API credential list.
#'
#' @return A `data.frame` describing the submitted transfer request.
#' @export
post_asset_transfer <- function(ccy, amt, from, to, type = "0", sub_acct = NULL, loan_trans = NULL, omit_pos_risk = NULL, client_id = NULL, tz = .okx_default_tz, config) {
  if (identical(from, to)) {
    stop("`from` and `to` must be different account codes.", call. = FALSE)
  }
  body <- .okx_asset_compact_body(list(
    type = type,
    ccy = ccy,
    amt = amt,
    from = from,
    to = to,
    subAcct = sub_acct,
    loanTrans = if (is.null(loan_trans)) NULL else isTRUE(loan_trans),
    omitPosRisk = if (is.null(omit_pos_risk)) NULL else isTRUE(omit_pos_risk),
    clientId = client_id
  ))
  .posts$asset_transfer(body_list = body, tz = tz, config = config)
}

#' Submit an Asset Withdrawal
#'
#' Submit a withdrawal request from the OKX funding account.
#'
#' @param ccy Currency to withdraw.
#' @param amt Withdrawal amount.
#' @param dest Destination type code from the OKX API.
#' @param to_addr Destination wallet address.
#' @param chain Optional chain identifier, such as `"USDT-ERC20"`.
#' @param to_addr_type Optional destination address type code.
#' @param area_code Optional phone area code when required by OKX.
#' @param rcvr_info Optional named list in the documented `rcvrInfo` shape.
#' @param client_id Optional client-supplied withdrawal request ID.
#' @param tz Timezone used for any timestamp parsing.
#' @param config API credential list.
#'
#' @return A `data.frame` describing the submitted withdrawal request.
#' @export
post_asset_withdrawal <- function(ccy, amt, dest, to_addr, chain = NULL, to_addr_type = NULL, area_code = NULL, rcvr_info = NULL, client_id = NULL, tz = .okx_default_tz, config) {
  body <- .okx_asset_compact_body(list(
    ccy = ccy,
    amt = amt,
    dest = dest,
    toAddr = to_addr,
    chain = chain,
    toAddrType = to_addr_type,
    areaCode = area_code,
    rcvrInfo = rcvr_info,
    clientId = client_id
  ))
  .posts$asset_withdrawal(body_list = body, tz = tz, config = config)
}

#' Cancel an Asset Withdrawal
#'
#' Cancel a pending withdrawal request.
#'
#' @param wd_id Withdrawal request ID.
#' @param tz Timezone used for any timestamp parsing.
#' @param config API credential list.
#'
#' @return A `data.frame` confirming the cancelled withdrawal ID.
#' @export
post_asset_cancel_withdrawal <- function(wd_id, tz = .okx_default_tz, config) {
  .posts$asset_cancel_withdrawal(
    body_list = list(wdId = wd_id),
    tz = tz,
    config = config
  )
}

#' Estimate an Asset Convert Quote
#'
#' Request a quote for an asset conversion without executing the trade.
#'
#' @param base_ccy Base currency.
#' @param quote_ccy Quote currency.
#' @param side Quote side, such as `"buy"` or `"sell"`.
#' @param rfq_sz RFQ size.
#' @param rfq_sz_ccy Currency in which `rfq_sz` is specified.
#' @param cl_q_req_id Optional client quote request ID.
#' @param tag Optional request tag.
#' @param convert_mode Optional OKX convert mode.
#' @param tz Timezone used for any timestamp parsing.
#' @param config API credential list.
#'
#' @return A `data.frame` describing the estimated conversion quote.
#' @export
post_asset_convert_estimate_quote <- function(base_ccy, quote_ccy, side, rfq_sz, rfq_sz_ccy, cl_q_req_id = NULL, tag = NULL, convert_mode = NULL, tz = .okx_default_tz, config) {
  body <- .okx_asset_compact_body(list(
    baseCcy = base_ccy,
    quoteCcy = quote_ccy,
    side = side,
    rfqSz = rfq_sz,
    rfqSzCcy = rfq_sz_ccy,
    clQReqId = cl_q_req_id,
    tag = tag,
    convertMode = convert_mode
  ))
  .posts$asset_convert_estimate_quote(body_list = body, tz = tz, config = config)
}

#' Execute an Asset Convert Trade
#'
#' Execute a confirmed asset conversion against a previously quoted price.
#'
#' @param quote_id Quote ID returned by [post_asset_convert_estimate_quote()].
#' @param base_ccy Base currency.
#' @param quote_ccy Quote currency.
#' @param side Trade side, such as `"buy"` or `"sell"`.
#' @param sz Trade size.
#' @param sz_ccy Currency in which `sz` is specified.
#' @param cl_t_req_id Optional client trade request ID.
#' @param tag Optional request tag.
#' @param convert_mode Optional OKX convert mode.
#' @param tz Timezone used for any timestamp parsing.
#' @param config API credential list.
#'
#' @return A `data.frame` describing the executed conversion trade.
#' @export
post_asset_convert_trade <- function(quote_id, base_ccy, quote_ccy, side, sz, sz_ccy, cl_t_req_id = NULL, tag = NULL, convert_mode = NULL, tz = .okx_default_tz, config) {
  body <- .okx_asset_compact_body(list(
    quoteId = quote_id,
    baseCcy = base_ccy,
    quoteCcy = quote_ccy,
    side = side,
    sz = sz,
    szCcy = sz_ccy,
    clTReqId = cl_t_req_id,
    tag = tag,
    convertMode = convert_mode
  ))
  .posts$asset_convert_trade(body_list = body, tz = tz, config = config)
}

mock_okx_response <- function(data, code = "0", msg = "") {
  body <- jsonlite::toJSON(
    list(code = code, msg = msg, data = data),
    auto_unbox = TRUE,
    null = "null"
  )

  structure(
    list(
      url = "https://www.okx.com/mock",
      status_code = 200L,
      headers = list(`content-type` = "application/json"),
      all_headers = list(list(headers = list(`content-type` = "application/json"))),
      cookies = data.frame(),
      content = charToRaw(body),
      date = Sys.time(),
      times = c()
    ),
    class = "response"
  )
}

mock_http_response <- function(status_code = 200L, body = list(code = "0", msg = "", data = list())) {
  structure(
    list(
      url = "https://www.okx.com/mock",
      status_code = status_code,
      headers = list(`content-type` = "application/json"),
      all_headers = list(list(headers = list(`content-type` = "application/json"))),
      cookies = data.frame(),
      content = charToRaw(jsonlite::toJSON(body, auto_unbox = TRUE, null = "null")),
      date = Sys.time(),
      times = c()
    ),
    class = "response"
  )
}

mock_text_response <- function(text, status_code = 200L) {
  structure(
    list(
      url = "https://www.okx.com/mock",
      status_code = status_code,
      headers = list(`content-type` = "application/json"),
      all_headers = list(list(headers = list(`content-type` = "application/json"))),
      cookies = data.frame(),
      content = charToRaw(text),
      date = Sys.time(),
      times = c()
    ),
    class = "response"
  )
}

test_that(".make_parser parses named responses with types and labels", {
  schema <- data.frame(
    okx = c("ts", "px", "count", "flag", "name"),
    formal = c("Timestamp", "Price", "Count", "Flag", "Name"),
    type = c("time", "numeric", "integer", "logical", "string"),
    stringsAsFactors = FALSE
  )
  parser <- okxr:::.make_parser(schema, mode = "named")
  res <- mock_okx_response(list(list(
    ts = "1000",
    px = "12.5",
    count = "2",
    flag = "true",
    name = "BTC"
  )))

  parsed <- parser(res, tz = "UTC")
  expect_s3_class(parsed$data_dt, "data.table")
  expect_equal(parsed$data_dt$px, 12.5)
  expect_equal(parsed$data_dt$count, 2L)
  expect_true(parsed$data_dt$flag)
  expect_equal(parsed$data_dt$name, "BTC")
  expect_equal(as.numeric(parsed$data_dt$ts), 1)
  expect_equal(attr(parsed$data_dt, "var_labels")[["px"]], "Price")
})

test_that(".make_parser parses positional responses", {
  schema <- data.frame(
    okx = c("ts", "o", "c"),
    formal = c("Timestamp", "Open", "Close"),
    type = c("time", "numeric", "numeric"),
    stringsAsFactors = FALSE
  )
  parser <- okxr:::.make_parser(schema, mode = "positional")
  res <- mock_okx_response(list(list("1000", "1.2", "1.4")))

  parsed <- parser(res, tz = "UTC")
  expect_equal(parsed$data_dt$o, 1.2)
  expect_equal(parsed$data_dt$c, 1.4)
})

test_that(".make_parser handles API failures, empty data, and list-valued fields", {
  schema <- data.frame(
    okx = "asks",
    formal = "Ask levels",
    type = "string",
    stringsAsFactors = FALSE
  )
  parser <- okxr:::.make_parser(schema, mode = "named")

  expect_warning(
    expect_null(parser(mock_okx_response(list(), code = "51000", msg = "bad"), tz = "UTC")),
    "bad"
  )
  expect_null(parser(mock_okx_response(list()), tz = "UTC"))

  parsed <- parser(mock_okx_response(list(list(asks = list(list("1", "2"))))), tz = "UTC")
  expect_match(parsed$data_dt$asks, "\\[")
})

test_that(".make_parser warns and returns NULL for malformed responses", {
  schema <- data.frame(
    okx = "px",
    formal = "Price",
    type = "numeric",
    stringsAsFactors = FALSE
  )
  parser <- okxr:::.make_parser(schema, mode = "named")

  expect_warning(
    expect_null(parser(mock_text_response("{not-json"), tz = "UTC")),
    "Response parsing failed"
  )
})

test_that(".make_parser keeps missing fields as typed missing values", {
  schema <- data.frame(
    okx = c("px", "count", "name"),
    formal = c("Price", "Count", "Name"),
    type = c("numeric", "integer", "string"),
    stringsAsFactors = FALSE
  )
  parser <- okxr:::.make_parser(schema, mode = "named")

  parsed <- parser(mock_okx_response(list(list(px = "12.5"))), tz = "UTC")

  expect_equal(parsed$data_dt$px, 12.5)
  expect_true(is.na(parsed$data_dt$count))
  expect_true(is.na(parsed$data_dt$name))
})

#---- Account: GET Wrappers ----

#' Get account balance
#'
#' Retrieve account-level margin and equity information for your OKX account.
#'
#' @details
#' This wraps `/api/v5/account/balance`. Returns one row per account-level
#' equity snapshot. Timestamps are parsed into `POSIXct` in the given `tz`.
#'
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`. May also include `base_url`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with account balance and margin metrics (e.g.,
#' `totalEq`, `isoEq`, `adjEq`, `availEq`, `ordFroz`, `imr`, `mmr`, `upl`,
#' `mgnRatio`, ...). Timestamp columns (`uTime`) are `POSIXct`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' bal <- get_account_balance(config = cfg)
#' head(bal)
#' }
#'
#' @seealso [get_account_positions()], [get_account_leverage_info()]
#' @note Since okxr 0.1.1
#' @export
get_account_balance <- function(config, tz = .okx_default_tz) {
  .gets$account_balance(query_string = "", tz = tz, config = config)
}

#' Get account open positions
#'
#' Retrieve all currently open positions under the account.
#'
#' @details
#' Wraps `/api/v5/account/positions`. Returns one row per open position.
#'
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with columns such as `instId`, `posId`, `posSide`, `pos`,
#' `lever`, `avgPx`, `markPx`, `upl`, `realizedPnl`, etc. Timestamps (`cTime`,
#' `uTime`) are `POSIXct`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' pos <- get_account_positions(config = cfg)
#' pos
#' }
#'
#' @seealso [get_account_balance()], [get_account_positions_history()]
#' @note Since okxr 0.1.1
#' @export
get_account_positions <- function(config, tz = .okx_default_tz) {
  .gets$account_positions(query_string = "", tz = tz, config = config)
}

#' Get account position history
#'
#' Retrieve historical records of closed or adjusted positions.
#'
#' @details
#' Wraps `/api/v5/account/positions-history`. Includes closed positions and
#' their realized PnL. Returns one row per historical record.
#'
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with columns such as `instId`, `posId`, `posSide`, `pos`,
#' `lever`, `realizedPnl`, `fee`, plus timestamp fields (`cTime`, `uTime`).
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' hist <- get_account_positions_history(config = cfg)
#' tail(hist)
#' }
#'
#' @seealso [get_account_positions()]
#' @note Since okxr 0.1.1
#' @export
get_account_positions_history <- function(config, tz = .okx_default_tz) {
  .gets$account_positions_history(query_string = "", tz = tz, config = config)
}

#' Get account configuration
#'
#' Retrieve account-level configuration information.
#'
#' @details
#' Wraps `/api/v5/account/config`. Includes account ID, mode, and position
#' mode flags. Returns one row.
#'
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with columns like `uid`, `mainUid`, `acctLv`, `posMode`,
#' `autoLoan`, etc.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' cfg_info <- get_account_config(config = cfg)
#' cfg_info
#' }
#'
#' @seealso [get_account_balance()], [get_account_leverage_info()]
#' @note Since okxr 0.1.2
#' @export
get_account_config <- function(config, tz = .okx_default_tz) {
  .gets$account_config(query_string = "", tz = tz, config = config)
}

#' Get account leverage settings
#'
#' Retrieve leverage configuration for a given instrument and margin mode.
#'
#' @details
#' Wraps `/api/v5/account/leverage-info`. Requires both `inst_id` and
#' `mgn_mode`. Returns current leverage values (numeric).
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"`.
#' @param mgn_mode Character. Margin mode. One of `"cross"` or `"isolated"`.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with columns `instId`, `mgnMode`, `posSide`, and `lever`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_account_leverage_info(
#'   inst_id = "BTC-USDT",
#'   mgn_mode = "cross",
#'   config = cfg
#' )
#' }
#'
#' @seealso [get_account_balance()], [get_account_positions()]
#' @note Since okxr 0.1.1
#' @export
get_account_leverage_info <- function(
  inst_id,
  mgn_mode,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instId = inst_id, mgnMode = mgn_mode)
  .gets$account_leverage_info(query_string = query_string, config = config, tz = tz)
}

#' Get account bills
#'
#' Retrieve account bill details from the last 7 days.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param ccy Character or `NULL`. Currency filter.
#' @param mgn_mode Character or `NULL`. Margin mode filter.
#' @param ct_type Character or `NULL`. Contract type filter.
#' @param type Character or `NULL`. Bill type filter.
#' @param sub_type Character or `NULL`. Bill subtype filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with account bill rows.
#'
#' @export
get_account_bills <- function(inst_type = NULL, ccy = NULL, mgn_mode = NULL, ct_type = NULL, type = NULL, sub_type = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    ccy = ccy,
    mgnMode = mgn_mode,
    ctType = ct_type,
    type = type,
    subType = sub_type,
    after = after,
    before = before,
    limit = limit
  )
  .gets$account_bills(query_string = query_string, config = config, tz = tz)
}

#' Get archived account bills
#'
#' Retrieve archived account bill details.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param ccy Character or `NULL`. Currency filter.
#' @param mgn_mode Character or `NULL`. Margin mode filter.
#' @param ct_type Character or `NULL`. Contract type filter.
#' @param type Character or `NULL`. Bill type filter.
#' @param sub_type Character or `NULL`. Bill subtype filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with archived account bill rows.
#'
#' @export
get_account_bills_archive <- function(inst_type = NULL, ccy = NULL, mgn_mode = NULL, ct_type = NULL, type = NULL, sub_type = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    ccy = ccy,
    mgnMode = mgn_mode,
    ctType = ct_type,
    type = type,
    subType = sub_type,
    after = after,
    before = before,
    limit = limit
  )
  .gets$account_bills_archive(query_string = query_string, config = config, tz = tz)
}

#' Get account-available instruments
#'
#' Retrieve available instruments for the current account and instrument type.
#'
#' @details
#' Wraps `/api/v5/account/instruments`. This endpoint is account-scoped and may
#' return a subset of instruments available to the authenticated account.
#'
#' @param inst_type Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
#'   `"SWAP"`, `"FUTURES"`, `"OPTION"`.
#' @param uly Character or `NULL`. Underlying, where applicable.
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param inst_id Character or `NULL`. Specific instrument ID filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with account-available instrument metadata, including
#'   identifiers, currencies, tick size, lot size, leverage, expiry/listing
#'   timestamps, and state where returned by OKX.
#'
#' @export
get_account_instruments <- function(inst_type, uly = NULL, inst_family = NULL, inst_id = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    uly = uly,
    instFamily = inst_family,
    instId = inst_id
  )
  .gets$account_instruments(query_string = query_string, config = config, tz = tz)
}

#' Get account and position risk snapshot
#'
#' Retrieve account-level adjusted equity together with same-snapshot balance
#' and position risk payloads.
#'
#' @param inst_type Character or `NULL`. Instrument type filter. One of
#'   `"MARGIN"`, `"SWAP"`, `"FUTURES"`, or `"OPTION"`.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per OKX risk snapshot. Nested balance and
#'   position payloads are returned as JSON strings in `balData` and `posData`.
#'
#' @export
get_account_position_risk <- function(
  inst_type = NULL,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$account_position_risk(query_string = query_string, config = config, tz = tz)
}

#' Get maximum order size
#'
#' Retrieve the maximum order quantity allowed for one or more instruments under
#' the requested trade mode.
#'
#' @param inst_id Character. One instrument ID or a comma-separated list of up
#'   to five IDs in the same instrument type.
#' @param td_mode Character. Trade mode: `"cross"`, `"isolated"`, `"cash"`, or
#'   `"spot_isolated"`.
#' @param ccy Character or `NULL`. Margin currency where applicable.
#' @param px Character, numeric, or `NULL`. Optional price override.
#' @param leverage Character, numeric, or `NULL`. Optional leverage override.
#' @param trade_quote_ccy Character or `NULL`. Quote currency used for trading
#'   for spot instruments.
#' @param outcome Character or `NULL`. Events market outcome, `"yes"` or `"no"`.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `instId`, `ccy`, `maxBuy`, and `maxSell`.
#'
#' @export
get_account_max_size <- function(inst_id, td_mode, ccy = NULL, px = NULL, leverage = NULL, trade_quote_ccy = NULL, outcome = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instId = inst_id,
    tdMode = td_mode,
    ccy = ccy,
    px = px,
    leverage = leverage,
    tradeQuoteCcy = trade_quote_ccy,
    outcome = outcome
  )
  .gets$account_max_size(query_string = query_string, config = config, tz = tz)
}

#' Get maximum available tradable amount
#'
#' Retrieve the maximum available buy and sell amount for an instrument under
#' the requested trade mode.
#'
#' @param inst_id Character. One instrument ID or a comma-separated list of up
#'   to five IDs.
#' @param td_mode Character. Trade mode: `"cross"`, `"isolated"`, `"cash"`, or
#'   `"spot_isolated"`.
#' @param ccy Character or `NULL`. Margin currency where applicable.
#' @param reduce_only Logical, character, or `NULL`. Whether to reduce position
#'   only. Only applicable to margin endpoints that support it.
#' @param px Character, numeric, or `NULL`. Optional closing price, when
#'   supported by OKX.
#' @param trade_quote_ccy Character or `NULL`. Quote currency used for trading
#'   for spot instruments.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `instId`, `availBuy`, and `availSell`.
#'
#' @export
get_account_max_avail_size <- function(inst_id, td_mode, ccy = NULL, reduce_only = NULL, px = NULL, trade_quote_ccy = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instId = inst_id,
    ccy = ccy,
    tdMode = td_mode,
    reduceOnly = reduce_only,
    px = px,
    tradeQuoteCcy = trade_quote_ccy
  )
  .gets$account_max_avail_size(query_string = query_string, config = config, tz = tz)
}

#' Get account trade fee rates
#'
#' Retrieve the account's trade fee schedule for a specific instrument type and
#' optional instrument, instrument family, or trading fee group.
#'
#' @param inst_type Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
#'   `"SWAP"`, `"FUTURES"`, `"OPTION"`, or `"EVENTS"`.
#' @param inst_id Character or `NULL`. Instrument ID for spot or margin.
#' @param inst_family Character or `NULL`. Instrument family for futures, swaps,
#'   or options.
#' @param group_id Character or `NULL`. Trading fee group ID.
#'   Do not combine with `inst_id` or `inst_family`.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with fee level, fee-rate columns, and any nested
#'   `feeGroup` or deprecated `fiat` details JSON-encoded as strings.
#'
#' @export
get_account_trade_fee <- function(inst_type, inst_id = NULL, inst_family = NULL, group_id = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instId = inst_id,
    instFamily = inst_family,
    groupId = group_id
  )
  .gets$account_trade_fee(query_string = query_string, config = config, tz = tz)
}

#' Get account borrowing interest rates
#'
#' Retrieve the current leveraged currency borrowing market interest rate for
#' one currency or for all eligible currencies.
#'
#' @param ccy Character or `NULL`. Currency filter, e.g. `"BTC"`.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `ccy` and `interestRate`.
#'
#' @export
get_account_interest_rate <- function(
  ccy = NULL,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(ccy = ccy)
  .gets$account_interest_rate(query_string = query_string, config = config, tz = tz)
}

#' Get account bill subtypes
#'
#' Retrieve available account bill types and subtype descriptions.
#'
#' @param type Character or `NULL`. Bill type filter. Multiple values may be
#'   provided as a comma-separated string.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with bill type descriptions and JSON-encoded
#'   `subTypeDetails`.
#'
#' @export
get_account_subtypes <- function(type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(type = type)
  .gets$account_subtypes(query_string = query_string, config = config, tz = tz)
}

#' Get account leverage adjustment estimate
#'
#' Estimate account effects under a target leverage.
#'
#' @param inst_type Character. Instrument type: `"MARGIN"`, `"SWAP"`, or `"FUTURES"`.
#' @param mgn_mode Character. Margin mode: `"cross"` or `"isolated"`.
#' @param lever Character or numeric. Target leverage.
#' @param inst_id Character or `NULL`. Instrument ID.
#' @param ccy Character or `NULL`. Margin currency.
#' @param pos_side Character or `NULL`. Position side.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with estimated leverage-adjustment metrics.
#'
#' @export
get_account_adjust_leverage_info <- function(inst_type, mgn_mode, lever, inst_id = NULL, ccy = NULL, pos_side = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    mgnMode = mgn_mode,
    lever = lever,
    instId = inst_id,
    ccy = ccy,
    posSide = pos_side
  )
  .gets$account_adjust_leverage_info(query_string = query_string, config = config, tz = tz)
}

#' Get account maximum loan
#'
#' Retrieve the maximum loan for manual borrow or margin borrowing scenarios.
#'
#' @param mgn_mode Character. Margin mode: `"cross"` or `"isolated"`.
#' @param inst_id Character or `NULL`. Instrument ID(s).
#' @param ccy Character or `NULL`. Currency.
#' @param mgn_ccy Character or `NULL`. Margin currency.
#' @param trade_quote_ccy Character or `NULL`. Quote currency for trading.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `instId`, `mgnMode`, `mgnCcy`, `maxLoan`, `ccy`,
#'   and `side`.
#'
#' @export
get_account_max_loan <- function(mgn_mode, inst_id = NULL, ccy = NULL, mgn_ccy = NULL, trade_quote_ccy = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    mgnMode = mgn_mode,
    instId = inst_id,
    ccy = ccy,
    mgnCcy = mgn_ccy,
    tradeQuoteCcy = trade_quote_ccy
  )
  .gets$account_max_loan(query_string = query_string, config = config, tz = tz)
}

#' Get account interest accrued history
#'
#' Retrieve accrued borrowing interest records for the past year.
#'
#' @param type Character or `NULL`. Loan type. Currently `"2"` for market loans.
#' @param ccy Character or `NULL`. Loan currency.
#' @param inst_id Character or `NULL`. Instrument ID.
#' @param mgn_mode Character or `NULL`. Margin mode.
#' @param after Character or `NULL`. Pagination cursor for earlier rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per accrued-interest event.
#'
#' @export
get_account_interest_accrued <- function(type = NULL, ccy = NULL, inst_id = NULL, mgn_mode = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    type = type,
    ccy = ccy,
    instId = inst_id,
    mgnMode = mgn_mode,
    after = after,
    before = before,
    limit = limit
  )
  .gets$account_interest_accrued(query_string = query_string, config = config, tz = tz)
}

#' Get account maximum withdrawals
#'
#' Retrieve the maximum transferable amount from trading to funding account.
#'
#' @param ccy Character or `NULL`. One currency or a comma-separated list of up
#'   to 20 currencies.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with per-currency maximum withdrawal values.
#'
#' @export
get_account_max_withdrawal <- function(
  ccy = NULL,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(ccy = ccy)
  .gets$account_max_withdrawal(query_string = query_string, config = config, tz = tz)
}

#' Get account risk state
#'
#' Retrieve portfolio-margin account risk flags and affected risk units.
#'
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with the account risk flag and JSON-encoded
#'   risk-unit arrays.
#'
#' @export
get_account_risk_state <- function(config, tz = .okx_default_tz) {
  .gets$account_risk_state(query_string = "", config = config, tz = tz)
}

#' Get account borrow interest and limits
#'
#' Retrieve account-level debt, next accrual timestamps, and nested per-currency
#' borrowing-limit records.
#'
#' @param type Character or `NULL`. Loan type. Currently `"2"` for market loans.
#' @param ccy Character or `NULL`. Loan currency.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with account-level limit fields; nested per-currency
#'   `records` are JSON-encoded.
#'
#' @export
get_account_interest_limits <- function(type = NULL, ccy = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(type = type, ccy = ccy)
  .gets$account_interest_limits(query_string = query_string, config = config, tz = tz)
}

#' Get account Greeks
#'
#' Retrieve currency-level Greeks across the account.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per currency and Greek metrics in both
#'   Black-Scholes and coin terms.
#'
#' @export
get_account_greeks <- function(ccy = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(ccy = ccy)
  .gets$account_greeks(query_string = query_string, config = config, tz = tz)
}

#' Get account position tiers
#'
#' Retrieve portfolio-margin position limits for one or more instrument
#' families.
#'
#' @param inst_type Character. Instrument type: `"SWAP"`, `"FUTURES"`, or
#'   `"OPTION"`.
#' @param inst_family Character. One instrument family or a comma-separated list
#'   of up to five families.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with instrument-family position limits.
#'
#' @export
get_account_position_tiers <- function(inst_type, inst_family, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type, instFamily = inst_family)
  .gets$account_position_tiers(query_string = query_string, config = config, tz = tz)
}

#' Get account collateral assets
#'
#' Retrieve collateral-enabled status for one or more currencies.
#'
#' @param ccy Character or `NULL`. One currency or a comma-separated list of up
#'   to 20 currencies.
#' @param collateral_enabled Logical, character, or `NULL`. Filter by collateral
#'   status.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with currency-level collateral flags.
#'
#' @export
get_account_collateral_assets <- function(ccy = NULL, collateral_enabled = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(ccy = ccy, collateralEnabled = collateral_enabled)
  .gets$account_collateral_assets(query_string = query_string, config = config, tz = tz)
}

#' Get account MMP configuration
#'
#' Retrieve option-market-maker-protection configuration for one or more
#' instrument families.
#'
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with MMP configuration fields.
#'
#' @export
get_account_mmp_config <- function(
  inst_family = NULL,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instFamily = inst_family)
  .gets$account_mmp_config(query_string = query_string, config = config, tz = tz)
}

#' Get account move positions history
#'
#' Retrieve move-position requests from the last three days.
#'
#' @param block_td_id Character or `NULL`. OKX block trade identifier.
#' @param client_id Character or `NULL`. Client-supplied identifier.
#' @param begin_ts Character or `NULL`. Inclusive start timestamp in
#'   milliseconds.
#' @param end_ts Character or `NULL`. Inclusive end timestamp in milliseconds.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param state Character or `NULL`. Transfer state filter, `"filled"` or
#'   `"pending"`.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with top-level move-position metadata; nested `legs`
#'   are JSON-encoded.
#'
#' @export
get_account_move_positions_history <- function(block_td_id = NULL, client_id = NULL, begin_ts = NULL, end_ts = NULL, limit = NULL, state = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    blockTdId = block_td_id,
    clientId = client_id,
    beginTs = begin_ts,
    endTs = end_ts,
    limit = limit,
    state = state
  )
  .gets$account_move_positions_history(query_string = query_string, config = config, tz = tz)
}

#' Precheck delta-neutral strategy switch
#'
#' Retrieve unmatched information that blocks switching into the requested
#' strategy type.
#'
#' @param stgy_type Character or numeric. Strategy type. `"0"` for general or
#'   `"1"` for delta neutral.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with JSON-encoded unmatched information.
#'
#' @export
get_account_precheck_set_delta_neutral <- function(
  stgy_type,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(stgyType = stgy_type)
  .gets$account_precheck_set_delta_neutral(query_string = query_string, config = config, tz = tz)
}

#' Get archived account bill export links
#'
#' Retrieve the generated CSV export link for historical account bills since
#' 2021.
#'
#' @param year Character or numeric. Four-digit year.
#' @param quarter Character. Quarter code, one of `"Q1"` to `"Q4"`.
#' @param type Character or `NULL`. Optional comma-separated bill type filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with file-link status rows.
#'
#' @export
get_account_bills_history_archive <- function(year, quarter, type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(year = year, quarter = quarter, type = type)
  .gets$account_bills_history_archive(query_string = query_string, config = config, tz = tz)
}

#' Get sub-account trading balances
#'
#' Retrieve account-level trading balances for a sub-account from the master
#' account.
#'
#' @param sub_acct Character. Sub-account name.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with account-level balance fields; nested per-currency
#'   `details` are JSON-encoded.
#'
#' @export
get_account_subaccount_balances <- function(
  sub_acct,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(subAcct = sub_acct)
  .gets$account_subaccount_balances(query_string = query_string, config = config, tz = tz)
}

#' Get sub-account maximum withdrawals
#'
#' Retrieve the maximum withdrawal information for a sub-account from the master
#' account.
#'
#' @param sub_acct Character. Sub-account name.
#' @param ccy Character or `NULL`. One currency or a comma-separated list of up
#'   to 20 currencies.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with per-currency maximum withdrawal values.
#'
#' @export
get_account_subaccount_max_withdrawal <- function(sub_acct, ccy = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(subAcct = sub_acct, ccy = ccy)
  .gets$account_subaccount_max_withdrawal(query_string = query_string, config = config, tz = tz)
}

#' Precheck account mode switch
#'
#' Retrieve precheck information and any unmatched requirements for switching to
#' a target account mode.
#'
#' @param acct_lv Character or numeric. Target account mode.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with switch-precheck fields; nested margin and
#'   unmatched-information structures are JSON-encoded.
#'
#' @export
get_account_set_account_switch_precheck <- function(
  acct_lv,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(acctLv = acct_lv)
  .gets$account_set_account_switch_precheck(query_string = query_string, config = config, tz = tz)
}

#' Get spot borrow and repay history
#'
#' Retrieve spot-mode borrow and repay history.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param type Character or `NULL`. Event type filter.
#' @param after Character or `NULL`. Pagination cursor for earlier rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per borrow or repay event.
#'
#' @export
get_account_spot_borrow_repay_history <- function(ccy = NULL, type = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ccy = ccy,
    type = type,
    after = after,
    before = before,
    limit = limit
  )
  .gets$account_spot_borrow_repay_history(query_string = query_string, config = config, tz = tz)
}

#---- Trade: GET Wrappers ----

#' Get trade order details
#'
#' Retrieve detailed information about a specific OKX order by either the
#' exchange-assigned `ord_id` or your client-assigned `cl_ord_id`.
#'
#' @details
#' You must provide exactly one identifier: `ord_id` **or** `cl_ord_id`.
#' Timestamps in the response are converted to `POSIXct` in the supplied `tz`.
#'
#' @param inst_id Character. Instrument ID, e.g. `"ETH-USDT-SWAP"` (perps),
#'   `"BTC-USDT"` (spot), or `"BTC-USD-240927"` (dated futures).
#' @param ord_id Character, optional. The OKX order ID. Provide this **or** `cl_ord_id`.
#' @param cl_ord_id Character, optional. Your client order ID. Provide this **or** `ord_id`.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`. May also include `base_url`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` (one row) with order details following OKX schema (e.g.,
#' `ordId`, `clOrdId`, `instId`, `ordType`, `px`, `sz`, `side`, `posSide`,
#' `tdMode`, `accFillSz`, `fillPx`, `fillSz`, `fillTime`, `avgPx`, `state`,
#' `lever`, etc.). Timestamp columns are `POSIXct` in `tz`.
#'
#' @section Common errors:
#' - `Either 'ord_id' or 'cl_ord_id' must be provided.` (client-side)
#' - HTTP 401 Unauthorized (missing/invalid credentials)
#' - OKX `code` like `51000` invalid sign or `51603` order not found
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_trade_order(
#'   inst_id = "ETH-USDT-SWAP",
#'   ord_id  = "1234567890",
#'   config  = cfg
#' )
#' }
#'
#' @seealso [get_trade_orders_pending()], [get_trade_orders_history_7d()]
#' @note Since okxr 0.1.1
#' @export
get_trade_order <- function(inst_id, ord_id = NULL, cl_ord_id = NULL, config, tz = .okx_default_tz) {
  if (xor(is.null(ord_id), is.null(cl_ord_id)) == FALSE) {
    stop("Provide exactly one of `ord_id` or `cl_ord_id`.", call. = FALSE)
  }

  query_string <- .okx_build_query(
    instId = inst_id,
    ordId = ord_id,
    clOrdId = cl_ord_id
  )
  .gets$trade_order(query_string = query_string, config = config, tz = tz)
}

#' Get all pending trade orders
#'
#' Retrieve all currently open (unfilled) orders for your OKX account.
#'
#' @details
#' Returns one row per open order. Timestamps are parsed to `POSIXct` using `tz`.
#'
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`. May also include `base_url`.
#' @param tz Character. Time zone for parsing timestamps (e.g. `"Asia/Hong_Kong"`).
#'
#' @return
#' A `data.frame` with one row per pending order and columns following the OKX
#' schema (e.g., `cTime`, `ordId`, `clOrdId`, `tag`, `instId`, `ordType`, `px`,
#' `sz`, `side`, `posSide`, `tdMode`, `accFillSz`, `fillPx`, `fillSz`,
#' `fillTime`, `avgPx`, `state`, `lever`, ...). Timestamp columns are `POSIXct`.
#'
#' @section Common errors:
#' - HTTP 401 Unauthorized (missing/invalid credentials)
#' - Rate limiting: HTTP 429 / OKX throttle codes
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' df <- get_trade_orders_pending(config = cfg, tz = "Asia/Hong_Kong")
#' head(df)
#' }
#'
#' @seealso [get_trade_order()], [get_trade_orders_history_7d()]
#' @note Since okxr 0.1.1
#' @export
get_trade_orders_pending <- function(config, tz = .okx_default_tz) {
  .gets$trade_orders_pending(query_string = "", config = config, tz = tz)
}

#' Get trade orders history (last 7 days)
#'
#' Retrieve recent order history for an instrument type.
#'
#' This wraps `/api/v5/trade/orders-history` and covers about 7 days of data.
#' Older data is available from OKX's archive endpoint.
#'
#' @param inst_type Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
#'   `"SWAP"`, `"FUTURES"`, `"OPTION"`. Default `"SWAP"`.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`. May also include `base_url`.
#' @param tz Character. Time zone for parsing timestamps (e.g. `"Asia/Hong_Kong"`).
#'
#' @return
#' A `data.frame` with one row per historical order and columns following the OKX
#' schema (same layout as pending orders, plus final states). Timestamp columns
#' are `POSIXct` in `tz`.
#'
#' @section Common errors:
#' - HTTP 401 Unauthorized
#' - HTTP 400 Bad Request for invalid `inst_type`
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' hist <- get_trade_orders_history_7d(
#'   inst_type = "SWAP",
#'   config = cfg,
#'   tz = "Asia/Hong_Kong"
#' )
#' tail(hist)
#' }
#'
#' @seealso [get_trade_order()], [get_trade_orders_pending()]
#' @note Since okxr 0.1.2
#' @export
get_trade_orders_history_7d <- function(
  inst_type = "SWAP",
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$trade_orders_history_7d(query_string = query_string, config = config, tz = tz)
}

#' Get trade fills
#'
#' Retrieve recently filled transaction details from the last 3 days.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param inst_id Character or `NULL`. Instrument ID filter.
#' @param ord_id Character or `NULL`. Order ID filter.
#' @param sub_type Character or `NULL`. Transaction subtype filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param begin Character or `NULL`. Begin timestamp in milliseconds.
#' @param end Character or `NULL`. End timestamp in milliseconds.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with fill rows.
#'
#' @export
get_trade_fills <- function(inst_type = NULL, inst_family = NULL, inst_id = NULL, ord_id = NULL, sub_type = NULL, after = NULL, before = NULL, limit = NULL, begin = NULL, end = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family,
    instId = inst_id,
    ordId = ord_id,
    subType = sub_type,
    after = after,
    before = before,
    limit = limit,
    begin = begin,
    end = end
  )
  .gets$trade_fills(query_string = query_string, config = config, tz = tz)
}

#' Get trade fills history
#'
#' Retrieve historical filled transaction details from the last 3 months.
#'
#' @param inst_type Character. Instrument type, e.g. `"SPOT"` or `"SWAP"`.
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param inst_id Character or `NULL`. Instrument ID filter.
#' @param ord_id Character or `NULL`. Order ID filter.
#' @param sub_type Character or `NULL`. Transaction subtype filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param begin Character or `NULL`. Begin timestamp in milliseconds.
#' @param end Character or `NULL`. End timestamp in milliseconds.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with historical fill rows.
#'
#' @export
get_trade_fills_history <- function(inst_type, inst_family = NULL, inst_id = NULL, ord_id = NULL, sub_type = NULL, after = NULL, before = NULL, limit = NULL, begin = NULL, end = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family,
    instId = inst_id,
    ordId = ord_id,
    subType = sub_type,
    after = after,
    before = before,
    limit = limit,
    begin = begin,
    end = end
  )
  .gets$trade_fills_history(query_string = query_string, config = config, tz = tz)
}

#' Get trade account rate limit
#'
#' Retrieve account rate limit information related to new and amended order
#' requests.
#'
#' @details
#' Wraps `/api/v5/trade/account-rate-limit`. Returns one row containing the
#' current account rate limit and fill-ratio metrics used by OKX.
#'
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with rate-limit metrics such as
#'   `accRateLimit`, `fillRatio`, `mainFillRatio`, `nextAccRateLimit`, and `ts`.
#'
#' @export
get_trade_account_rate_limit <- function(config, tz = .okx_default_tz) {
  .gets$trade_account_rate_limit(query_string = "", config = config, tz = tz)
}

#' Get trade orders history
#'
#' Retrieve completed orders from the last 7 days.
#'
#' @param inst_type Character. Instrument type.
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param inst_id Character or `NULL`. Instrument ID filter.
#' @param ord_type Character or `NULL`. Order type filter.
#' @param state Character or `NULL`. Order state filter.
#' @param category Character or `NULL`. Order category filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param begin Character or `NULL`. Begin timestamp in milliseconds.
#' @param end Character or `NULL`. End timestamp in milliseconds.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with historical order rows.
#' @export
get_trade_orders_history <- function(inst_type, inst_family = NULL, inst_id = NULL, ord_type = NULL, state = NULL, category = NULL, after = NULL, before = NULL, begin = NULL, end = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family,
    instId = inst_id,
    ordType = ord_type,
    state = state,
    category = category,
    after = after,
    before = before,
    begin = begin,
    end = end,
    limit = limit
  )
  .gets$trade_orders_history(query_string = query_string, config = config, tz = tz)
}

#' Get archived trade orders history
#'
#' Retrieve completed orders from the last 3 months.
#'
#' @inheritParams get_trade_orders_history
#'
#' @return A `data.frame` with archived order rows.
#' @export
get_trade_orders_history_archive <- function(inst_type, inst_family = NULL, inst_id = NULL, ord_type = NULL, state = NULL, category = NULL, after = NULL, before = NULL, begin = NULL, end = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family,
    instId = inst_id,
    ordType = ord_type,
    state = state,
    category = category,
    after = after,
    before = before,
    begin = begin,
    end = end,
    limit = limit
  )
  .gets$trade_orders_history_archive(query_string = query_string, config = config, tz = tz)
}

#' Get easy convert currency list
#'
#' Retrieve currencies available for easy convert.
#'
#' @param source Character or `NULL`. Source account type filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with source and target currency metadata.
#' @export
get_trade_easy_convert_currency_list <- function(source = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(source = source)
  .gets$trade_easy_convert_currency_list(query_string = query_string, config = config, tz = tz)
}

#' Get easy convert history
#'
#' Retrieve easy convert history.
#'
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with easy convert history rows.
#' @export
get_trade_easy_convert_history <- function(after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(after = after, before = before, limit = limit)
  .gets$trade_easy_convert_history(query_string = query_string, config = config, tz = tz)
}

#' Get one-click repay currency list
#'
#' Retrieve repayable currencies for the legacy one-click repay endpoint.
#'
#' @param debt_type Character or `NULL`. Debt type filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with debt and repay currency metadata.
#' @export
get_trade_one_click_repay_currency_list <- function(debt_type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(debtType = debt_type)
  .gets$trade_one_click_repay_currency_list(query_string = query_string, config = config, tz = tz)
}

#' Get one-click repay history
#'
#' Retrieve legacy one-click repay history.
#'
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with one-click repay history rows.
#' @export
get_trade_one_click_repay_history <- function(after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(after = after, before = before, limit = limit)
  .gets$trade_one_click_repay_history(query_string = query_string, config = config, tz = tz)
}

#' Get one-click repay currency list v2
#'
#' Retrieve repayable currencies for the new one-click repay endpoint.
#'
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with debt and repay currency metadata.
#' @export
get_trade_one_click_repay_currency_list_v2 <- function(
  config,
  tz = .okx_default_tz
) {
  .gets$trade_one_click_repay_currency_list_v2(query_string = "", config = config, tz = tz)
}

#' Get one-click repay history v2
#'
#' Retrieve new one-click repay history.
#'
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with one-click repay history rows.
#' @export
get_trade_one_click_repay_history_v2 <- function(after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(after = after, before = before, limit = limit)
  .gets$trade_one_click_repay_history_v2(query_string = query_string, config = config, tz = tz)
}

#' Get a single algo order
#'
#' Retrieve a specific algo order by algo ID or client algo order ID.
#'
#' @param algo_id Character or `NULL`. Algo order ID.
#' @param algo_cl_ord_id Character or `NULL`. Client algo order ID.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with algo order details.
#' @export
get_trade_order_algo <- function(algo_id = NULL, algo_cl_ord_id = NULL, config, tz = .okx_default_tz) {
  if (xor(is.null(algo_id), is.null(algo_cl_ord_id)) == FALSE) {
    stop("Provide exactly one of `algo_id` or `algo_cl_ord_id`.", call. = FALSE)
  }

  query_string <- .okx_build_query(algoId = algo_id, algoClOrdId = algo_cl_ord_id)
  .gets$trade_order_algo(query_string = query_string, config = config, tz = tz)
}

#' Get pending algo orders
#'
#' Retrieve untriggered algo orders.
#'
#' @param ord_type Character. Algo order type filter.
#' @param algo_id Character or `NULL`. Algo order ID.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param inst_id Character or `NULL`. Instrument ID filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with pending algo orders.
#' @export
get_trade_orders_algo_pending <- function(ord_type, algo_id = NULL, inst_type = NULL, inst_id = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ordType = ord_type,
    algoId = algo_id,
    instType = inst_type,
    instId = inst_id,
    after = after,
    before = before,
    limit = limit
  )
  .gets$trade_orders_algo_pending(query_string = query_string, config = config, tz = tz)
}

#' Get algo order history
#'
#' Retrieve historical algo orders.
#'
#' @param ord_type Character. Algo order type filter.
#' @param state Character or `NULL`. Algo order state filter.
#' @param algo_id Character or `NULL`. Algo order ID.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param inst_id Character or `NULL`. Instrument ID filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of results to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps.
#'
#' @return A `data.frame` with algo order history rows.
#' @export
get_trade_orders_algo_history <- function(ord_type, state = NULL, algo_id = NULL, inst_type = NULL, inst_id = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ordType = ord_type,
    state = state,
    algoId = algo_id,
    instType = inst_type,
    instId = inst_id,
    after = after,
    before = before,
    limit = limit
  )
  .gets$trade_orders_algo_history(query_string = query_string, config = config, tz = tz)
}

#' Internal OKX request helpers
#'
#' Shared helpers for request validation, query construction, and package-wide
#' defaults used by user-facing wrappers.
#'
#' @keywords internal
.okx_default_tz <- "Asia/Hong_Kong"

#' @keywords internal
.okx_default_timeout <- 10

#' @keywords internal
.okx_validate_config <- function(config) {
  if (!is.list(config)) {
    stop("`config` must be a list.", call. = FALSE)
  }

  required_fields <- c("api_key", "secret_key", "passphrase")
  missing_fields <- setdiff(required_fields, names(config))
  if (length(missing_fields) > 0L) {
    stop(
      "Missing required config field(s): ",
      paste(missing_fields, collapse = ", "),
      call. = FALSE
    )
  }

  invisible(config)
}

#' @keywords internal
.okx_request_timeout <- function(config = NULL) {
  timeout <- if (is.list(config) && !is.null(config$timeout)) {
    config$timeout
  } else {
    getOption("okxr.timeout", .okx_default_timeout)
  }

  if (!is.numeric(timeout) || length(timeout) != 1L || is.na(timeout) || timeout <= 0) {
    stop("Request timeout must be a single positive number of seconds.", call. = FALSE)
  }

  httr::timeout(timeout)
}

#' @keywords internal
.okx_build_query <- function(...) {
  params <- list(...)
  keep <- !vapply(
    params,
    function(value) is.null(value) || length(value) == 0L || identical(value, ""),
    logical(1)
  )
  params <- params[keep]

  if (length(params) == 0L) {
    return("")
  }

  parts <- Map(
    function(name, value) {
      if (length(value) != 1L) {
        stop("Each query parameter must be length 1.", call. = FALSE)
      }

      paste0(
        utils::URLencode(name, reserved = TRUE),
        "=",
        utils::URLencode(as.character(value), reserved = TRUE)
      )
    },
    names(params),
    params
  )

  paste0("?", paste(unlist(parts, use.names = FALSE), collapse = "&"))
}

#' @keywords internal
.okx_datetime_to_ms <- function(value, tz = .okx_default_tz, format = "%Y-%m-%d %H:%M:%S") {
  if (is.null(value)) {
    return(NULL)
  }

  timestamp <- as.POSIXct(value, format = format, tz = tz)
  if (is.na(timestamp)) {
    stop(
      "`value` must be parseable with format '", format, "'.",
      call. = FALSE
    )
  }

  as.integer(as.numeric(timestamp) * 1000)
}

#' @keywords internal
.okx_extract_result <- function(parsed_res, raw_data = FALSE) {
  if (is.null(parsed_res)) {
    return(NULL)
  }

  if (raw_data) parsed_res$data_raw else parsed_res$data_dt
}

#' @keywords internal
.okx_has_value <- function(value) {
  !(is.null(value) || length(value) == 0L || identical(value, ""))
}

#' @keywords internal
.okx_assert_exactly_one_present <- function(..., names = NULL) {
  values <- list(...)
  keep <- vapply(values, .okx_has_value, logical(1))

  if (sum(keep) != 1L) {
    if (is.null(names)) {
      names <- names(values)
    }
    stop(
      "Provide exactly one of ",
      paste(sprintf("`%s`", names), collapse = " or "),
      ".",
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' @keywords internal
.okx_assert_non_empty_list <- function(x, arg) {
  if (!is.list(x) || length(x) == 0L) {
    stop("`", arg, "` must be a non-empty list.", call. = FALSE)
  }

  invisible(TRUE)
}

#' @keywords internal
.okx_assert_has_fields <- function(x, fields, arg) {
  missing_fields <- fields[!vapply(fields, function(field) .okx_has_value(x[[field]]), logical(1))]
  if (length(missing_fields) > 0L) {
    stop(
      "`", arg, "` is missing required field(s): ",
      paste(missing_fields, collapse = ", "),
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' @keywords internal
.okx_assert_any_field_present <- function(x, fields, arg) {
  has_any <- any(vapply(fields, function(field) .okx_has_value(x[[field]]), logical(1)))
  if (!has_any) {
    stop(
      "`", arg, "` must include at least one of: ",
      paste(fields, collapse = ", "),
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' @keywords internal
.okx_generate_client_order_id <- function(prefix = "r") {
  paste0(prefix, format(Sys.time(), "%Y%m%d%H%M%S"), sample(1000:9999, 1))
}

test_that(".okx_build_query drops null/empty values and URL-encodes", {
  expect_equal(okxr:::.okx_build_query(), "")
  expect_equal(okxr:::.okx_build_query(instId = "BTC-USDT"), "?instId=BTC-USDT")
  expect_equal(
    okxr:::.okx_build_query(instId = "BTC-USDT", empty = "", missing = NULL),
    "?instId=BTC-USDT"
  )
  expect_equal(
    okxr:::.okx_build_query(instId = "BTC USDT", ordType = "post_only,fok"),
    "?instId=BTC%20USDT&ordType=post_only%2Cfok"
  )
  expect_error(okxr:::.okx_build_query(instId = c("BTC-USDT", "ETH-USDT")), "length 1")
})

test_that(".okx_validate_config validates required credentials", {
  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")
  expect_invisible(okxr:::.okx_validate_config(cfg))
  expect_error(okxr:::.okx_validate_config("bad"), "`config` must be a list")
  expect_error(
    okxr:::.okx_validate_config(list(api_key = "key")),
    "secret_key, passphrase"
  )
})

test_that(".okx_datetime_to_ms parses expected timestamp format", {
  expect_equal(
    okxr:::.okx_datetime_to_ms("1970-01-01 00:00:01", tz = "UTC"),
    1000L
  )
  expect_null(okxr:::.okx_datetime_to_ms(NULL))
  expect_error(okxr:::.okx_datetime_to_ms("bad", tz = "UTC"), "parseable")
})

test_that(".okx_extract_result respects raw_data flag", {
  parsed <- list(data_raw = list(a = 1), data_dt = data.frame(a = 1))
  expect_equal(okxr:::.okx_extract_result(parsed, raw_data = TRUE), parsed$data_raw)
  expect_equal(okxr:::.okx_extract_result(parsed, raw_data = FALSE), parsed$data_dt)
  expect_null(okxr:::.okx_extract_result(NULL))
})

test_that("internal POST validation helpers enforce required shapes", {
  expect_true(okxr:::.okx_has_value("x"))
  expect_false(okxr:::.okx_has_value(""))
  expect_false(okxr:::.okx_has_value(NULL))

  expect_invisible(okxr:::.okx_assert_exactly_one_present("1", NULL, names = c("a", "b")))
  expect_error(
    okxr:::.okx_assert_exactly_one_present(NULL, NULL, names = c("a", "b")),
    "Provide exactly one of `a` or `b`"
  )
  expect_error(
    okxr:::.okx_assert_non_empty_list(list(), "orders"),
    "`orders` must be a non-empty list"
  )
  expect_error(
    okxr:::.okx_assert_has_fields(list(inst_id = "BTC-USDT"), c("inst_id", "side"), "orders[[1]]"),
    "missing required field\\(s\\): side"
  )
  expect_error(
    okxr:::.okx_assert_any_field_present(list(a = NULL, b = ""), c("a", "b"), "req"),
    "must include at least one of: a, b"
  )
})

test_that(".okx_request_timeout validates timeout sources", {
  timeout <- okxr:::.okx_request_timeout(list(timeout = 3))
  expect_s3_class(timeout, "request")
  expect_error(okxr:::.okx_request_timeout(list(timeout = 0)), "positive")
  expect_error(okxr:::.okx_request_timeout(list(timeout = "bad")), "positive")
})

test_that(".build_request can build unsigned public requests without config", {
  req <- okxr:::.build_request(
    httr_method = "GET",
    base_url = "https://www.okx.com",
    api_path = "/api/v5/public/time",
    query_string = "",
    auth = FALSE
  )

  expect_equal(req$url, "https://www.okx.com/api/v5/public/time")
  expect_null(req$headers)
})

test_that(".execute_get_action handles unsigned success, HTTP error, and request error", {
  called <- new.env(parent = emptyenv())
  called$url <- NULL
  called$has_headers <- FALSE

  testthat::local_mocked_bindings(
    GET = function(url, ...) {
      called$url <- url
      called$has_headers <- length(list(...)) > 1L
      mock_http_response()
    },
    .package = "httr"
  )

  res <- okxr:::.execute_get_action("/api/v5/public/time", "", auth = FALSE)
  expect_s3_class(res, "response")
  expect_equal(called$url, "https://www.okx.com/api/v5/public/time")
  expect_false(called$has_headers)

  testthat::local_mocked_bindings(
    GET = function(url, ...) mock_http_response(status_code = 500L),
    .package = "httr"
  )
  expect_warning(
    expect_null(okxr:::.execute_get_action("/api/v5/public/time", "", auth = FALSE)),
    "Request failed: 500"
  )

  testthat::local_mocked_bindings(
    GET = function(url, ...) stop("timeout"),
    .package = "httr"
  )
  expect_warning(
    expect_null(okxr:::.execute_get_action("/api/v5/public/time", "", auth = FALSE)),
    "timeout"
  )
})

test_that(".execute_get_action validates credentials for private requests before HTTP", {
  testthat::local_mocked_bindings(
    GET = function(url, ...) stop("should not call HTTP"),
    .package = "httr"
  )

  expect_error(
    okxr:::.execute_get_action("/api/v5/account/balance", "", config = list(), auth = TRUE),
    "Missing required config field"
  )
})

test_that(".execute_post_action handles success, HTTP error, and request error", {
  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")
  called <- new.env(parent = emptyenv())
  called$body <- NULL
  called$encode <- NULL

  testthat::local_mocked_bindings(
    POST = function(url, ..., body = NULL, encode = NULL) {
      called$body <- body
      called$encode <- encode
      mock_http_response()
    },
    .package = "httr"
  )

  res <- okxr:::.execute_post_action("/api/v5/trade/order", list(instId = "BTC-USDT"), cfg)
  expect_s3_class(res, "response")
  expect_match(called$body, "BTC-USDT")
  expect_equal(called$encode, "raw")

  testthat::local_mocked_bindings(
    POST = function(url, ..., body = NULL, encode = NULL) mock_http_response(status_code = 429L),
    .package = "httr"
  )
  expect_warning(
    expect_null(okxr:::.execute_post_action("/api/v5/trade/order", list(), cfg)),
    "Request failed: 429"
  )

  testthat::local_mocked_bindings(
    POST = function(url, ..., body = NULL, encode = NULL) stop("connection failed"),
    .package = "httr"
  )
  expect_warning(
    expect_null(okxr:::.execute_post_action("/api/v5/trade/order", list(), cfg)),
    "connection failed"
  )
})

#' @importFrom rlang %||%
NULL

#' Set or get okxr options
#'
#' @description
#' Convenience wrapper to set global options for okxr, such as whether to
#' return raw data instead of parsed data.
#'
#' @param raw_data Logical. If `TRUE`, return raw API `data`.
#'   If `NULL`, the current value is left unchanged.
#' @param timeout Numeric. HTTP request timeout in seconds. If `NULL`, the
#'   current value is left unchanged.
#' @return An invisible named list with the current package options:
#'   `raw_data` (logical) and `timeout` (numeric seconds). This return value
#'   can be used to inspect the effective option state after updating it.
#'
#' @examples
#' old <- getOption("okxr.raw_data")
#' old_timeout <- getOption("okxr.timeout")
#' set_okxr_options(raw_data = TRUE)
#' set_okxr_options(timeout = 5)
#' options(okxr.raw_data = old, okxr.timeout = old_timeout)
#'
#' set_okxr_options()  # check current values
#' @export
set_okxr_options <- function(raw_data = NULL, timeout = NULL) {
  if (!is.null(raw_data)) {
    if (!is.logical(raw_data) || length(raw_data) != 1L)
      stop("`raw_data` must be a single TRUE or FALSE value.", call. = FALSE)

    options(okxr.raw_data = raw_data)
  }

  if (!is.null(timeout)) {
    if (!is.numeric(timeout) || length(timeout) != 1L || is.na(timeout) || timeout <= 0)
      stop("`timeout` must be a single positive number.", call. = FALSE)

    options(okxr.timeout = timeout)
  }

  invisible(list(
    raw_data = getOption("okxr.raw_data", FALSE),
    timeout = getOption("okxr.timeout", .okx_default_timeout)
  ))
}

#' Base URL for OKX API
#'
#' Canonical base URL used by all OKX REST requests in **okxr**.
#' Keep this as a single source of truth so higher-level request builders can
#' compose full URLs as `paste0(.okx_base_url, okx_path)`.
#'
#' @details
#' This package assumes the public production host. If you intend to support
#' alternative environments (e.g., sandbox), consider injecting a config value
#' at runtime rather than modifying this constant.
#'
#' @format A length-one character vector.
#' @seealso [`.api_GET_specs`], [`.api_POST_specs`]
#' @family okxr-internal
#' @note Since okxr 0.1.1
#' @keywords internal
#' @noRd
.okx_base_url <- "https://www.okx.com"

#----.API_POST_SPECS----

#' OKX POST endpoint specifications (internal)
#'
#' A named list describing how to call and parse selected **POST** endpoints.
#'
#' Each entry has:
#' - `okx_path` (character): REST path beginning with `/api/...`.
#' - `parser_schema` (`data.frame`): three columns
#'   - `okx`: field name as returned by OKX
#'   - `formal`: human-readable label
#'   - `type`: one of `"string"`, `"numeric"`, `"integer"`, `"time"`
#' - `parser_mode` (character): either `"named"` or `"positional"`.
#'
#' @section Parser modes:
#' - **named**: parse by JSON key (robust to column order).
#' - **positional**: parse by array position (used by endpoints returning
#'   vectors/arrays such as candles; order is critical).
#'
#' @section Included endpoints:
#' - `account_set_leverage`: `/api/v5/account/set-leverage`
#' - `trade_order`: `/api/v5/trade/order`
#' - `trade_cancel_order`: `/api/v5/trade/cancel-order`
#' - `trade_close_position`: `/api/v5/trade/close-position`
#'
#' @examples
#' # Access path for placing an order
#' .api_POST_specs$trade_order$okx_path
#'
#' # Inspect the expected fields for the cancel-order response
#' .api_POST_specs$trade_cancel_order$parser_schema
#'
#' @format A named list of endpoint specification lists.
#' @seealso [`.api_GET_specs`]
#' @family okxr-internal
#' @note Since okxr 0.1.1
#' @keywords internal
#' @noRd
.trade_cancel_response_schema <- data.frame(
  okx    = c("ts", "ordId", "clOrdId", "sCode", "sMsg"),
  formal = c("Timestamp", "Order ID", "Client Order ID", "Code of the execution result", "Execution message"),
  type   = c("time", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.trade_amend_response_schema <- data.frame(
  okx    = c("ts", "ordId", "clOrdId", "reqId", "sCode", "sMsg", "subCode"),
  formal = c("Timestamp", "Order ID", "Client Order ID", "Client request ID", "Code of the execution result", "Execution message", "Execution sub-code"),
  type   = c("time", "string", "string", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.trade_algo_cancel_response_schema <- data.frame(
  okx    = c("algoId", "algoClOrdId", "clOrdId", "sCode", "sMsg", "tag"),
  formal = c("Algo ID", "Client-supplied Algo ID", "Client Order ID", "Code of the execution result", "Execution message", "Order tag"),
  type   = c("string", "string", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.trade_algo_amend_response_schema <- data.frame(
  okx    = c("algoId", "algoClOrdId", "reqId", "sCode", "sMsg"),
  formal = c("Algo ID", "Client-supplied Algo ID", "Client request ID", "Code of the execution result", "Execution message"),
  type   = c("string", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.account_position_mode_schema <- data.frame(
  okx    = c("posMode"),
  formal = c("Position mode"),
  type   = c("string"),
  stringsAsFactors = FALSE
)

.account_fee_type_schema <- data.frame(
  okx    = c("feeType"),
  formal = c("Fee type"),
  type   = c("string"),
  stringsAsFactors = FALSE
)

.account_greeks_type_schema <- data.frame(
  okx    = c("greeksType"),
  formal = c("Greeks display type"),
  type   = c("string"),
  stringsAsFactors = FALSE
)

.account_auto_flag_schema <- data.frame(
  okx    = c("autoRepay"),
  formal = c("Auto repay enabled"),
  type   = c("logical"),
  stringsAsFactors = FALSE
)

.account_auto_loan_schema <- data.frame(
  okx    = c("autoLoan"),
  formal = c("Auto loan enabled"),
  type   = c("logical"),
  stringsAsFactors = FALSE
)

.account_level_schema <- data.frame(
  okx    = c("acctLv"),
  formal = c("Account mode"),
  type   = c("string"),
  stringsAsFactors = FALSE
)

.account_collateral_assets_schema <- data.frame(
  okx    = c("type", "ccyList", "collateralEnabled"),
  formal = c("Type", "Currency list", "Collateral enabled"),
  type   = c("string", "string", "logical"),
  stringsAsFactors = FALSE
)

.account_margin_balance_schema <- data.frame(
  okx    = c("instId", "posSide", "amt", "type", "leverage", "ccy"),
  formal = c("Instrument ID", "Position side", "Amount", "Margin action type", "Leverage", "Currency"),
  type   = c("string", "string", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.account_borrow_repay_schema <- data.frame(
  okx    = c("ccy", "side", "amt"),
  formal = c("Currency", "Side", "Amount"),
  type   = c("string", "string", "string"),
  stringsAsFactors = FALSE
)

.account_switch_preset_schema <- data.frame(
  okx    = c("curAcctLv", "acctLv", "lever", "riskOffsetType"),
  formal = c("Current account mode", "Target account mode", "Preset leverage", "Risk offset type"),
  type   = c("string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.account_result_schema <- data.frame(
  okx    = c("result"),
  formal = c("Result"),
  type   = c("logical"),
  stringsAsFactors = FALSE
)

.account_mmp_config_schema <- data.frame(
  okx    = c("instFamily", "timeInterval", "frozenInterval", "qtyLimit"),
  formal = c("Instrument family", "Time interval", "Frozen interval", "Quantity limit"),
  type   = c("string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.account_move_positions_schema <- data.frame(
  okx    = c("clientId", "blockTdId", "state", "ts", "fromAcct", "toAcct", "legs"),
  formal = c("Client ID", "Block trade ID", "State", "Timestamp", "Source account", "Destination account", "Legs"),
  type   = c("string", "string", "string", "time", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.asset_transfer_response_schema <- data.frame(
  okx    = c("transId", "ccy", "clientId", "from", "amt", "to"),
  formal = c("Transfer ID", "Currency", "Client ID", "Source account", "Amount", "Destination account"),
  type   = c("string", "string", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.asset_withdrawal_response_schema <- data.frame(
  okx    = c("amt", "wdId", "ccy", "clientId", "chain"),
  formal = c("Amount", "Withdrawal ID", "Currency", "Client ID", "Chain"),
  type   = c("string", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.asset_cancel_withdrawal_response_schema <- data.frame(
  okx    = c("wdId"),
  formal = c("Withdrawal ID"),
  type   = c("string"),
  stringsAsFactors = FALSE
)

.asset_convert_estimate_quote_schema <- data.frame(
  okx    = c("quoteTime", "ttlMs", "clQReqId", "quoteId", "baseCcy", "quoteCcy", "side", "origRfqSz", "rfqSz", "rfqSzCcy", "cnvtPx", "baseSz", "quoteSz"),
  formal = c("Quote time", "Quote TTL in milliseconds", "Client quote request ID", "Quote ID", "Base currency", "Quote currency", "Side", "Original RFQ size", "RFQ size", "RFQ size currency", "Conversion price", "Base size", "Quote size"),
  type   = c("time", "numeric", "string", "string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric"),
  stringsAsFactors = FALSE
)

.asset_convert_trade_schema <- data.frame(
  okx    = c("tradeId", "quoteId", "clTReqId", "state", "instId", "baseCcy", "quoteCcy", "side", "fillPx", "fillBaseSz", "fillQuoteSz", "ts"),
  formal = c("Trade ID", "Quote ID", "Client trade request ID", "State", "Instrument ID", "Base currency", "Quote currency", "Side", "Fill price", "Filled base size", "Filled quote size", "Timestamp"),
  type   = c("string", "string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric", "time"),
  stringsAsFactors = FALSE
)

.api_POST_specs <- list(
  
  #----account_set_leverage----
  account_set_leverage = list(
    okx_path     = "/api/v5/account/set-leverage",
    parser_schema       = data.frame(
      okx    = c("mgnMode", "instId", "posSide"),
      formal = c("Margin mode", "Instrument ID", "Position side"),
      type   = c("string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_set_position_mode----
  account_set_position_mode = list(
    okx_path     = "/api/v5/account/set-position-mode",
    parser_schema = .account_position_mode_schema,
    parser_mode = "named"
  ),

  #----account_set_fee_type----
  account_set_fee_type = list(
    okx_path     = "/api/v5/account/set-fee-type",
    parser_schema = .account_fee_type_schema,
    parser_mode = "named"
  ),

  #----account_set_greeks----
  account_set_greeks = list(
    okx_path     = "/api/v5/account/set-greeks",
    parser_schema = .account_greeks_type_schema,
    parser_mode = "named"
  ),

  #----account_set_auto_repay----
  account_set_auto_repay = list(
    okx_path     = "/api/v5/account/set-auto-repay",
    parser_schema = .account_auto_flag_schema,
    parser_mode = "named"
  ),

  #----account_set_auto_loan----
  account_set_auto_loan = list(
    okx_path     = "/api/v5/account/set-auto-loan",
    parser_schema = .account_auto_loan_schema,
    parser_mode = "named"
  ),

  #----account_set_account_level----
  account_set_account_level = list(
    okx_path     = "/api/v5/account/set-account-level",
    parser_schema = .account_level_schema,
    parser_mode = "named"
  ),

  #----account_set_collateral_assets----
  account_set_collateral_assets = list(
    okx_path     = "/api/v5/account/set-collateral-assets",
    parser_schema = .account_collateral_assets_schema,
    parser_mode = "named"
  ),

  #----account_position_margin_balance----
  account_position_margin_balance = list(
    okx_path     = "/api/v5/account/position/margin-balance",
    parser_schema = .account_margin_balance_schema,
    parser_mode = "named"
  ),

  #----account_spot_manual_borrow_repay----
  account_spot_manual_borrow_repay = list(
    okx_path     = "/api/v5/account/spot-manual-borrow-repay",
    parser_schema = .account_borrow_repay_schema,
    parser_mode = "named"
  ),

  #----account_account_level_switch_preset----
  account_account_level_switch_preset = list(
    okx_path     = "/api/v5/account/account-level-switch-preset",
    parser_schema = .account_switch_preset_schema,
    parser_mode = "named"
  ),

  #----account_mmp_reset----
  account_mmp_reset = list(
    okx_path     = "/api/v5/account/mmp-reset",
    parser_schema = .account_result_schema,
    parser_mode = "named"
  ),

  #----account_mmp_config----
  account_mmp_config = list(
    okx_path     = "/api/v5/account/mmp-config",
    parser_schema = .account_mmp_config_schema,
    parser_mode = "named"
  ),

  #----account_move_positions----
  account_move_positions = list(
    okx_path     = "/api/v5/account/move-positions",
    parser_schema = .account_move_positions_schema,
    parser_mode = "named"
  ),

  #----asset_transfer----
  asset_transfer = list(
    okx_path     = "/api/v5/asset/transfer",
    parser_schema = .asset_transfer_response_schema,
    parser_mode = "named"
  ),

  #----asset_withdrawal----
  asset_withdrawal = list(
    okx_path     = "/api/v5/asset/withdrawal",
    parser_schema = .asset_withdrawal_response_schema,
    parser_mode = "named"
  ),

  #----asset_cancel_withdrawal----
  asset_cancel_withdrawal = list(
    okx_path     = "/api/v5/asset/cancel-withdrawal",
    parser_schema = .asset_cancel_withdrawal_response_schema,
    parser_mode = "named"
  ),

  #----asset_convert_estimate_quote----
  asset_convert_estimate_quote = list(
    okx_path     = "/api/v5/asset/convert/estimate-quote",
    parser_schema = .asset_convert_estimate_quote_schema,
    parser_mode = "named"
  ),

  #----asset_convert_trade----
  asset_convert_trade = list(
    okx_path     = "/api/v5/asset/convert/trade",
    parser_schema = .asset_convert_trade_schema,
    parser_mode = "named"
  ),
  
  #----trade_order----
  trade_order = list(
    okx_path     = "/api/v5/trade/order",
    parser_schema       = data.frame(
      okx    = c("ts", "ordId", "clOrdId", "sCode"),
      formal = c("Timestamp", "Order ID", "Client Order ID", "Code of the execution result"),
      type   = c("time", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----trade_cancel_order----
  trade_cancel_order = list(
    okx_path     = "/api/v5/trade/cancel-order",
    parser_schema       = .trade_cancel_response_schema,
    parser_mode = "named"
  ),
  
  #----trade_close_position----
  trade_close_position = list(
    okx_path     = "/api/v5/trade/close-position",
    parser_schema       = data.frame(
      okx    = c("instId", "posSide", "clOrdId", "tag"),
      formal = c("Instrument ID", "Position side", "Client Order ID", "Order tag"),
      type   = c("string", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_batch_orders----
  trade_batch_orders = list(
    okx_path     = "/api/v5/trade/batch-orders",
    parser_schema = data.frame(
      okx    = c("ts", "ordId", "clOrdId", "tag", "sCode", "sMsg"),
      formal = c("Timestamp", "Order ID", "Client Order ID", "Order tag", "Code of the execution result", "Execution message"),
      type   = c("time", "string", "string", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_cancel_batch_orders----
  trade_cancel_batch_orders = list(
    okx_path     = "/api/v5/trade/cancel-batch-orders",
    parser_schema = .trade_cancel_response_schema,
    parser_mode = "named"
  ),

  #----trade_amend_order----
  trade_amend_order = list(
    okx_path     = "/api/v5/trade/amend-order",
    parser_schema = .trade_amend_response_schema,
    parser_mode = "named"
  ),

  #----trade_amend_batch_orders----
  trade_amend_batch_orders = list(
    okx_path     = "/api/v5/trade/amend-batch-orders",
    parser_schema = .trade_amend_response_schema,
    parser_mode = "named"
  ),

  #----trade_cancel_all_after----
  trade_cancel_all_after = list(
    okx_path     = "/api/v5/trade/cancel-all-after",
    parser_schema = data.frame(
      okx    = c("triggerTime", "tag", "ts"),
      formal = c("Trigger time", "Cancel-all-after tag", "Request time"),
      type   = c("numeric", "string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_order_precheck----
  trade_order_precheck = list(
    okx_path     = "/api/v5/trade/order-precheck",
    parser_schema = data.frame(
      okx    = c("adjEq", "adjEqChg", "availBal", "availBalChg", "imr", "imrChg", "liab", "liabChg", "liabChgCcy", "liqPx", "liqPxDiff", "liqPxDiffRatio", "mgnRatio", "mgnRatioChg", "mmr", "mmrChg", "posBal", "posBalChg", "type"),
      formal = c("Adjusted equity", "Adjusted equity change", "Available balance", "Available balance change", "Initial margin requirement", "Initial margin requirement change", "Liability", "Liability change", "Liability change currency", "Liquidation price", "Liquidation price change", "Liquidation price change ratio", "Margin ratio", "Margin ratio change", "Maintenance margin requirement", "Maintenance margin requirement change", "Position balance", "Position balance change", "Type"),
      type   = c("numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_mass_cancel----
  trade_mass_cancel = list(
    okx_path     = "/api/v5/trade/mass-cancel",
    parser_schema = data.frame(
      okx    = c("result"),
      formal = c("Result"),
      type   = c("logical"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_cancel_algos----
  trade_cancel_algos = list(
    okx_path     = "/api/v5/trade/cancel-algos",
    parser_schema = .trade_algo_cancel_response_schema,
    parser_mode = "named"
  ),

  #----trade_amend_algos----
  trade_amend_algos = list(
    okx_path     = "/api/v5/trade/amend-algos",
    parser_schema = .trade_algo_amend_response_schema,
    parser_mode = "named"
  )
  
  
)

#----.API_GET_SPECS----

#' OKX POST endpoint specifications (internal)
#'
#' A named list describing how to call and parse selected **POST** endpoints.
#'
#' Each entry has:
#' - `okx_path` (character): REST path beginning with `/api/...`.
#' - `parser_schema` (`data.frame`): three columns
#'   - `okx`: field name as returned by OKX
#'   - `formal`: human-readable label
#'   - `type`: one of `"string"`, `"numeric"`, `"integer"`, `"time"`
#' - `parser_mode` (character): either `"named"` or `"positional"`.
#'
#' @section Parser modes:
#' - **named**: parse by JSON key (robust to column order).
#' - **positional**: parse by array position (used by endpoints returning
#'   vectors/arrays such as candles; order is critical).
#'
#' @section Included endpoints:
#' - `account_set_leverage`: `/api/v5/account/set-leverage`
#' - `trade_order`: `/api/v5/trade/order`
#' - `trade_cancel_order`: `/api/v5/trade/cancel-order`
#' - `trade_close_position`: `/api/v5/trade/close-position`
#'
#' @examples
#' # Access path for placing an order
#' .api_POST_specs$trade_order$okx_path
#'
#' # Inspect the expected fields for the cancel-order response
#' .api_POST_specs$trade_cancel_order$parser_schema
#'
#' @format A named list of endpoint specification lists.
#' @seealso [`.api_GET_specs`]
#' @family okxr-internal
#' @note Since okxr 0.1.1
#' @keywords internal
#' @noRd
.api_GET_specs <- list(
  
  #----trade_order----
  trade_order = list(
    okx_path     = "/api/v5/trade/order",
    parser_schema       = data.frame(
      okx    = c("cTime", "ordId", "clOrdId", "tag", "instId", "ordType", "px", "sz", "side", "posSide", "tdMode", "accFillSz", "fillPx", "fillSz", "fillTime", "avgPx", "state", "lever"),
      formal = c("Creation time", "Order ID", "Client Order ID", "Order tag", "Instrument ID", "Order type", "Price", "Quantity to buy or sell", "Order side", "Position side", "Trade mode", "Accumulated fill quantity", "Last filled price", "Last filled quantity", "Last filled time", "Average filled price", "State", "Leverage"),
      type   = c("time", "string", "string", "string", "string", "string", "numeric", "numeric", "string", "string", "string", "numeric", "numeric", "numeric", "time", "numeric", "string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----trade_orders_pending----
  trade_orders_pending = list(
    okx_path     = "/api/v5/trade/orders-pending",
    parser_schema       = data.frame(
      okx    = c("cTime", "ordId", "clOrdId", "tag", "instId", "ordType", "px", "sz", "side", "posSide", "tdMode", "accFillSz", "fillPx", "fillSz", "fillTime", "avgPx", "state", "lever"),
      formal = c("Creation time", "Order ID", "Client Order ID", "Order tag", "Instrument ID", "Order type", "Price", "Quantity to buy or sell", "Order side", "Position side", "Trade mode", "Accumulated fill quantity", "Last filled price", "Last filled quantity", "Last filled time", "Average filled price", "State", "Leverage"),
      type   = c("time", "string", "string", "string", "string", "string", "numeric", "numeric", "string", "string", "string", "numeric", "numeric", "numeric", "time", "numeric", "string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----trade_orders_history_7d----
  trade_orders_history_7d = list(
    okx_path     = "/api/v5/trade/orders-history",
    parser_schema       = data.frame(
      okx    = c("cTime", "ordId", "clOrdId", "tag", "instId", "ordType", "px", "sz", "side", "posSide", "tdMode", "accFillSz", "fillPx", "fillSz", "fillTime", "avgPx", "state", "lever"),
      formal = c("Creation time", "Order ID", "Client Order ID", "Order tag", "Instrument ID", "Order type", "Price", "Quantity to buy or sell", "Order side", "Position side", "Trade mode", "Accumulated fill quantity", "Last filled price", "Last filled quantity", "Last filled time", "Average filled price", "State", "Leverage"),
      type   = c("time", "string", "string", "string", "string", "string", "numeric", "numeric", "string", "string", "string", "numeric", "numeric", "numeric", "time", "numeric", "string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_fills----
  trade_fills = list(
    okx_path     = "/api/v5/trade/fills",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "tradeId", "ordId", "clOrdId", "billId", "subType", "tag", "fillPx", "fillSz", "fillIdxPx", "fillPnl", "fillPxVol", "fillPxUsd", "fee", "feeCcy", "ts"),
      formal = c("Instrument type", "Instrument ID", "Trade ID", "Order ID", "Client Order ID", "Bill ID", "Transaction type", "Order tag", "Filled price", "Filled size", "Index price at fill", "Filled profit and loss", "Filled implied volatility", "Filled option price in USD", "Fee", "Fee currency", "Trade time"),
      type   = c("string", "string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_fills_history----
  trade_fills_history = list(
    okx_path     = "/api/v5/trade/fills-history",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "tradeId", "ordId", "clOrdId", "billId", "subType", "tag", "fillPx", "fillSz", "fillIdxPx", "fillPnl", "fillPxVol", "fillPxUsd", "fee", "feeCcy", "ts"),
      formal = c("Instrument type", "Instrument ID", "Trade ID", "Order ID", "Client Order ID", "Bill ID", "Transaction type", "Order tag", "Filled price", "Filled size", "Index price at fill", "Filled profit and loss", "Filled implied volatility", "Filled option price in USD", "Fee", "Fee currency", "Trade time"),
      type   = c("string", "string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----trade_account_rate_limit----
  trade_account_rate_limit = list(
    okx_path     = "/api/v5/trade/account-rate-limit",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("accRateLimit", "fillRatio", "mainFillRatio", "nextAccRateLimit", "ts"),
      formal = c("Account rate limit", "Sub-account fill ratio", "Main-account fill ratio", "Next account rate limit", "Request time"),
      type   = c("integer", "numeric", "numeric", "integer", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----copy_trade_settings----
  copy_trade_settings = list(
    okx_path     = "/api/v5/copytrading/copy-settings",
    parser_schema       = data.frame(
      okx    = c("ccy", "copyAmt", "copyInstIdType", "copyMgnMode", "copyMode", "copyRatio", "copyState", "copyTotalAmt", "instIds", "slRatio", "slTotalAmt", "subPosCloseType", "tpRatio", "tag"),
      formal = c("Margin currency", "Copy amount", "Copy instrument selection type", "Copy margin mode", "Copy mode", "Copy ratio", "Current copy state", "Maximum total copy amount", "Instrument list", "Stop-loss ratio", "Total stop-loss amount", "Subposition close type", "Take-profit ratio", "Order tag"),
      type   = c("string", "numeric", "string", "string", "string", "numeric", "string", "numeric", "string", "numeric", "numeric", "string", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----copy_trade_my_leaders----
  copy_trade_my_leaders = list(
    okx_path     = "/api/v5/copytrading/current-lead-traders",
    parser_schema       = data.frame(
      okx    = c("nickName", "uniqueCode"),
      formal = c("Nick name", "Lead trader unique code"),
      type   = c("string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----copy_trade_current_subpos----
  copy_trade_current_subpos = list(
    okx_path     = "/api/v5/copytrading/current-subpositions",
    parser_schema       = data.frame(
      okx    = c("algoId", "availSubPos", "ccy", "instId", "instType", "lever", "margin", "markPx", "mgnMode", "openAvgPx", "openOrdId", "openTime", "posSide", "slOrdPx", "slTriggerPx", "subPos", "subPosId", "tpOrdPx", "tpTriggerPx", "uniqueCode", "upl", "uplRatio"),
      formal = c("Stop order ID", "Available subposition size", "Margin currency", "Instrument ID", "Instrument type", "Leverage", "Margin", "Mark price", "Margin mode", "Average open price", "Opening order ID", "Open time", "Position side", "Stop-loss order price", "Stop-loss trigger price", "Subposition size", "Subposition ID", "Take-profit order price", "Take-profit trigger price", "Lead trader unique code", "Unrealized profit and loss", "Unrealized profit and loss ratio"),
      type   = c("string", "numeric", "string", "string", "string", "numeric", "numeric", "numeric", "string", "numeric", "string", "time", "string", "numeric", "numeric", "numeric", "string", "numeric", "numeric", "string", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----copy_trade_historical_subpos----
  copy_trade_historical_subpos = list(
    okx_path     = "/api/v5/copytrading/subpositions-history",
    parser_schema       = data.frame(
      okx    = c("ccy", "closeAvgPx", "closeSubPos", "closeTime", "instId", "instType", "lever", "margin", "markPx", "mgnMode", "openAvgPx", "openOrdId", "openTime", "pnl", "pnlRatio", "posSide", "profitSharingAmt", "subPos", "subPosId", "type", "uniqueCode"),
      formal = c("Margin currency", "Average close price", "Closed subposition size", "Close time", "Instrument ID", "Instrument type", "Leverage", "Margin", "Mark price", "Margin mode", "Average open price", "Opening order ID", "Open time", "Profit and loss", "Profit and loss ratio", "Position side", "Profit sharing amount", "Subposition size", "Subposition ID", "Close type", "Lead trader unique code"),
      type   = c("string", "numeric", "numeric", "time", "string", "string", "numeric", "numeric", "numeric", "string", "numeric", "string", "time", "numeric", "numeric", "string", "numeric", "numeric", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_instruments----
  copy_trade_instruments = list(
    okx_path     = "/api/v5/copytrading/instruments",
    parser_schema = data.frame(
      okx    = c("instId", "enabled"),
      formal = c("Instrument ID", "Copy trading enabled"),
      type   = c("string", "logical"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_config----
  copy_trade_config = list(
    okx_path     = "/api/v5/copytrading/config",
    parser_schema = data.frame(
      okx    = c("uniqueCode", "nickName", "portLink", "details"),
      formal = c("Lead trader unique code", "Nick name", "Portfolio link", "Copy trading configuration details"),
      type   = c("string", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_config----
  copy_trade_public_config = list(
    okx_path     = "/api/v5/copytrading/public-config",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("maxCopyAmt", "minCopyAmt", "maxCopyTotalAmt", "minCopyRatio", "maxCopyRatio", "maxTpRatio", "maxSlRatio"),
      formal = c("Maximum copy amount", "Minimum copy amount", "Maximum total copy amount", "Minimum copy ratio", "Maximum copy ratio", "Maximum take-profit ratio", "Maximum stop-loss ratio"),
      type   = c("numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_copy_traders----
  copy_trade_public_copy_traders = list(
    okx_path     = "/api/v5/copytrading/public-copy-traders",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("ccy", "copyTotalPnl", "copyTraderNumChg", "copyTraderNumChgRatio", "copyTraders"),
      formal = c("Profit currency", "Total copy trader profit and loss", "Copy trader count change", "Copy trader count change ratio", "Copy trader details"),
      type   = c("string", "numeric", "integer", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_lead_traders----
  copy_trade_public_lead_traders = list(
    okx_path     = "/api/v5/copytrading/public-lead-traders",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("dataVer", "totalPage", "ranks"),
      formal = c("Data version", "Total pages", "Lead trader ranks"),
      type   = c("string", "integer", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_current_subpositions----
  copy_trade_public_current_subpositions = list(
    okx_path     = "/api/v5/copytrading/public-current-subpositions",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("ccy", "instId", "instType", "lever", "margin", "markPx", "mgnMode", "openAvgPx", "openTime", "posSide", "subPos", "subPosId", "uniqueCode", "upl", "uplRatio"),
      formal = c("Margin currency", "Instrument ID", "Instrument type", "Leverage", "Margin", "Mark price", "Margin mode", "Average open price", "Open time", "Position side", "Subposition size", "Subposition ID", "Lead trader unique code", "Unrealized profit and loss", "Unrealized profit and loss ratio"),
      type   = c("string", "string", "string", "numeric", "numeric", "numeric", "string", "numeric", "time", "string", "numeric", "string", "string", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_subpositions_history----
  copy_trade_public_subpositions_history = list(
    okx_path     = "/api/v5/copytrading/public-subpositions-history",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("ccy", "closeAvgPx", "closeTime", "instId", "instType", "lever", "margin", "mgnMode", "openAvgPx", "openTime", "pnl", "pnlRatio", "posSide", "subPos", "subPosId", "uniqueCode"),
      formal = c("Margin currency", "Average close price", "Close time", "Instrument ID", "Instrument type", "Leverage", "Margin", "Margin mode", "Average open price", "Open time", "Profit and loss", "Profit and loss ratio", "Position side", "Subposition size", "Subposition ID", "Lead trader unique code"),
      type   = c("string", "numeric", "time", "string", "string", "numeric", "numeric", "string", "numeric", "time", "numeric", "numeric", "string", "numeric", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_pnl----
  copy_trade_public_pnl = list(
    okx_path     = "/api/v5/copytrading/public-pnl",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("beginTs", "pnl", "pnlRatio"),
      formal = c("Window start time", "Profit and loss", "Profit and loss ratio"),
      type   = c("time", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_stats----
  copy_trade_public_stats = list(
    okx_path     = "/api/v5/copytrading/public-stats",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("avgSubPosNotional", "ccy", "curCopyTraderPnl", "investAmt", "lossDays", "profitDays", "winRatio"),
      formal = c("Average subposition notional", "Quote currency", "Current copy trader profit and loss", "Invested amount", "Loss days", "Profit days", "Win ratio"),
      type   = c("numeric", "string", "numeric", "numeric", "integer", "integer", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_weekly_pnl----
  copy_trade_public_weekly_pnl = list(
    okx_path     = "/api/v5/copytrading/public-weekly-pnl",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("beginTs", "pnl", "pnlRatio"),
      formal = c("Week start time", "Profit and loss", "Profit and loss ratio"),
      type   = c("time", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_public_preference_currency----
  copy_trade_public_preference_currency = list(
    okx_path     = "/api/v5/copytrading/public-preference-currency",
    auth         = FALSE,
    parser_schema = data.frame(
      okx    = c("ccy", "ratio"),
      formal = c("Currency", "Preference ratio"),
      type   = c("string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_profit_sharing_details----
  copy_trade_profit_sharing_details = list(
    okx_path     = "/api/v5/copytrading/profit-sharing-details",
    parser_schema = data.frame(
      okx    = c("ccy", "nickName", "profitSharingAmt", "profitSharingId", "portLink", "ts", "instType"),
      formal = c("Settlement currency", "Nick name", "Profit sharing amount", "Profit sharing ID", "Portfolio link", "Settlement time", "Instrument type"),
      type   = c("string", "string", "numeric", "string", "string", "time", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_unrealized_profit_sharing_details----
  copy_trade_unrealized_profit_sharing_details = list(
    okx_path     = "/api/v5/copytrading/unrealized-profit-sharing-details",
    parser_schema = data.frame(
      okx    = c("ccy", "nickName", "portLink", "ts", "unrealizedProfitSharingAmt", "instType"),
      formal = c("Settlement currency", "Nick name", "Portfolio link", "Observation time", "Unrealized profit sharing amount", "Instrument type"),
      type   = c("string", "string", "string", "time", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_total_profit_sharing----
  copy_trade_total_profit_sharing = list(
    okx_path     = "/api/v5/copytrading/total-profit-sharing",
    parser_schema = data.frame(
      okx    = c("ccy", "totalProfitSharingAmt", "instType"),
      formal = c("Settlement currency", "Total profit sharing amount", "Instrument type"),
      type   = c("string", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----copy_trade_total_unrealized_profit_sharing----
  copy_trade_total_unrealized_profit_sharing = list(
    okx_path     = "/api/v5/copytrading/total-unrealized-profit-sharing",
    parser_schema = data.frame(
      okx    = c("profitSharingTs", "totalUnrealizedProfitSharingAmt"),
      formal = c("Profit sharing snapshot time", "Total unrealized profit sharing amount"),
      type   = c("time", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----asset_balances----
  asset_balances = list(
    okx_path     = "/api/v5/asset/balances",
    parser_schema       = data.frame(
      okx    = c("bal", "availBal", "frozenBal", "ccy"),
      formal = c("Balance", "Available", "Frozen", "Currency"),
      type   = c("numeric", "numeric", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----asset_deposit_history----
  asset_deposit_history = list(
    okx_path     = "/api/v5/asset/deposit-history",
    parser_schema       = data.frame(
      okx    = c("actualDepBlkConfirm", "amt", "areaCodeFrom", "ccy", "chain", "depId", "from", "fromWdId", "state", "to", "ts", "txId"),
      formal = c("Actual deposit block confirmations", "Amount", "Origin area code", "Currency", "Chain", "Deposit ID", "Origin address", "Source withdrawal ID", "Deposit state", "Destination address", "Timestamp", "Transaction hash"),
      type   = c("numeric", "numeric", "string", "string", "string", "string", "string", "string", "string", "string", "time", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----asset_withdrawal_history----
  asset_withdrawal_history = list(
    okx_path     = "/api/v5/asset/withdrawal-history",
    parser_schema       = data.frame(
      okx    = c("note", "chain", "fee", "feeCcy", "ccy", "clientId", "toAddrType", "amt", "txId", "from", "areaCodeFrom", "to", "areaCodeTo", "state", "ts", "nonTradableAsset", "wdId"),
      formal = c("Withdrawal note", "Chain", "Withdrawal fee", "Fee currency", "Currency", "Client ID", "Destination address type", "Amount", "Transaction hash", "Origin address", "Origin area code", "Destination address", "Destination area code", "Withdrawal state", "Timestamp", "Non-tradable asset", "Withdrawal ID"),
      type   = c("string", "string", "numeric", "string", "string", "string", "string", "numeric", "string", "string", "string", "string", "string", "string", "time", "logical", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----asset_currencies----
  asset_currencies = list(
    okx_path     = "/api/v5/asset/currencies",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("ccy", "name", "chain", "ctAddr", "canDep", "canWd", "canInternal", "minDep", "minWd", "maxWd", "wdTickSz", "depEstOpenTime", "wdEstOpenTime"),
      formal = c("Currency", "Currency name", "Chain", "Contract address", "Deposit available", "Withdrawal available", "Internal transfer available", "Minimum deposit", "Minimum withdrawal", "Maximum withdrawal", "Withdrawal tick size", "Estimated deposit open time", "Estimated withdrawal open time"),
      type   = c("string", "string", "string", "string", "logical", "logical", "logical", "numeric", "numeric", "numeric", "numeric", "time", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----asset_deposit_address----
  asset_deposit_address = list(
    okx_path     = "/api/v5/asset/deposit-address",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("ccy", "chain", "addr", "selected", "to", "memo", "tag", "pmtId", "addrEx"),
      formal = c("Currency", "Chain", "Deposit address", "Selected address", "Address owner", "Memo", "Tag", "Payment ID", "Address extension"),
      type   = c("string", "string", "string", "logical", "string", "string", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----account_instruments----
  account_instruments = list(
    okx_path = "/api/v5/account/instruments",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "uly", "instFamily", "baseCcy", "quoteCcy", "settleCcy", "ctVal", "ctMult", "ctValCcy", "listTime", "openType", "expTime", "lever", "tickSz", "lotSz", "minSz", "ctType", "state"),
      formal = c("Instrument type", "Instrument ID", "Underlying", "Instrument family", "Base currency", "Quote currency", "Settlement and margin currency", "Contract value", "Contract multiplier", "Contract value currency", "Listing time", "Open type", "Expiry time", "Max Leverage", "Tick size", "Lot size", "Minimum order size", "Contract type", "Instrument status"),
      type   = c("string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric", "time", "string", "time", "numeric", "numeric", "numeric", "numeric", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_position_risk----
  account_position_risk = list(
    okx_path = "/api/v5/account/account-position-risk",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("adjEq", "balData", "posData", "ts"),
      formal = c("Adjusted equity", "Balance snapshot data", "Position snapshot data", "Snapshot time"),
      type   = c("numeric", "string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_max_size----
  account_max_size = list(
    okx_path = "/api/v5/account/max-size",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "ccy", "maxBuy", "maxSell"),
      formal = c("Instrument ID", "Margin currency", "Maximum buy size", "Maximum sell size"),
      type   = c("string", "string", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_max_avail_size----
  account_max_avail_size = list(
    okx_path = "/api/v5/account/max-avail-size",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "availBuy", "availSell"),
      formal = c("Instrument ID", "Maximum available buy amount", "Maximum available sell amount"),
      type   = c("string", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_trade_fee----
  account_trade_fee = list(
    okx_path = "/api/v5/account/trade-fee",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("level", "feeGroup", "delivery", "exercise", "instType", "ts", "taker", "maker", "takerU", "makerU", "takerUSDC", "makerUSDC", "ruleType", "category", "fiat", "settle"),
      formal = c("Fee rate level", "Fee groups", "Delivery fee rate", "Exercise fee rate", "Instrument type", "Data return time", "Taker fee rate", "Maker fee rate", "USDT-margined taker fee rate", "USDT-margined maker fee rate", "USDC or USD stablecoin taker fee rate", "USDC or USD stablecoin maker fee rate", "Trading rule type", "Currency category", "Deprecated fiat fee detail", "Settlement fee rate"),
      type   = c("string", "string", "numeric", "numeric", "string", "time", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "string", "string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_interest_rate----
  account_interest_rate = list(
    okx_path = "/api/v5/account/interest-rate",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("interestRate", "ccy"),
      formal = c("Hourly borrowing interest rate", "Currency"),
      type   = c("numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_subtypes----
  account_subtypes = list(
    okx_path = "/api/v5/account/subtypes",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("type", "typeDesc", "subTypeDetails"),
      formal = c("Bill type", "Bill type description", "Sub-type details"),
      type   = c("string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_adjust_leverage_info----
  account_adjust_leverage_info = list(
    okx_path = "/api/v5/account/adjust-leverage-info",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("estAvailQuoteTrans", "estAvailTrans", "estLiqPx", "estMaxAmt", "estMgn", "estQuoteMaxAmt", "estQuoteMgn", "existOrd", "maxLever", "minLever"),
      formal = c("Estimated transferable quote margin", "Estimated transferable margin", "Estimated liquidation price", "Estimated maximum amount", "Estimated margin", "Estimated maximum quote amount", "Estimated quote margin", "Existing pending orders", "Maximum leverage", "Minimum leverage"),
      type   = c("numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "logical", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_max_loan----
  account_max_loan = list(
    okx_path = "/api/v5/account/max-loan",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "mgnMode", "mgnCcy", "maxLoan", "ccy", "side"),
      formal = c("Instrument ID", "Margin mode", "Margin currency", "Maximum loan", "Currency", "Order side"),
      type   = c("string", "string", "string", "numeric", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_interest_accrued----
  account_interest_accrued = list(
    okx_path = "/api/v5/account/interest-accrued",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("type", "ccy", "instId", "mgnMode", "interest", "interestRate", "liab", "totalLiab", "interestFreeLiab", "ts"),
      formal = c("Loan type", "Loan currency", "Instrument ID", "Margin mode", "Interest accrued", "Hourly borrowing interest rate", "Liability", "Total liability", "Interest-free liability", "Timestamp"),
      type   = c("string", "string", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_max_withdrawal----
  account_max_withdrawal = list(
    okx_path = "/api/v5/account/max-withdrawal",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ccy", "maxWd", "maxWdEx", "spotOffsetMaxWd", "spotOffsetMaxWdEx"),
      formal = c("Currency", "Maximum withdrawal", "Maximum withdrawal including borrowed assets", "Spot offset maximum withdrawal", "Spot offset maximum withdrawal including borrowed assets"),
      type   = c("string", "numeric", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_risk_state----
  account_risk_state = list(
    okx_path = "/api/v5/account/risk-state",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("atRisk", "atRiskIdx", "atRiskMgn", "ts"),
      formal = c("Account at risk", "Derivatives risk units", "Margin risk units", "Timestamp"),
      type   = c("logical", "string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_interest_limits----
  account_interest_limits = list(
    okx_path = "/api/v5/account/interest-limits",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("debt", "interest", "loanAlloc", "nextDiscountTime", "nextInterestTime", "records"),
      formal = c("Current debt in USD", "Current interest in USD", "VIP loan allocation", "Next discount time", "Next accrual time", "Per-currency loan records"),
      type   = c("numeric", "numeric", "numeric", "time", "time", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_greeks----
  account_greeks = list(
    okx_path = "/api/v5/account/greeks",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("thetaBS", "thetaPA", "deltaBS", "deltaPA", "gammaBS", "gammaPA", "vegaBS", "vegaPA", "ccy", "ts"),
      formal = c("Black-Scholes theta", "Coin theta", "Black-Scholes delta", "Coin delta", "Black-Scholes gamma", "Coin gamma", "Black-Scholes vega", "Coin vega", "Currency", "Timestamp"),
      type   = c("numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_position_tiers----
  account_position_tiers = list(
    okx_path = "/api/v5/account/position-tiers",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("uly", "instFamily", "maxSz", "posType"),
      formal = c("Underlying", "Instrument family", "Maximum position size", "Position limitation type"),
      type   = c("string", "string", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_collateral_assets----
  account_collateral_assets = list(
    okx_path = "/api/v5/account/collateral-assets",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ccy", "collateralEnabled"),
      formal = c("Currency", "Collateral enabled"),
      type   = c("string", "logical"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_mmp_config----
  account_mmp_config = list(
    okx_path = "/api/v5/account/mmp-config",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instFamily", "mmpFrozen", "mmpFrozenUntil", "timeInterval", "frozenInterval", "qtyLimit"),
      formal = c("Instrument family", "MMP frozen", "MMP frozen until", "Monitoring interval", "Frozen interval", "Quantity limit"),
      type   = c("string", "logical", "numeric", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_move_positions_history----
  account_move_positions_history = list(
    okx_path = "/api/v5/account/move-positions-history",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("clientId", "blockTdId", "state", "ts", "fromAcct", "toAcct", "legs"),
      formal = c("Client-supplied ID", "Block trade ID", "Transfer state", "Processed time", "Source account", "Destination account", "Position transfer legs"),
      type   = c("string", "string", "string", "time", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_precheck_set_delta_neutral----
  account_precheck_set_delta_neutral = list(
    okx_path = "/api/v5/account/precheck-set-delta-neutral",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("unmatchedInfoCheck"),
      formal = c("Delta-neutral unmatched information"),
      type   = c("string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_bills_history_archive----
  account_bills_history_archive = list(
    okx_path = "/api/v5/account/bills-history-archive",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("fileHref", "state", "ts"),
      formal = c("Download file link", "Download link status", "Request time"),
      type   = c("string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_subaccount_balances----
  account_subaccount_balances = list(
    okx_path = "/api/v5/account/subaccount/balances",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("uTime", "totalEq", "isoEq", "adjEq", "availEq", "ordFroz", "imr", "mmr", "borrowFroz", "mgnRatio", "notionalUsd", "notionalUsdForBorrow", "notionalUsdForSwap", "notionalUsdForFutures", "notionalUsdForOption", "upl", "delta", "deltaLever", "deltaNeutralStatus", "details"),
      formal = c("Update time", "Total equity", "Isolated margin equity", "Adjusted / Effective equity", "Available equity", "Frozen order margin", "Initial margin requirement", "Maintenance margin requirement", "Potential borrowing IMR", "Maintenance margin ratio", "Notional USD", "Notional USD for borrow", "Notional USD for swaps", "Notional USD for futures", "Notional USD for options", "Unrealized profit and loss", "Delta", "Delta leverage", "Delta neutral status", "Per-currency balance details"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_subaccount_max_withdrawal----
  account_subaccount_max_withdrawal = list(
    okx_path = "/api/v5/account/subaccount/max-withdrawal",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ccy", "maxWd", "maxWdEx", "spotOffsetMaxWd", "spotOffsetMaxWdEx"),
      formal = c("Currency", "Maximum withdrawal", "Maximum withdrawal including borrowed assets", "Spot offset maximum withdrawal", "Spot offset maximum withdrawal including borrowed assets"),
      type   = c("string", "numeric", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_set_account_switch_precheck----
  account_set_account_switch_precheck = list(
    okx_path = "/api/v5/account/set-account-switch-precheck",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("acctLv", "curAcctLv", "mgnAft", "mgnBf", "posList", "posTierCheck", "riskOffsetType", "sCode", "unmatchedInfoCheck"),
      formal = c("Target account mode", "Current account mode", "Margin after switch", "Margin before switch", "Position list", "Position tier check", "Risk offset type", "Status code", "Unmatched information"),
      type   = c("string", "string", "string", "string", "string", "string", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_spot_borrow_repay_history----
  account_spot_borrow_repay_history = list(
    okx_path = "/api/v5/account/spot-borrow-repay-history",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ccy", "type", "amt", "accBorrowed", "ts"),
      formal = c("Currency", "Event type", "Amount", "Accumulated borrowed amount", "Timestamp"),
      type   = c("string", "string", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_balance----
  account_balance = list(
    okx_path     = "/api/v5/account/balance",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("uTime", "totalEq", "isoEq", "adjEq", "availEq", "ordFroz", "imr", "mmr", "borrowFroz", "mgnRatio", "notionalUsd", "notionalUsdForBorrow", "notionalUsdForSwap", "notionalUsdForFutures", "notionalUsdForOption", "upl"),
      formal = c("Update time", "Total equity", "Isolated margin equity", "Adjusted / Effective equity", "Account level available equity", "Cross margin frozen for pending orders", "Initial margin requirement", "Maintenance margin requirement", "Potential borrowing IMR of the account", "Maintenance margin ratio", "Notional value of positions", "Notional value for Borrow", "Notional value of positions for Perpetual Futures", "Notional value of positions for Expiry Futures", "Notional value of positions for Option", "Cross-margin info of unrealized profit and loss at the account level"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----account_positions----
  account_positions = list(
    okx_path     = "/api/v5/account/positions",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("cTime", "uTime", "instId", "lever", "posId", "posSide", "pos", "imr", "mmr", "avgPx", "markPx", "upl", "realizedPnl", "settledPnl", "pnl", "fee", "fundingFee", "liqPenalty"),
      formal = c("Creation time", "Update time", "Instrument ID", "Leverage", "Position ID", "Position side", "Quantity of positions", "Initial margin requirement", "Maintenance margin requirement", "Average open price", "Latest Mark price", "Unrealized profit and loss", "Realized profit and loss", "Accumulated settled profit and loss", "Accumulated pnl of closing order(s)", "Accumulated fee", "Accumulated funding fee", "	Accumulated liquidation penalty"),
      type   = c("time", "time", "string", "numeric", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----account_positions_history----
  account_positions_history = list(
    okx_path     = "/api/v5/account/positions-history",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("cTime", "uTime", "instId", "lever", "posId", "posSide", "pos", "realizedPnl", "fee"),
      formal = c("Creation time", "Update time", "Instrument ID", "Leverage", "Position ID", "Position side", "Quantity of positions", "Realized profit and loss", "Accumulated fee"),
      type   = c("time", "time", "string", "numeric", "string", "string", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----account_config----
  account_config = list(
    okx_path     = "/api/v5/account/config",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("uid", "mainUid", "acctLv", "posMode", "autoLoan"),
      formal = c("Account ID", "Main Account ID", "Account mode", "Position mode", "Whether to borrow coins automatically"),
      type   = c("string", "string", "string", "string", "logical"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----account_leverage_info----
  account_leverage_info = list(
    okx_path     = "/api/v5/account/leverage-info",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("instId", "mgnMode", "posSide", "lever"),
      formal = c("Instrument ID", "Margin mode", "Position side", "Leverage"),
      type   = c("string", "string", "string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_bills----
  account_bills = list(
    okx_path     = "/api/v5/account/bills",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("billId", "instType", "instId", "ccy", "mgnMode", "type", "subType", "bal", "balChg", "posBal", "posBalChg", "sz", "px", "fee", "pnl", "ordId", "from", "to", "ts"),
      formal = c("Bill ID", "Instrument type", "Instrument ID", "Currency", "Margin mode", "Bill type", "Bill subtype", "Balance", "Balance change", "Position balance", "Position balance change", "Quantity", "Price", "Fee", "Profit and loss", "Order ID", "Transfer from", "Transfer to", "Bill time"),
      type   = c("string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----account_bills_archive----
  account_bills_archive = list(
    okx_path     = "/api/v5/account/bills-archive",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("billId", "instType", "instId", "ccy", "mgnMode", "type", "subType", "bal", "balChg", "posBal", "posBalChg", "sz", "px", "fee", "pnl", "ordId", "from", "to", "ts"),
      formal = c("Bill ID", "Instrument type", "Instrument ID", "Currency", "Margin mode", "Bill type", "Bill subtype", "Balance", "Balance change", "Position balance", "Position balance change", "Quantity", "Price", "Fee", "Profit and loss", "Order ID", "Transfer from", "Transfer to", "Bill time"),
      type   = c("string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "string", "string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----market_ticker----
  market_ticker = list(
    okx_path     = "/api/v5/market/ticker",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "last", "askPx", "bidPx", "ts"),
      formal = c("Instrument type", "Instrument ID", "Last traded price", "Best ask price", "Best bid price", "Ticker data generation time"),
      type   = c("string", "string", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_tickers----
  market_tickers = list(
    okx_path     = "/api/v5/market/tickers",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "last", "lastSz", "askPx", "askSz", "bidPx", "bidSz", "open24h", "high24h", "low24h", "volCcy24h", "vol24h", "sodUtc0", "sodUtc8", "ts"),
      formal = c("Instrument type", "Instrument ID", "Last traded price", "Last traded size", "Best ask price", "Best ask size", "Best bid price", "Best bid size", "Open price in past 24 hours", "Highest price in past 24 hours", "Lowest price in past 24 hours", "24h volume in currency", "24h volume", "UTC 0 open price", "UTC 8 open price", "Ticker data generation time"),
      type   = c("string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_books----
  market_books = list(
    okx_path     = "/api/v5/market/books",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("asks", "bids", "ts"),
      formal = c("Ask levels", "Bid levels", "Order book generation time"),
      type   = c("string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_trades----
  market_trades = list(
    okx_path     = "/api/v5/market/trades",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("instId", "tradeId", "px", "sz", "side", "source", "ts"),
      formal = c("Instrument ID", "Trade ID", "Trade price", "Trade quantity", "Taker side", "Order source", "Trade time"),
      type   = c("string", "string", "numeric", "numeric", "string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_history_trades----
  market_history_trades = list(
    okx_path     = "/api/v5/market/history-trades",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("instId", "tradeId", "px", "sz", "side", "source", "ts"),
      formal = c("Instrument ID", "Trade ID", "Trade price", "Trade quantity", "Taker side", "Order source", "Trade time"),
      type   = c("string", "string", "numeric", "numeric", "string", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_option_instrument_family_trades----
  market_option_instrument_family_trades = list(
    okx_path     = "/api/v5/market/option/instrument-family-trades",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "tradeId", "px", "sz", "side", "ts"),
      formal = c("Instrument ID", "Trade ID", "Trade price", "Trade quantity", "Trade side", "Trade time"),
      type   = c("string", "string", "numeric", "numeric", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_mark_price_candles----
  market_mark_price_candles = list(
    okx_path     = "/api/v5/market/mark-price-candles",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts", "o", "h", "l", "c", "confirm"),
      formal = c("Timestamp", "Open price", "Highest price", "Lowest price", "Close price", "The state of candlesticks"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "integer"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "positional"
  ),

  #----market_history_mark_price_candles----
  market_history_mark_price_candles = list(
    okx_path     = "/api/v5/market/history-mark-price-candles",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts", "o", "h", "l", "c", "confirm"),
      formal = c("Timestamp", "Open price", "Highest price", "Lowest price", "Close price", "The state of candlesticks"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "integer"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "positional"
  ),

  #----market_index_tickers----
  market_index_tickers = list(
    okx_path     = "/api/v5/market/index-tickers",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "idxPx", "high24h", "low24h", "open24h", "sodUtc0", "sodUtc8", "ts"),
      formal = c("Index ID", "Latest index price", "Highest price in the past 24 hours", "Lowest price in the past 24 hours", "Open price in the past 24 hours", "UTC 0 open price", "UTC 8 open price", "Index price update time"),
      type   = c("string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_index_candles----
  market_index_candles = list(
    okx_path     = "/api/v5/market/index-candles",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts", "o", "h", "l", "c", "confirm"),
      formal = c("Timestamp", "Open price", "Highest price", "Lowest price", "Close price", "The state of candlesticks"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "integer"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "positional"
  ),

  #----market_history_index_candles----
  market_history_index_candles = list(
    okx_path     = "/api/v5/market/history-index-candles",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts", "o", "h", "l", "c", "confirm"),
      formal = c("Timestamp", "Open price", "Highest price", "Lowest price", "Close price", "The state of candlesticks"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "integer"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "positional"
  ),

  #----market_exchange_rate----
  market_exchange_rate = list(
    okx_path     = "/api/v5/market/exchange-rate",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("usdCny"),
      formal = c("USD to CNY exchange rate"),
      type   = c("numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_index_components----
  market_index_components = list(
    okx_path     = "/api/v5/market/index-components",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("index", "last", "ts", "components"),
      formal = c("Index", "Latest index price", "Data generation time", "Index components"),
      type   = c("string", "numeric", "time", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_platform_24_volume----
  market_platform_24_volume = list(
    okx_path     = "/api/v5/market/platform-24-volume",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("volCny", "volUsd", "ts"),
      formal = c("24-hour total volume in CNY", "24-hour total volume in USD", "Data return time"),
      type   = c("numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_block_ticker----
  market_block_ticker = list(
    okx_path     = "/api/v5/market/block-ticker",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "volCcy24h", "vol24h", "ts"),
      formal = c("Instrument type", "Instrument ID", "24-hour block volume in currency", "24-hour block volume", "Block ticker time"),
      type   = c("string", "string", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----market_block_tickers----
  market_block_tickers = list(
    okx_path     = "/api/v5/market/block-tickers",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "volCcy24h", "vol24h", "ts"),
      formal = c("Instrument type", "Instrument ID", "24-hour block volume in currency", "24-hour block volume", "Block ticker time"),
      type   = c("string", "string", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----market_candles----
  market_candles = list(
    okx_path     = "/api/v5/market/candles",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("ts", "o", "h", "l", "c", "vol", "volCcy", "volCcyQuote", "confirm"),
      formal = c("Timestamp", "Open price", "Highest price", "Lowest price", "Close price", "Trading volume (contract)", "Trading volume (currency)", "Trading volume (quote currency)", "The state of candlesticks"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "integer"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "positional"
  ),
  
  #----market_history_candles----
  market_history_candles = list(
    okx_path     = "/api/v5/market/history-candles",
    parser_schema       = data.frame(
      check.names = FALSE,
      okx    = c("ts", "o", "h", "l", "c", "vol", "volCcy", "volCcyQuote", "confirm"),
      formal = c("Timestamp", "Open price", "Highest price", "Lowest price", "Close price", "Trading volume (contract)", "Trading volume (currency)", "Trading volume (quote currency)", "The state of candlesticks"),
      type   = c("time", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "integer"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "positional"
  ),
  
  #----public_instruments----
  public_instruments = list(
    okx_path = "/api/v5/public/instruments",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "uly", "instFamily", "baseCcy", "quoteCcy", "settleCcy", "ctVal", "ctMult", "ctValCcy", "listTime", "openType", "expTime", "lever", "tickSz", "lotSz", "minSz", "ctType", "state"),
      formal = c("Instrument type", "Instrument ID", "Underlying", "Instrument family", "Base currency", "Quote currency", "Settlement and margin currency", "Contract value", "Contract multiplier", "Contract value currency", "Listing time", "Open type", "Expiry time", "Max Leverage", "Tick size", "Lot size", "Minimum order size", "Contract type", "Instrument status"),
      type   = c("string", "string", "string", "string", "string", "string", "string", "numeric", "numeric", "numeric", "time", "string", "time", "numeric", "numeric", "numeric", "numeric", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_underlying----
  public_underlying = list(
    okx_path = "/api/v5/public/underlying",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("uly"),
      formal = c("Underlying"),
      type   = c("string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "vector"
  ),

  #----public_mark_price----
  public_mark_price = list(
    okx_path = "/api/v5/public/mark-price",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts", "instId", "markPx"),
      formal = c("Timestamp", "Instrument ID", "Price"),
      type   = c("time", "string", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----public_funding_rate----
  public_funding_rate = list(
    okx_path = "/api/v5/public/funding-rate",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "formulaType", "fundingRate", "fundingTime", "nextFundingTime", "minFundingRate", "maxFundingRate", "interestRate", "impactValue", "premium", "ts"),
      formal = c("Instrument ID", "Formula type", "Current funding rate", "Settlement time", "Forecasted funding time for the next period", "The lower limit of the funding rate", "The upper limit of the funding rate", "Interest rate", "Depth weighted amount", "Premium index", "Data return time"),
      type   = c("string", "string", "numeric", "time", "time", "numeric", "numeric", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----public_funding_rate_history----
  public_funding_rate_history = list(
    okx_path = "/api/v5/public/funding-rate-history",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "formulaType", "fundingRate", "realizedRate", "fundingTime", "method"),
      formal = c("Instrument ID", "Formula type", "Predicted funding rate", "Actual funding rate", "Settlement time", "Funding rate mechanism"),
      type   = c("string", "string", "numeric", "numeric", "time", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),
  
  #----public_open_interest----
  public_open_interest = list(
    okx_path = "/api/v5/public/open-interest",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "oi", "oiCcy", "oiUsd", "ts"),
      formal = c("Instrument ID", "Open interest in number of contracts", "Open interest in number of coin", "Open interest in number of USD", "Data return time"),
      type   = c("string", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_time----
  public_time = list(
    okx_path = "/api/v5/public/time",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts"),
      formal = c("System time"),
      type   = c("time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_price_limit----
  public_price_limit = list(
    okx_path = "/api/v5/public/price-limit",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "buyLmt", "sellLmt", "ts"),
      formal = c("Instrument ID", "Highest buy price", "Lowest sell price", "Data return time"),
      type   = c("string", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_estimated_price----
  public_estimated_price = list(
    okx_path = "/api/v5/public/estimated-price",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "settlePx", "ts"),
      formal = c("Instrument type", "Instrument ID", "Estimated delivery or exercise price", "Data return time"),
      type   = c("string", "string", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_delivery_exercise_history----
  public_delivery_exercise_history = list(
    okx_path = "/api/v5/public/delivery-exercise-history",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts", "details"),
      formal = c("Delivery or exercise time", "Delivery or exercise details"),
      type   = c("time", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_estimated_settlement_info----
  public_estimated_settlement_info = list(
    okx_path = "/api/v5/public/estimated-settlement-info",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "nextSettleTime", "estSettlePx", "ts"),
      formal = c("Instrument ID", "Next settlement time", "Estimated settlement price", "Data return time"),
      type   = c("string", "time", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_settlement_history----
  public_settlement_history = list(
    okx_path = "/api/v5/public/settlement-history",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ts", "details"),
      formal = c("Settlement time", "Settlement details"),
      type   = c("time", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_discount_rate_interest_free_quota----
  public_discount_rate_interest_free_quota = list(
    okx_path = "/api/v5/public/discount-rate-interest-free-quota",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("ccy", "colRes", "collateralRestrict", "amt", "discountLv", "minDiscountRate", "details"),
      formal = c("Currency", "Platform level collateral restriction status", "Deprecated collateral restriction flag", "Interest-free quota", "Deprecated discount level", "Minimum discount rate", "Discount tier details"),
      type   = c("string", "string", "logical", "numeric", "string", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_opt_summary----
  public_opt_summary = list(
    okx_path = "/api/v5/public/opt-summary",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instId", "uly", "delta", "gamma", "vega", "theta", "deltaBS", "gammaBS", "vegaBS", "thetaBS", "lever", "markVol", "bidVol", "askVol", "realVol", "volLv", "fwdPx", "ts"),
      formal = c("Instrument type", "Instrument ID", "Underlying", "Delta", "Gamma", "Vega", "Theta", "Delta in BS mode", "Gamma in BS mode", "Vega in BS mode", "Theta in BS mode", "Leverage", "Mark volatility", "Bid volatility", "Ask volatility", "Realized volatility", "ATM implied volatility", "Forward price", "Data update time"),
      type   = c("string", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_position_tiers----
  public_position_tiers = list(
    okx_path = "/api/v5/public/position-tiers",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("uly", "instFamily", "instId", "tier", "maxSz", "minSz", "maxLever", "imr", "mmr", "optMgnFactor", "quoteMaxLoan", "baseMaxLoan"),
      formal = c("Underlying", "Instrument family", "Instrument ID", "Tier", "Maximum size", "Minimum size", "Maximum leverage", "Initial margin requirement", "Maintenance margin requirement", "Option margin factor", "Quote currency maximum loan", "Base currency maximum loan"),
      type   = c("string", "string", "string", "string", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric", "numeric"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_economic_calendar----
  public_economic_calendar = list(
    okx_path = "/api/v5/public/economic-calendar",
    auth = TRUE,
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("calendarId", "date", "region", "category", "event", "refDate", "actual", "previous", "forecast", "dateSpan", "importance", "uTime", "prevInitial", "ccy", "unit"),
      formal = c("Calendar ID", "Release time", "Region", "Category", "Event", "Reference date", "Actual value", "Previous value", "Forecast value", "Date span flag", "Importance level", "Update time", "Initial previous value", "Currency", "Unit"),
      type   = c("string", "time", "string", "string", "string", "time", "string", "string", "string", "string", "string", "time", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_interest_rate_loan_quota----
  public_interest_rate_loan_quota = list(
    okx_path = "/api/v5/public/interest-rate-loan-quota",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("basic", "vip", "regular", "configCcyList", "config"),
      formal = c("Basic interest-rate table", "VIP interest information", "Regular-user interest information", "Currencies with customized quota configuration", "Customized quota configuration"),
      type   = c("string", "string", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_insurance_fund----
  public_insurance_fund = list(
    okx_path = "/api/v5/public/insurance-fund",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("total", "instFamily", "instType", "details"),
      formal = c("Total security fund balance in USD", "Instrument family", "Instrument type", "Security fund details"),
      type   = c("numeric", "string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_convert_contract_coin----
  public_convert_contract_coin = list(
    okx_path = "/api/v5/public/convert-contract-coin",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("type", "instId", "px", "sz", "unit"),
      formal = c("Convert type", "Instrument ID", "Order price", "Converted quantity", "Currency unit"),
      type   = c("string", "string", "numeric", "numeric", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_instrument_tick_bands----
  public_instrument_tick_bands = list(
    okx_path = "/api/v5/public/instrument-tick-bands",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instType", "instFamily", "tickBand"),
      formal = c("Instrument type", "Instrument family", "Tick size band"),
      type   = c("string", "string", "string"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_premium_history----
  public_premium_history = list(
    okx_path = "/api/v5/public/premium-history",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "premium", "ts"),
      formal = c("Instrument ID", "Premium index", "Data generation time"),
      type   = c("string", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_block_trades----
  public_block_trades = list(
    okx_path = "/api/v5/public/block-trades",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("fillVol", "fwdPx", "groupId", "idxPx", "instId", "markPx", "px", "side", "sz", "tradeId", "ts"),
      formal = c("Implied volatility", "Forward price", "Group RFQ ID", "Index price", "Instrument ID", "Mark price", "Trade price", "Trade side", "Trade quantity", "Trade ID", "Trade time"),
      type   = c("numeric", "numeric", "string", "numeric", "string", "numeric", "numeric", "string", "numeric", "string", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  ),

  #----public_option_trades----
  public_option_trades = list(
    okx_path = "/api/v5/public/option-trades",
    parser_schema = data.frame(
      check.names = FALSE,
      okx    = c("instId", "instFamily", "tradeId", "px", "sz", "side", "optType", "fillVol", "fwdPx", "idxPx", "markPx", "ts"),
      formal = c("Instrument ID", "Instrument family", "Trade ID", "Trade price", "Trade quantity", "Trade side", "Option type", "Implied volatility while trading", "Forward price while trading", "Index price while trading", "Mark price while trading", "Trade time"),
      type   = c("string", "string", "string", "numeric", "numeric", "string", "string", "numeric", "numeric", "numeric", "numeric", "time"),
      stringsAsFactors = FALSE
    ),
    parser_mode = "named"
  )
)

.api_GET_specs$trade_orders_history <- list(
  okx_path = "/api/v5/trade/orders-history",
  parser_schema = .api_GET_specs$trade_orders_history_7d$parser_schema,
  parser_mode = "named"
)

.api_GET_specs$trade_orders_history_archive <- list(
  okx_path = "/api/v5/trade/orders-history-archive",
  parser_schema = .api_GET_specs$trade_orders_history_7d$parser_schema,
  parser_mode = "named"
)

.api_GET_specs$trade_easy_convert_currency_list <- list(
  okx_path = "/api/v5/trade/easy-convert-currency-list",
  parser_schema = data.frame(
    okx = c("fromData", "toCcy"),
    formal = c("Source currency data", "Target currency list"),
    type = c("string", "string"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$trade_easy_convert_history <- list(
  okx_path = "/api/v5/trade/easy-convert-history",
  parser_schema = data.frame(
    okx = c("fromCcy", "fillFromSz", "toCcy", "fillToSz", "acct", "status", "uTime"),
    formal = c("Source currency", "Filled source size", "Target currency", "Filled target size", "Account type", "Conversion status", "Trade time"),
    type = c("string", "numeric", "string", "numeric", "string", "string", "time"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$trade_one_click_repay_currency_list <- list(
  okx_path = "/api/v5/trade/one-click-repay-currency-list",
  parser_schema = data.frame(
    okx = c("debtType", "debtData", "repayData"),
    formal = c("Debt type", "Debt currency data", "Repay currency data"),
    type = c("string", "string", "string"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$trade_one_click_repay_history <- list(
  okx_path = "/api/v5/trade/one-click-repay-history",
  parser_schema = data.frame(
    okx = c("debtCcy", "fillDebtSz", "repayCcy", "fillRepaySz", "status", "uTime"),
    formal = c("Debt currency", "Filled debt size", "Repay currency", "Filled repay size", "Repay status", "Trade time"),
    type = c("string", "numeric", "string", "numeric", "string", "time"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$trade_one_click_repay_currency_list_v2 <- list(
  okx_path = "/api/v5/trade/one-click-repay-currency-list-v2",
  parser_schema = data.frame(
    okx = c("debtData", "repayData"),
    formal = c("Debt currency data", "Repay currency data"),
    type = c("string", "string"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$trade_one_click_repay_history_v2 <- list(
  okx_path = "/api/v5/trade/one-click-repay-history-v2",
  parser_schema = data.frame(
    okx = c("debtCcy", "repayCcyList", "fillDebtSz", "status", "ordIdInfo", "ts"),
    formal = c("Debt currency", "Repay currency list", "Filled debt size", "Repay status", "Repay order details", "Request time"),
    type = c("string", "string", "numeric", "string", "string", "time"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.trade_algo_schema <- data.frame(
  okx = c("algoId", "algoClOrdId", "instType", "instId", "ordType", "state", "side", "posSide", "tdMode", "sz", "cTime", "uTime", "attachAlgoOrds", "linkedOrd", "triggerParams", "failCode"),
  formal = c("Algo order ID", "Client algo order ID", "Instrument type", "Instrument ID", "Algo order type", "Algo order state", "Order side", "Position side", "Trade mode", "Order size", "Creation time", "Update time", "Attached algo orders", "Linked order detail", "Trigger parameters", "Failure code"),
  type = c("string", "string", "string", "string", "string", "string", "string", "string", "string", "numeric", "time", "time", "string", "string", "string", "string"),
  stringsAsFactors = FALSE
)

.api_GET_specs$trade_order_algo <- list(
  okx_path = "/api/v5/trade/order-algo",
  parser_schema = .trade_algo_schema,
  parser_mode = "named"
)

.api_GET_specs$trade_orders_algo_pending <- list(
  okx_path = "/api/v5/trade/orders-algo-pending",
  parser_schema = .trade_algo_schema,
  parser_mode = "named"
)

.api_GET_specs$trade_orders_algo_history <- list(
  okx_path = "/api/v5/trade/orders-algo-history",
  parser_schema = .trade_algo_schema,
  parser_mode = "named"
)

.api_GET_specs$asset_non_tradable_assets <- list(
  okx_path = "/api/v5/asset/non-tradable-assets",
  parser_schema = data.frame(
    okx = c("ccy", "chain", "name", "bal", "canWd", "minWd", "wdAll", "fee", "feeCcy", "burningFeeRate", "needTag", "logoLink", "ctAddr", "wdTickSz"),
    formal = c("Currency", "Chain", "Asset name", "Balance", "Can withdraw", "Minimum withdrawal", "Withdraw-all amount", "Withdrawal fee", "Fee currency", "Burning fee rate", "Requires tag", "Logo URL", "Contract address suffix", "Withdrawal tick size"),
    type = c("string", "string", "string", "numeric", "logical", "numeric", "numeric", "numeric", "string", "numeric", "logical", "string", "string", "numeric"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$asset_asset_valuation <- list(
  okx_path = "/api/v5/asset/asset-valuation",
  parser_schema = data.frame(
    okx = c("totalBal", "ts", "details"),
    formal = c("Total balance in valuation currency", "Valuation time", "Valuation details"),
    type = c("numeric", "time", "string"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$asset_transfer_state <- list(
  okx_path = "/api/v5/asset/transfer-state",
  parser_schema = data.frame(
    okx = c("transId", "clientId", "ccy", "amt", "type", "from", "to", "subAcct", "state"),
    formal = c("Transfer ID", "Client ID", "Currency", "Transfer amount", "Transfer type", "From account", "To account", "Sub-account", "Transfer state"),
    type = c("string", "string", "string", "numeric", "string", "string", "string", "string", "string"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.asset_bills_schema <- data.frame(
  okx = c("billId", "ccy", "clientId", "balChg", "bal", "type", "notes", "ts"),
  formal = c("Bill ID", "Currency", "Client ID", "Balance change", "Balance", "Bill type", "Notes", "Creation time"),
  type = c("string", "string", "string", "numeric", "numeric", "string", "string", "time"),
  stringsAsFactors = FALSE
)

.api_GET_specs$asset_bills <- list(
  okx_path = "/api/v5/asset/bills",
  parser_schema = .asset_bills_schema,
  parser_mode = "named"
)

.api_GET_specs$asset_bills_history <- list(
  okx_path = "/api/v5/asset/bills-history",
  parser_schema = .asset_bills_schema,
  parser_mode = "named"
)

.api_GET_specs$asset_deposit_withdraw_status <- list(
  okx_path = "/api/v5/asset/deposit-withdraw-status",
  parser_schema = data.frame(
    okx = c("wdId", "txId", "state", "estCompleteTime"),
    formal = c("Withdrawal ID", "Transaction hash", "Detailed status", "Estimated completion time"),
    type = c("string", "string", "string", "string"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$asset_exchange_list <- list(
  okx_path = "/api/v5/asset/exchange-list",
  auth = FALSE,
  parser_schema = data.frame(
    okx = c("exchName", "exchId"),
    formal = c("Exchange name", "Exchange ID"),
    type = c("string", "string"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$asset_convert_currencies <- list(
  okx_path = "/api/v5/asset/convert/currencies",
  parser_schema = data.frame(
    okx = c("ccy", "min", "max"),
    formal = c("Currency", "Minimum amount (deprecated)", "Maximum amount (deprecated)"),
    type = c("string", "numeric", "numeric"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$asset_convert_currency_pair <- list(
  okx_path = "/api/v5/asset/convert/currency-pair",
  parser_schema = data.frame(
    okx = c("instId", "baseCcy", "baseCcyMax", "baseCcyMin", "quoteCcy", "quoteCcyMax", "quoteCcyMin"),
    formal = c("Currency pair", "Base currency", "Base currency maximum", "Base currency minimum", "Quote currency", "Quote currency maximum", "Quote currency minimum"),
    type = c("string", "string", "numeric", "numeric", "string", "numeric", "numeric"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.api_GET_specs$asset_convert_history <- list(
  okx_path = "/api/v5/asset/convert/history",
  parser_schema = data.frame(
    okx = c("clTReqId", "instId", "side", "fillPx", "baseCcy", "quoteCcy", "fillBaseSz", "state", "tradeId", "fillQuoteSz", "ts"),
    formal = c("Client trade request ID", "Currency pair", "Trade side", "Filled price", "Base currency", "Quote currency", "Filled base size", "Trade state", "Trade ID", "Filled quote size", "Trade time"),
    type = c("string", "string", "string", "numeric", "string", "string", "numeric", "string", "string", "numeric", "time"),
    stringsAsFactors = FALSE
  ),
  parser_mode = "named"
)

.public_GET_spec_names <- c(
  "market_ticker",
  "market_tickers",
  "market_books",
  "market_trades",
  "market_history_trades",
  "market_option_instrument_family_trades",
  "market_mark_price_candles",
  "market_history_mark_price_candles",
  "market_index_tickers",
  "market_index_candles",
  "market_history_index_candles",
  "market_exchange_rate",
  "market_index_components",
  "market_platform_24_volume",
  "market_block_ticker",
  "market_block_tickers",
  "market_candles",
  "market_history_candles",
  "public_instruments",
  "public_underlying",
  "public_estimated_price",
  "public_delivery_exercise_history",
  "public_estimated_settlement_info",
  "public_settlement_history",
  "public_discount_rate_interest_free_quota",
  "public_opt_summary",
  "public_position_tiers",
  "public_mark_price",
  "public_interest_rate_loan_quota",
  "public_insurance_fund",
  "public_convert_contract_coin",
  "public_instrument_tick_bands",
  "public_premium_history",
  "public_block_trades",
  "public_funding_rate",
  "public_funding_rate_history",
  "public_open_interest",
  "public_time",
  "public_price_limit",
  "public_option_trades"
)

.api_GET_specs[.public_GET_spec_names] <- lapply(
  .api_GET_specs[.public_GET_spec_names],
  function(api) {
    api$auth <- FALSE
    api
  }
)

#' Set Account Leverage
#'
#' Sets the leverage level for a specific trading instrument and margin mode.
#'
#' @param inst_id Instrument ID (e.g., \code{"BTC-USDT"}).
#' @param lever Leverage level to apply (as a string or numeric, e.g., \code{"10"}).
#' @param mgn_mode Margin mode: \code{"cross"} or \code{"isolated"}.
#' @param pos_side Optional. Position side: \code{"long"} or \code{"short"}. Required for isolated mode.
#' @param tz Timezone used for any timestamp parsing (default: \code{"Asia/Hong_Kong"}).
#' @param config API credential list with keys `api_key`, `secret_key`,
#'   and `passphrase`.
#'
#' @return A \code{data.frame} with leverage update confirmation (including instrument ID and leverage settings).
#'
#' @export
post_account_set_leverage <- function(inst_id, lever, mgn_mode, pos_side = NULL, tz = .okx_default_tz, config) {
  body <- list(instId = inst_id, lever = lever, mgnMode = mgn_mode)
  if (!is.null(pos_side) && mgn_mode == "isolated") {
    body$posSide <- pos_side
  }
  .posts$account_set_leverage(body_list = body, tz = tz, config = config)
}

#' Set Account Position Mode
#'
#' Set the account position mode.
#'
#' @param pos_mode Position mode. Use `long_short_mode` or `net_mode`.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` confirming the applied position mode.
#' @export
post_account_set_position_mode <- function(
  pos_mode,
  tz = .okx_default_tz,
  config
) {
  .posts$account_set_position_mode(
    body_list = list(posMode = pos_mode),
    tz = tz,
    config = config
  )
}

#' Set Account Fee Type
#'
#' Configure the fee charging mode for spot trading.
#'
#' @param fee_type Fee type string, typically `"0"` or `"1"`.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` confirming the applied fee type.
#' @export
post_account_set_fee_type <- function(fee_type, tz = .okx_default_tz, config) {
  .posts$account_set_fee_type(
    body_list = list(feeType = fee_type),
    tz = tz,
    config = config
  )
}

#' Set Account Greeks Display Type
#'
#' Configure whether Greeks are displayed in PA or BS mode.
#'
#' @param greeks_type Greeks display type, typically `"PA"` or `"BS"`.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` confirming the applied Greeks display type.
#' @export
post_account_set_greeks <- function(
  greeks_type,
  tz = .okx_default_tz,
  config
) {
  .posts$account_set_greeks(
    body_list = list(greeksType = greeks_type),
    tz = tz,
    config = config
  )
}

#' Set Account Auto Repay
#'
#' Enable or disable spot-mode auto repay.
#'
#' @param auto_repay Logical. Whether auto repay should be enabled.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` confirming the applied auto-repay setting.
#' @export
post_account_set_auto_repay <- function(
  auto_repay,
  tz = .okx_default_tz,
  config
) {
  .posts$account_set_auto_repay(
    body_list = list(autoRepay = isTRUE(auto_repay)),
    tz = tz,
    config = config
  )
}

#' Set Account Auto Loan
#'
#' Enable or disable automatic borrowing.
#'
#' @param auto_loan Logical. Whether auto loan should be enabled.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` confirming the applied auto-loan setting.
#' @export
post_account_set_auto_loan <- function(
  auto_loan = TRUE,
  tz = .okx_default_tz,
  config
) {
  .posts$account_set_auto_loan(
    body_list = list(autoLoan = isTRUE(auto_loan)),
    tz = tz,
    config = config
  )
}

#' Set Account Level
#'
#' Switch the account mode.
#'
#' @param acct_lv Account level string, such as `"1"`, `"2"`, `"3"`, or `"4"`.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` confirming the applied account level.
#' @export
post_account_set_account_level <- function(
  acct_lv,
  tz = .okx_default_tz,
  config
) {
  .posts$account_set_account_level(
    body_list = list(acctLv = acct_lv),
    tz = tz,
    config = config
  )
}

#' Set Account Collateral Assets
#'
#' Configure whether all or selected assets are treated as collateral.
#'
#' @param type Type of update, typically `"all"` or `"custom"`.
#' @param collateral_enabled Logical. Whether the selected assets should be
#'   collateral-enabled.
#' @param ccy_list Optional character vector of currencies. Required when
#'   `type = "custom"`.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` confirming the applied collateral asset setting.
#' @export
post_account_set_collateral_assets <- function(type, collateral_enabled, ccy_list = NULL, tz = .okx_default_tz, config) {
  body <- list(
    type = type,
    collateralEnabled = isTRUE(collateral_enabled)
  )
  if (!is.null(ccy_list)) {
    body$ccyList <- ccy_list
  }
  .posts$account_set_collateral_assets(
    body_list = body,
    tz = tz,
    config = config
  )
}

#' Adjust Position Margin Balance
#'
#' Increase or reduce margin for an existing position.
#'
#' @param inst_id Instrument ID.
#' @param pos_side Position side.
#' @param type Margin adjustment type, typically `"add"` or `"reduce"`.
#' @param amt Amount to add or reduce.
#' @param ccy Optional currency for isolated margin orders.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` describing the applied margin adjustment.
#' @export
post_account_position_margin_balance <- function(inst_id, pos_side, type, amt, ccy = NULL, tz = .okx_default_tz, config) {
  body <- list(
    instId = inst_id,
    posSide = pos_side,
    type = type,
    amt = amt
  )
  if (!is.null(ccy)) {
    body$ccy <- ccy
  }
  .posts$account_position_margin_balance(body_list = body, tz = tz, config = config)
}

#' Submit Spot Manual Borrow or Repay
#'
#' Manually borrow or repay under spot mode.
#'
#' @param ccy Currency.
#' @param side Action side, typically `"borrow"` or `"repay"`.
#' @param amt Amount.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` describing the executed borrow/repay request.
#' @export
post_account_spot_manual_borrow_repay <- function(ccy, side, amt, tz = .okx_default_tz, config) {
  .posts$account_spot_manual_borrow_repay(
    body_list = list(ccy = ccy, side = side, amt = amt),
    tz = tz,
    config = config
  )
}

#' Preset Account Level Switch
#'
#' Preset required values before switching account mode.
#'
#' @param acct_lv Target account level.
#' @param lever Optional leverage preset.
#' @param risk_offset_type Optional deprecated risk offset type field.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` describing the stored preset values.
#' @export
post_account_account_level_switch_preset <- function(acct_lv, lever = NULL, risk_offset_type = NULL, tz = .okx_default_tz, config) {
  body <- list(acctLv = acct_lv)
  if (!is.null(lever)) {
    body$lever <- lever
  }
  if (!is.null(risk_offset_type)) {
    body$riskOffsetType <- risk_offset_type
  }
  .posts$account_account_level_switch_preset(body_list = body, tz = tz, config = config)
}

#' Reset MMP Status
#'
#' Reset market maker protection status for an instrument family.
#'
#' @param inst_family Instrument family.
#' @param inst_type Optional instrument type. Defaults to `"OPTION"` on OKX.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` with the request result.
#' @export
post_account_mmp_reset <- function(inst_family, inst_type = NULL, tz = .okx_default_tz, config) {
  body <- list(instFamily = inst_family)
  if (!is.null(inst_type)) {
    body$instType <- inst_type
  }
  .posts$account_mmp_reset(body_list = body, tz = tz, config = config)
}

#' Configure MMP
#'
#' Set market maker protection thresholds for an options instrument family.
#'
#' @param inst_family Instrument family.
#' @param time_interval Time window in milliseconds.
#' @param frozen_interval Frozen interval in milliseconds.
#' @param qty_limit Quantity limit in number of contracts.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` describing the applied MMP configuration.
#' @export
post_account_mmp_config <- function(inst_family, time_interval, frozen_interval, qty_limit, tz = .okx_default_tz, config) {
  .posts$account_mmp_config(
    body_list = list(
      instFamily = inst_family,
      timeInterval = time_interval,
      frozenInterval = frozen_interval,
      qtyLimit = qty_limit
    ),
    tz = tz,
    config = config
  )
}

#' Move Positions Between Accounts
#'
#' Move positions between accounts under the same master account.
#'
#' @param from_acct Source account name.
#' @param to_acct Destination account name.
#' @param legs List of move-position leg objects in the documented OKX shape.
#' @param client_id Client-supplied request ID.
#' @param tz Timezone used for any timestamp parsing.
#' @param config A list containing API credentials.
#'
#' @return A `data.frame` describing the move-position request result.
#' @export
post_account_move_positions <- function(from_acct, to_acct, legs, client_id, tz = .okx_default_tz, config) {
  .okx_assert_non_empty_list(legs, "legs")
  .posts$account_move_positions(
    body_list = list(
      fromAcct = from_acct,
      toAcct = to_acct,
      legs = legs,
      clientId = client_id
    ),
    tz = tz,
    config = config
  )
}

#---- Market: GET Wrappers ----

.okx_standardize_ohlcv_names <- function(df) {
  if (is.null(df)) {
    return(NULL)
  }

  names(df)[names(df) == "ts"] <- "timestamp"
  names(df)[names(df) == "o"]  <- "open"
  names(df)[names(df) == "h"]  <- "high"
  names(df)[names(df) == "l"]  <- "low"
  names(df)[names(df) == "c"]  <- "close"
  names(df)[names(df) == "vol"]  <- "volume"
  names(df)[names(df) == "volCcyQuote"]  <- "volQuote"
  df
}

#' Get recent market candles
#'
#' Retrieve the latest candlestick data for a given instrument and bar size.
#'
#' @details
#' Wraps `/api/v5/market/candles`. Returns up to `limit` bars, sorted by
#' timestamp. Candlestick fields can be standardized to common OHLCV names via
#' `standardize_names = TRUE`.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"`, `"ETH-USDT-SWAP"`.
#' @param bar Character. Candlestick granularity, e.g. `"1m"`, `"5m"`, `"1H"`, `"1D"`.
#' @param limit Integer. Number of bars to retrieve. Default `100L`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#' @param standardize_names Logical. If `TRUE` (default), renames columns to
#'   `timestamp`, `open`, `high`, `low`, `close`, `volume`, `volQuote`.
#'
#' @return
#' A `data.frame` with columns including `timestamp`, `open`, `high`, `low`,
#' `close`, `volume`, and `volQuote`. Timestamps are `POSIXct` in `tz`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_market_candles("BTC-USDT", bar = "5m", limit = 50, config = cfg)
#' }
#'
#' @seealso [get_market_history_candles()], [get_public_mark_price()]
#' @note Since okxr 0.1.1
#' @export
get_market_candles <- function(inst_id, bar, limit = 100L, config = NULL, tz = .okx_default_tz, standardize_names = TRUE) {
  query_string <- .okx_build_query(instId = inst_id, bar = bar, limit = as.integer(limit))
  df <- .gets$market_candles(query_string = query_string, config = config, tz = tz)
  if (standardize_names) return(.okx_standardize_ohlcv_names(df))
  df
}

#' Get historical market candles
#'
#' Retrieve candlestick data before a specific datetime.
#'
#' @details
#' Wraps `/api/v5/market/history-candles`. If `before` is supplied, it is
#' converted to milliseconds since epoch (in `tz`) and sent as `after=...`
#' (per OKX semantics: *return data before this time*).
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"`.
#' @param bar Character. Candlestick granularity, e.g. `"1m"`, `"5m"`, `"1H"`.
#' @param before Character or `NULL`. Timestamp string like
#'   `"\%Y-\%m-\%d \%H:\%M:\%S"`. If `NULL`, fetch recent history.
#' @param limit Integer. Number of bars to retrieve. Default `100L`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#' @param standardize_names Logical. If `TRUE` (default), renames columns to
#'   `timestamp`, `open`, `high`, `low`, `close`, `volume`, `volQuote`.
#'
#' @return
#' A `data.frame` of candlestick bars. If `standardize_names = TRUE`,
#' column names are normalized. Timestamps are `POSIXct` in `tz`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_market_history_candles(
#'   "ETH-USDT-SWAP", bar = "1H",
#'   before = "2025-08-20 00:00:00", config = cfg
#' )
#' }
#'
#' @seealso [get_market_candles()]
#' @note Since okxr 0.1.1
#' @export
get_market_history_candles <- function(inst_id, bar, before = NULL, limit = 100L, config = NULL, tz = .okx_default_tz, standardize_names = TRUE) {
  before_ms <- .okx_datetime_to_ms(before, tz = tz)
  query_string <- .okx_build_query(
    instId = inst_id,
    bar = bar,
    after = before_ms,
    limit = as.integer(limit)
  )
  df <- .gets$market_history_candles(query_string = query_string, config = config, tz = tz)
  if (is.null(df) || length(df) == 0L) {
    return(NULL)
  }
  if (standardize_names) return(.okx_standardize_ohlcv_names(df))
  df
}

#' Get current mark price
#'
#' Retrieve the current mark price for a given instrument.
#'
#' @details
#' Wraps `/api/v5/public/mark-price`. Useful for margin calculations and PnL
#' estimation. Returns a single row with the latest mark price and timestamp.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"`, `"ETH-USDT-SWAP"`.
#' @param inst_type Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
#'   `"SWAP"` (default), `"FUTURES"`, `"OPTION"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with columns `timestamp`, `instId`, `markPx`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_public_mark_price("BTC-USDT", inst_type = "SWAP", config = cfg)
#' }
#'
#' @seealso [get_public_instruments()]
#' @note Since okxr 0.1.1
#' @export
get_public_mark_price <- function(inst_id, inst_type = "SWAP", config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type, instId = inst_id)
  .gets$public_mark_price(query_string = query_string, config = config, tz = tz)
}

#' Get market ticker
#'
#' Retrieve the latest ticker snapshot for a specific instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"` or `"ETH-USDT-SWAP"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with the latest ticker fields returned by OKX.
#'
#' @export
get_market_ticker <- function(inst_id, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instId = inst_id)
  .gets$market_ticker(query_string = query_string, config = config, tz = tz)
}

#' Get market tickers
#'
#' Retrieve ticker snapshots for all instruments under an instrument type.
#'
#' @param inst_type Character. Instrument type, e.g. `"SPOT"`, `"SWAP"`,
#'   `"FUTURES"`, or `"OPTION"`.
#' @param uly Character or `NULL`. Underlying. Optional filter for derivatives.
#' @param inst_family Character or `NULL`. Instrument family. Optional filter
#'   for derivatives and options.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per ticker.
#'
#' @export
get_market_tickers <- function(inst_type, uly = NULL, inst_family = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    uly = uly,
    instFamily = inst_family
  )
  .gets$market_tickers(query_string = query_string, config = config, tz = tz)
}

#' Get order book
#'
#' Retrieve the current order book for an instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"`.
#' @param sz Integer or `NULL`. Order book depth. If `NULL`, OKX uses its
#'   endpoint default.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with JSON-encoded `asks` and `bids` columns plus `ts`.
#'
#' @export
get_market_books <- function(
  inst_id,
  sz = NULL,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instId = inst_id, sz = sz)
  .gets$market_books(query_string = query_string, config = config, tz = tz)
}

#' Get recent public trades
#'
#' Retrieve recent public trades for an instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"`.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with recent public trades.
#'
#' @export
get_market_trades <- function(
  inst_id,
  limit = NULL,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instId = inst_id, limit = limit)
  .gets$market_trades(query_string = query_string, config = config, tz = tz)
}

#' Get historical public trades
#'
#' Retrieve public trade history for an instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT"`.
#' @param type Character or `NULL`. Pagination type, using OKX values.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with historical public trades.
#'
#' @export
get_market_history_trades <- function(inst_id, type = NULL, after = NULL, before = NULL, limit = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instId = inst_id,
    type = type,
    after = after,
    before = before,
    limit = limit
  )
  .gets$market_history_trades(query_string = query_string, config = config, tz = tz)
}

#' Get mark price candles
#'
#' Retrieve recent candlestick data for the mark price of an instrument.
#'
#' @param inst_id Character. Instrument ID.
#' @param bar Character or `NULL`. Bar size.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param limit Integer. Number of rows to request. Default `100L`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#' @param standardize_names Logical. If `TRUE` (default), renames OHLC columns
#'   to `timestamp`, `open`, `high`, `low`, and `close`.
#'
#' @return A `data.frame` of mark-price candlesticks.
#'
#' @export
get_market_mark_price_candles <- function(inst_id, bar = NULL, after = NULL, before = NULL, limit = 100L, config = NULL, tz = .okx_default_tz, standardize_names = TRUE) {
  query_string <- .okx_build_query(
    instId = inst_id,
    after = after,
    before = before,
    bar = bar,
    limit = as.integer(limit)
  )
  df <- .gets$market_mark_price_candles(query_string = query_string, config = config, tz = tz)
  if (standardize_names) return(.okx_standardize_ohlcv_names(df))
  df
}

#' Get historical mark price candles
#'
#' Retrieve historical candlestick data for mark price.
#'
#' @inheritParams get_market_mark_price_candles
#'
#' @return A `data.frame` of historical mark-price candlesticks.
#'
#' @export
get_market_history_mark_price_candles <- function(inst_id, bar = NULL, after = NULL, before = NULL, limit = 100L, config = NULL, tz = .okx_default_tz, standardize_names = TRUE) {
  query_string <- .okx_build_query(
    instId = inst_id,
    after = after,
    before = before,
    bar = bar,
    limit = as.integer(limit)
  )
  df <- .gets$market_history_mark_price_candles(query_string = query_string, config = config, tz = tz)
  if (standardize_names) return(.okx_standardize_ohlcv_names(df))
  df
}

#' Get exchange rate
#'
#' Retrieve the two-week average exchange rate series summary.
#'
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with `usdCny`.
#'
#' @export
get_market_exchange_rate <- function(config = NULL, tz = .okx_default_tz) {
  .gets$market_exchange_rate(query_string = "", config = config, tz = tz)
}

#' Get index components
#'
#' Retrieve component-exchange information for an index.
#'
#' @param index Character. Index identifier, e.g. `"BTC-USD"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with `index`, `last`, `ts`, and JSON-encoded
#'   `components`.
#'
#' @export
get_market_index_components <- function(
  index,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(index = index)
  .gets$market_index_components(query_string = query_string, config = config, tz = tz)
}

#' Get platform 24-hour volume
#'
#' Retrieve total platform order-book trading volume over the last 24 hours.
#'
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with `volUsd`, `volCny`, and `ts`.
#'
#' @export
get_market_platform_24_volume <- function(
  config = NULL,
  tz = .okx_default_tz
) {
  .gets$market_platform_24_volume(query_string = "", config = config, tz = tz)
}

#' Get block ticker
#'
#' Retrieve the latest 24-hour block-trading volume for a single instrument.
#'
#' @param inst_id Character. Instrument ID.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with block trading volume fields.
#'
#' @export
get_market_block_ticker <- function(
  inst_id,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instId = inst_id)
  .gets$market_block_ticker(query_string = query_string, config = config, tz = tz)
}

#' Get block tickers
#'
#' Retrieve the latest 24-hour block-trading volume for instruments under an
#' instrument type.
#'
#' @param inst_type Character. Instrument type.
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per block ticker.
#'
#' @export
get_market_block_tickers <- function(inst_type, inst_family = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family
  )
  .gets$market_block_tickers(query_string = query_string, config = config, tz = tz)
}

#' Get instrument metadata
#'
#' Retrieve metadata for instruments of a given type.
#'
#' @details
#' Wraps `/api/v5/public/instruments`. Returns one row per instrument,
#' including contract specifications, tick size, lot size, expiry, and state.
#'
#' @param inst_id Character or `NULL`. Specific instrument ID to query. Use
#'   `NULL` to fetch all instruments of `inst_type`.
#' @param inst_type Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
#'   `"SWAP"` (default), `"FUTURES"`, `"OPTION"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with instrument metadata (e.g., `instType`, `instId`, `uly`,
#' `baseCcy`, `quoteCcy`, `settleCcy`, `ctVal`, `ctMult`, `tickSz`, `lotSz`,
#' `minSz`, `expTime`, `lever`, `state`, ...).
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' # Get metadata for all SWAP instruments
#' df <- get_public_instruments(inst_type = "SWAP", config = cfg)
#'
#' # Get metadata for one instrument
#' get_public_instruments("ETH-USDT-SWAP", inst_type = "SWAP", config = cfg)
#' }
#'
#' @seealso [get_public_mark_price()]
#' @note Since okxr 0.1.2
#' @export
get_public_instruments <- function(inst_id = NULL, inst_type = "SWAP", config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type, instId = inst_id)
  .gets$public_instruments(query_string = query_string, config = config, tz = tz)
}

#' Get underlying list
#'
#' Retrieve available underlyings for derivatives instruments.
#'
#' @param inst_type Character. Instrument type: `"SWAP"`, `"FUTURES"`, or
#'   `"OPTION"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Unused except for interface consistency.
#'
#' @return A one-column `data.frame` with `uly`.
#'
#' @export
get_public_underlying <- function(
  inst_type,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$public_underlying(query_string = query_string, config = config, tz = tz)
}

#' Get current funding rate
#'
#' Retrieve the current funding rate for a perpetual swap instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT-SWAP"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` containing the current funding rate fields returned by OKX.
#'
#' @export
get_public_funding_rate <- function(
  inst_id,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instId = inst_id)
  .gets$public_funding_rate(query_string = query_string, config = config, tz = tz)
}

#' Get funding rate history
#'
#' Retrieve historical funding rate entries for a perpetual swap instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT-SWAP"`.
#' @param before Optional cursor for records earlier than the supplied value.
#' @param after Optional cursor for records later than the supplied value.
#' @param limit Integer. Number of records to request. Default `400`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` containing funding rate history rows returned by OKX.
#'
#' @export
get_public_funding_rate_history <- function(inst_id, before = NULL, after = NULL, limit = 400, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instId = inst_id,
    before = before,
    after = after,
    limit = as.integer(limit)
  )
  .gets$public_funding_rate_history(query_string = query_string, config = config, tz = tz)
}

#' Get open interest
#'
#' Retrieve current open interest for an instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT-SWAP"`.
#' @param inst_type Character. Instrument type such as `"SWAP"` or `"FUTURES"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` containing open interest fields returned by OKX.
#'
#' @export
get_public_open_interest <- function(inst_id, inst_type, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type, instId = inst_id)
  .gets$public_open_interest(query_string = query_string, config = config, tz = tz)
}

#' Get OKX system time
#'
#' Retrieve OKX system time.
#'
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A one-row `data.frame` with system time.
#'
#' @export
get_public_time <- function(config = NULL, tz = .okx_default_tz) {
  .gets$public_time(query_string = "", config = config, tz = tz)
}

#' Get price limit
#'
#' Retrieve buy and sell price limits for an instrument.
#'
#' @param inst_id Character. Instrument ID, e.g. `"BTC-USDT-SWAP"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with price limit fields.
#'
#' @export
get_public_price_limit <- function(
  inst_id,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instId = inst_id)
  .gets$public_price_limit(query_string = query_string, config = config, tz = tz)
}

#' Get estimated delivery or exercise price
#'
#' Retrieve the estimated delivery, exercise, or settlement price for
#' derivatives and events instruments.
#'
#' @param inst_type Character. Instrument type, such as `"FUTURES"`, `"OPTION"`,
#'   `"SWAP"`, or `"EVENTS"`.
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param inst_id Character or `NULL`. Specific instrument ID filter.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `instType`, `instId`, `settlePx`, and `ts`.
#'
#' @export
get_public_estimated_price <- function(inst_type, inst_family = NULL, inst_id = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family,
    instId = inst_id
  )
  .gets$public_estimated_price(query_string = query_string, config = config, tz = tz)
}

#' Get delivery or exercise history
#'
#' Retrieve futures delivery records or option exercise records.
#'
#' @param inst_type Character. `"FUTURES"` or `"OPTION"`.
#' @param inst_family Character. Instrument family.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `ts` and JSON-encoded `details`.
#'
#' @export
get_public_delivery_exercise_history <- function(inst_type, inst_family, after = NULL, before = NULL, limit = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family,
    after = after,
    before = before,
    limit = limit
  )
  .gets$public_delivery_exercise_history(query_string = query_string, config = config, tz = tz)
}

#' Get estimated settlement info
#'
#' Retrieve the estimated settlement price for a futures instrument close to
#' settlement.
#'
#' @param inst_id Character. Instrument ID.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `instId`, `nextSettleTime`, `estSettlePx`, and `ts`.
#'
#' @export
get_public_estimated_settlement_info <- function(inst_id, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instId = inst_id)
  .gets$public_estimated_settlement_info(query_string = query_string, config = config, tz = tz)
}

#' Get settlement history
#'
#' Retrieve futures settlement history for an instrument family.
#'
#' @param inst_family Character. Instrument family.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `ts` and JSON-encoded `details`.
#'
#' @export
get_public_settlement_history <- function(inst_family, after = NULL, before = NULL, limit = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instFamily = inst_family,
    after = after,
    before = before,
    limit = limit
  )
  .gets$public_settlement_history(query_string = query_string, config = config, tz = tz)
}

#' Get collateral discount rate and interest-free quota
#'
#' Retrieve public collateral discount-rate tiers and interest-free quota
#' information for supported currencies.
#'
#' @param ccy Character or `NULL`. Currency filter, e.g. `"BTC"`.
#' @param discount_lv Character or `NULL`. Discount level filter.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with top-level discount and interest-free quota
#'   fields; nested tier details are JSON-encoded in `details`.
#'
#' @export
get_public_discount_rate_interest_free_quota <- function(ccy = NULL, discount_lv = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ccy = ccy,
    discountLv = discount_lv
  )
  .gets$public_discount_rate_interest_free_quota(query_string = query_string, config = config, tz = tz)
}

#' Get option summary
#'
#' Retrieve option market summary data for an instrument family.
#'
#' @param inst_family Character. Option instrument family.
#' @param exp_time Character or `NULL`. Expiry date in `YYMMDD`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with option greeks, volatility, forward price, and
#'   timestamp fields.
#'
#' @export
get_public_opt_summary <- function(inst_family, exp_time = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instFamily = inst_family,
    expTime = exp_time
  )
  .gets$public_opt_summary(query_string = query_string, config = config, tz = tz)
}

#' Get public position tiers
#'
#' Retrieve public tier, margin, and maximum leverage information.
#'
#' @param inst_type Character. Instrument type.
#' @param td_mode Character. Trade mode.
#' @param inst_family Character or `NULL`. Instrument family.
#' @param inst_id Character or `NULL`. Instrument ID(s).
#' @param ccy Character or `NULL`. Margin currency.
#' @param tier Character or `NULL`. Tier filter.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with public position tier information.
#'
#' @export
get_public_position_tiers <- function(inst_type, td_mode, inst_family = NULL, inst_id = NULL, ccy = NULL, tier = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    tdMode = td_mode,
    instFamily = inst_family,
    instId = inst_id,
    ccy = ccy,
    tier = tier
  )
  .gets$public_position_tiers(query_string = query_string, config = config, tz = tz)
}

#' Get economic calendar
#'
#' Retrieve macro-economic calendar records. OKX requires authentication for
#' this endpoint.
#'
#' @param region Character or `NULL`. Region filter.
#' @param importance Character or `NULL`. Importance level filter.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with calendar event fields and timestamps.
#'
#' @export
get_public_economic_calendar <- function(region = NULL, importance = NULL, before = NULL, after = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    region = region,
    importance = importance,
    before = before,
    after = after,
    limit = limit
  )
  .gets$public_economic_calendar(query_string = query_string, config = config, tz = tz)
}

#' Get interest rate and loan quota
#'
#' Retrieve public borrowing-rate and loan-quota tables.
#'
#' @param ccy Character or `NULL`. Currency filter.
#' @param vip_level Character or `NULL`. VIP level filter when supported by OKX.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` where nested basic, VIP, regular, and custom quota
#'   tables are JSON-encoded string columns.
#'
#' @export
get_public_interest_rate_loan_quota <- function(ccy = NULL, vip_level = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    ccy = ccy,
    vipLevel = vip_level
  )
  .gets$public_interest_rate_loan_quota(query_string = query_string, config = config, tz = tz)
}

#' Get security fund balance information
#'
#' Retrieve public insurance-fund or security-fund balance information.
#'
#' @param inst_type Character. Instrument type, such as `"MARGIN"`, `"SWAP"`,
#'   `"FUTURES"`, or `"OPTION"`.
#' @param type Character or `NULL`. Fund update type filter.
#' @param inst_family Character or `NULL`. Instrument family filter for
#'   derivatives.
#' @param ccy Character or `NULL`. Currency filter for margin data.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with top-level fund totals; nested per-currency detail
#'   rows are JSON-encoded in `details`.
#'
#' @export
get_public_insurance_fund <- function(inst_type, type = NULL, inst_family = NULL, ccy = NULL, before = NULL, after = NULL, limit = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    type = type,
    instFamily = inst_family,
    ccy = ccy,
    before = before,
    after = after,
    limit = limit
  )
  .gets$public_insurance_fund(query_string = query_string, config = config, tz = tz)
}

#' Convert between contract size and currency amount
#'
#' Convert the crypto value to the number of contracts, or vice versa.
#'
#' @param inst_id Character. Instrument ID.
#' @param sz Character or numeric. Quantity to convert.
#' @param type Character or `NULL`. Convert type: `"1"` for currency to
#'   contract, `"2"` for contract to currency.
#' @param px Character, numeric, or `NULL`. Optional order price.
#' @param unit Character or `NULL`. Currency unit, `"coin"` or `"usds"`.
#' @param op_type Character or `NULL`. Order type for futures or swaps, such as
#'   `"open"` or `"close"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with conversion result fields `type`, `instId`, `px`,
#'   `sz`, and `unit`.
#'
#' @export
get_public_convert_contract_coin <- function(inst_id, sz, type = NULL, px = NULL, unit = NULL, op_type = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    type = type,
    instId = inst_id,
    sz = sz,
    px = px,
    unit = unit,
    opType = op_type
  )
  .gets$public_convert_contract_coin(query_string = query_string, config = config, tz = tz)
}

#' Get option instrument tick bands
#'
#' Retrieve option tick-band information for one or more option instrument
#' families.
#'
#' @param inst_type Character. Instrument type. Currently `"OPTION"`.
#' @param inst_family Character or `NULL`. Instrument family filter.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `instType`, `instFamily`, and JSON-encoded
#'   `tickBand` details.
#'
#' @export
get_public_instrument_tick_bands <- function(inst_type = "OPTION", inst_family = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instFamily = inst_family
  )
  .gets$public_instrument_tick_bands(query_string = query_string, config = config, tz = tz)
}

#' Get premium history
#'
#' Retrieve premium-index history for an instrument.
#'
#' @param inst_id Character. Instrument ID.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param bar Character or `NULL`. Bar size such as `"1m"` or `"1H"`.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with `instId`, `premium`, and `ts`.
#'
#' @export
get_public_premium_history <- function(inst_id, after = NULL, before = NULL, bar = NULL, limit = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instId = inst_id,
    after = after,
    before = before,
    bar = bar,
    limit = limit
  )
  .gets$public_premium_history(query_string = query_string, config = config, tz = tz)
}

#' Get public block trades
#'
#' Retrieve recent single-leg public block trades for an instrument.
#'
#' @param inst_id Character. Instrument ID.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with block trade fields such as price, size, trade
#'   side, volatility, and timestamps.
#'
#' @export
get_public_block_trades <- function(
  inst_id,
  config = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instId = inst_id)
  .gets$public_block_trades(query_string = query_string, config = config, tz = tz)
}

#' Get index tickers
#'
#' Retrieve the latest public index-price snapshots.
#'
#' @param quote_ccy Character or `NULL`. Quote currency filter.
#' @param inst_id Character or `NULL`. Specific index ID filter.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per index ticker.
#'
#' @export
get_market_index_tickers <- function(quote_ccy = NULL, inst_id = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    quoteCcy = quote_ccy,
    instId = inst_id
  )
  .gets$market_index_tickers(query_string = query_string, config = config, tz = tz)
}

#' Get recent index candles
#'
#' Retrieve the latest candlestick data for an index.
#'
#' @param inst_id Character. Index ID.
#' @param bar Character or `NULL`. Bar size.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param limit Integer. Number of rows to request. Default `100L`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#' @param standardize_names Logical. If `TRUE` (default), renames OHLC columns
#'   to `timestamp`, `open`, `high`, `low`, and `close`.
#'
#' @return A `data.frame` of index candlesticks.
#'
#' @export
get_market_index_candles <- function(inst_id, bar = NULL, after = NULL, before = NULL, limit = 100L, config = NULL, tz = .okx_default_tz, standardize_names = TRUE) {
  query_string <- .okx_build_query(
    instId = inst_id,
    after = after,
    before = before,
    bar = bar,
    limit = as.integer(limit)
  )
  df <- .gets$market_index_candles(query_string = query_string, config = config, tz = tz)
  if (standardize_names) return(.okx_standardize_ohlcv_names(df))
  df
}

#' Get historical index candles
#'
#' Retrieve historical candlestick data for an index.
#'
#' @param inst_id Character. Index ID.
#' @param bar Character or `NULL`. Bar size.
#' @param after Character or `NULL`. Pagination cursor for older rows.
#' @param before Character or `NULL`. Pagination cursor for newer rows.
#' @param limit Integer. Number of rows to request. Default `100L`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#' @param standardize_names Logical. If `TRUE` (default), renames OHLC columns
#'   to `timestamp`, `open`, `high`, `low`, and `close`.
#'
#' @return A `data.frame` of historical index candlesticks.
#'
#' @export
get_market_history_index_candles <- function(inst_id, bar = NULL, after = NULL, before = NULL, limit = 100L, config = NULL, tz = .okx_default_tz, standardize_names = TRUE) {
  query_string <- .okx_build_query(
    instId = inst_id,
    after = after,
    before = before,
    bar = bar,
    limit = as.integer(limit)
  )
  df <- .gets$market_history_index_candles(query_string = query_string, config = config, tz = tz)
  if (standardize_names) return(.okx_standardize_ohlcv_names(df))
  df
}

#' Get option trades by instrument family
#'
#' Retrieve recent option trades for all instruments under the same instrument
#' family.
#'
#' @param inst_family Character. Instrument family, e.g. `"BTC-USD"`.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with recent option trades under the requested
#'   instrument family, including `instId`, `tradeId`, `px`, `sz`, `side`, and
#'   `ts`.
#'
#' @export
get_market_option_instrument_family_trades <- function(inst_family, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instFamily = inst_family)
  .gets$market_option_instrument_family_trades(query_string = query_string, config = config, tz = tz)
}

#' Get public option trades
#'
#' Retrieve recent public option trades filtered by instrument ID or instrument
#' family.
#'
#' @param inst_id Character or `NULL`. Specific option instrument ID.
#' @param inst_family Character or `NULL`. Option instrument family, e.g.
#'   `"BTC-USD"`. Either `inst_id` or `inst_family` should be supplied.
#' @param opt_type Character or `NULL`. Option type filter: `"C"` for call or
#'   `"P"` for put.
#' @param config Optional list. Public endpoint request options, such as
#'   `timeout`; credentials are not required.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with recent option trade rows, including option
#'   instrument identifiers, trade price and size, option side/type, forward,
#'   index and mark prices, implied volatility, and trade time.
#'
#' @export
get_public_option_trades <- function(inst_id = NULL, inst_family = NULL, opt_type = NULL, config = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instId = inst_id,
    instFamily = inst_family,
    optType = opt_type
  )
  .gets$public_option_trades(query_string = query_string, config = config, tz = tz)
}

test_that("GET wrappers build expected query strings", {
  ns <- asNamespace("okxr")
  old_gets <- get(".gets", envir = ns)
  unlockBinding(".gets", ns)
  on.exit({
    assign(".gets", old_gets, envir = ns)
    lockBinding(".gets", ns)
  }, add = TRUE)
  assign(
    ".gets",
    list(
      market_tickers = function(query_string, config, tz) query_string,
      market_history_trades = function(query_string, config, tz) query_string,
      market_mark_price_candles = function(query_string, config, tz) query_string,
      market_history_mark_price_candles = function(query_string, config, tz) query_string,
      market_exchange_rate = function(query_string, config, tz) query_string,
      market_index_components = function(query_string, config, tz) query_string,
      market_platform_24_volume = function(query_string, config, tz) query_string,
      market_block_ticker = function(query_string, config, tz) query_string,
      market_block_tickers = function(query_string, config, tz) query_string,
      market_option_instrument_family_trades = function(query_string, config, tz) query_string,
      market_index_tickers = function(query_string, config, tz) query_string,
      market_index_candles = function(query_string, config, tz) query_string,
      market_history_index_candles = function(query_string, config, tz) query_string,
      public_underlying = function(query_string, config, tz) query_string,
      public_estimated_price = function(query_string, config, tz) query_string,
      public_delivery_exercise_history = function(query_string, config, tz) query_string,
      public_estimated_settlement_info = function(query_string, config, tz) query_string,
      public_settlement_history = function(query_string, config, tz) query_string,
      public_discount_rate_interest_free_quota = function(query_string, config, tz) query_string,
      public_opt_summary = function(query_string, config, tz) query_string,
      public_position_tiers = function(query_string, config, tz) query_string,
      public_economic_calendar = function(query_string, config, tz) query_string,
      public_interest_rate_loan_quota = function(query_string, config, tz) query_string,
      public_insurance_fund = function(query_string, config, tz) query_string,
      public_convert_contract_coin = function(query_string, config, tz) query_string,
      public_instrument_tick_bands = function(query_string, config, tz) query_string,
      public_premium_history = function(query_string, config, tz) query_string,
      public_block_trades = function(query_string, config, tz) query_string,
      public_option_trades = function(query_string, config, tz) query_string,
      account_bills = function(query_string, config, tz) query_string,
      account_instruments = function(query_string, config, tz) query_string,
      account_subtypes = function(query_string, config, tz) query_string,
      account_adjust_leverage_info = function(query_string, config, tz) query_string,
      account_max_loan = function(query_string, config, tz) query_string,
      account_position_risk = function(query_string, config, tz) query_string,
      account_max_size = function(query_string, config, tz) query_string,
      account_max_avail_size = function(query_string, config, tz) query_string,
      account_trade_fee = function(query_string, config, tz) query_string,
      account_interest_rate = function(query_string, config, tz) query_string,
      account_interest_accrued = function(query_string, config, tz) query_string,
      account_max_withdrawal = function(query_string, config, tz) query_string,
      account_risk_state = function(query_string, config, tz) query_string,
      account_interest_limits = function(query_string, config, tz) query_string,
      account_greeks = function(query_string, config, tz) query_string,
      account_position_tiers = function(query_string, config, tz) query_string,
      account_collateral_assets = function(query_string, config, tz) query_string,
      account_mmp_config = function(query_string, config, tz) query_string,
      account_move_positions_history = function(query_string, config, tz) query_string,
      account_precheck_set_delta_neutral = function(query_string, config, tz) query_string,
      account_bills_history_archive = function(query_string, config, tz) query_string,
      account_subaccount_balances = function(query_string, config, tz) query_string,
      account_subaccount_max_withdrawal = function(query_string, config, tz) query_string,
      account_set_account_switch_precheck = function(query_string, config, tz) query_string,
      account_spot_borrow_repay_history = function(query_string, config, tz) query_string,
      copy_trade_settings = function(query_string, config, tz) query_string,
      copy_trade_current_subpos = function(query_string, config, tz) query_string,
      copy_trade_historical_subpos = function(query_string, config, tz) query_string,
      copy_trade_instruments = function(query_string, config, tz) query_string,
      copy_trade_config = function(query_string, config, tz) query_string,
      copy_trade_public_config = function(query_string, config, tz) query_string,
      copy_trade_public_copy_traders = function(query_string, config, tz) query_string,
      copy_trade_public_lead_traders = function(query_string, config, tz) query_string,
      copy_trade_public_current_subpositions = function(query_string, config, tz) query_string,
      copy_trade_public_subpositions_history = function(query_string, config, tz) query_string,
      copy_trade_public_pnl = function(query_string, config, tz) query_string,
      copy_trade_public_stats = function(query_string, config, tz) query_string,
      copy_trade_public_weekly_pnl = function(query_string, config, tz) query_string,
      copy_trade_public_preference_currency = function(query_string, config, tz) query_string,
      copy_trade_profit_sharing_details = function(query_string, config, tz) query_string,
      copy_trade_unrealized_profit_sharing_details = function(query_string, config, tz) query_string,
      copy_trade_total_profit_sharing = function(query_string, config, tz) query_string,
      copy_trade_total_unrealized_profit_sharing = function(query_string, config, tz) query_string,
      asset_deposit_history = function(query_string, config, tz) query_string,
      asset_withdrawal_history = function(query_string, config, tz) query_string,
      asset_currencies = function(query_string, config, tz) query_string,
      asset_non_tradable_assets = function(query_string, config, tz) query_string,
      asset_asset_valuation = function(query_string, config, tz) query_string,
      asset_transfer_state = function(query_string, config, tz) query_string,
      asset_bills = function(query_string, config, tz) query_string,
      asset_bills_history = function(query_string, config, tz) query_string,
      asset_deposit_withdraw_status = function(query_string, config, tz) query_string,
      asset_exchange_list = function(query_string, config, tz) query_string,
      asset_convert_currencies = function(query_string, config, tz) query_string,
      asset_convert_currency_pair = function(query_string, config, tz) query_string,
      asset_convert_history = function(query_string, config, tz) query_string,
      trade_orders_history = function(query_string, config, tz) query_string,
      trade_orders_history_archive = function(query_string, config, tz) query_string,
      trade_easy_convert_currency_list = function(query_string, config, tz) query_string,
      trade_easy_convert_history = function(query_string, config, tz) query_string,
      trade_one_click_repay_currency_list = function(query_string, config, tz) query_string,
      trade_one_click_repay_history = function(query_string, config, tz) query_string,
      trade_one_click_repay_currency_list_v2 = function(query_string, config, tz) query_string,
      trade_one_click_repay_history_v2 = function(query_string, config, tz) query_string,
      trade_order_algo = function(query_string, config, tz) query_string,
      trade_orders_algo_pending = function(query_string, config, tz) query_string,
      trade_orders_algo_history = function(query_string, config, tz) query_string,
      trade_fills = function(query_string, config, tz) query_string,
      trade_fills_history = function(query_string, config, tz) query_string,
      trade_account_rate_limit = function(query_string, config, tz) query_string
    ),
    envir = ns
  )

  expect_equal(
    okxr::get_market_tickers(inst_type = "SWAP", inst_family = "BTC-USDT"),
    "?instType=SWAP&instFamily=BTC-USDT"
  )
  expect_equal(
    okxr::get_market_history_trades("BTC-USDT", type = "1", limit = 10),
    "?instId=BTC-USDT&type=1&limit=10"
  )
  expect_equal(
    okxr::get_market_mark_price_candles("BTC-USD-SWAP", bar = "1H", limit = 20, standardize_names = FALSE),
    "?instId=BTC-USD-SWAP&bar=1H&limit=20"
  )
  expect_equal(
    okxr::get_market_history_mark_price_candles("BTC-USD-SWAP", before = "200", limit = 20, standardize_names = FALSE),
    "?instId=BTC-USD-SWAP&before=200&limit=20"
  )
  expect_equal(
    okxr::get_market_exchange_rate(),
    ""
  )
  expect_equal(
    okxr::get_market_index_components("BTC-USD"),
    "?index=BTC-USD"
  )
  expect_equal(
    okxr::get_market_platform_24_volume(),
    ""
  )
  expect_equal(
    okxr::get_market_block_ticker("BTC-USD-SWAP"),
    "?instId=BTC-USD-SWAP"
  )
  expect_equal(
    okxr::get_market_block_tickers(inst_type = "SWAP", inst_family = "BTC-USD"),
    "?instType=SWAP&instFamily=BTC-USD"
  )
  expect_equal(
    okxr::get_market_index_tickers(quote_ccy = "USD"),
    "?quoteCcy=USD"
  )
  expect_equal(
    okxr::get_market_index_candles("BTC-USD", bar = "1H", limit = 50, standardize_names = FALSE),
    "?instId=BTC-USD&bar=1H&limit=50"
  )
  expect_equal(
    okxr::get_market_history_index_candles("BTC-USD", before = "200", limit = 20, standardize_names = FALSE),
    "?instId=BTC-USD&before=200&limit=20"
  )
  expect_equal(
    okxr::get_market_option_instrument_family_trades("BTC-USD"),
    "?instFamily=BTC-USD"
  )
  expect_equal(
    okxr::get_public_underlying(inst_type = "FUTURES"),
    "?instType=FUTURES"
  )
  expect_equal(
    okxr::get_public_estimated_price(inst_type = "FUTURES", inst_family = "BTC-USD"),
    "?instType=FUTURES&instFamily=BTC-USD"
  )
  expect_equal(
    okxr::get_public_delivery_exercise_history(inst_type = "OPTION", inst_family = "BTC-USD", limit = 5),
    "?instType=OPTION&instFamily=BTC-USD&limit=5"
  )
  expect_equal(
    okxr::get_public_estimated_settlement_info("XRP-USDT-250307"),
    "?instId=XRP-USDT-250307"
  )
  expect_equal(
    okxr::get_public_settlement_history(inst_family = "XRP-USD", limit = 10),
    "?instFamily=XRP-USD&limit=10"
  )
  expect_equal(
    okxr::get_public_discount_rate_interest_free_quota(ccy = "BTC", discount_lv = "1"),
    "?ccy=BTC&discountLv=1"
  )
  expect_equal(
    okxr::get_public_opt_summary(inst_family = "BTC-USD", exp_time = "250328"),
    "?instFamily=BTC-USD&expTime=250328"
  )
  expect_equal(
    okxr::get_public_position_tiers(inst_type = "SWAP", td_mode = "cross", inst_family = "BTC-USDT"),
    "?instType=SWAP&tdMode=cross&instFamily=BTC-USDT"
  )
  expect_equal(
    okxr::get_public_interest_rate_loan_quota(ccy = "USDT", vip_level = "VIP1"),
    "?ccy=USDT&vipLevel=VIP1"
  )
  expect_equal(
    okxr::get_public_block_trades("BTC-USDT"),
    "?instId=BTC-USDT"
  )
  expect_equal(
    okxr::get_public_insurance_fund(inst_type = "SWAP", inst_family = "BTC-USD", limit = 5),
    "?instType=SWAP&instFamily=BTC-USD&limit=5"
  )
  expect_equal(
    okxr::get_public_convert_contract_coin(inst_id = "BTC-USD-SWAP", sz = "1", type = "1", px = "35000"),
    "?type=1&instId=BTC-USD-SWAP&sz=1&px=35000"
  )
  expect_equal(
    okxr::get_public_instrument_tick_bands(inst_family = "BTC-USD"),
    "?instType=OPTION&instFamily=BTC-USD"
  )
  expect_equal(
    okxr::get_public_premium_history(inst_id = "BTC-USD-SWAP", bar = "1H", limit = 100),
    "?instId=BTC-USD-SWAP&bar=1H&limit=100"
  )
  expect_equal(
    okxr::get_public_option_trades(inst_family = "BTC-USD", opt_type = "P"),
    "?instFamily=BTC-USD&optType=P"
  )
  expect_equal(
    okxr::get_copy_trade_public_config(inst_type = "SWAP"),
    "?instType=SWAP"
  )
  expect_equal(
    okxr::get_copy_trade_public_copy_traders(unique_code = "leader-1", inst_type = "SWAP", limit = 5),
    "?uniqueCode=leader-1&instType=SWAP&limit=5"
  )
  expect_equal(
    okxr::get_copy_trade_public_lead_traders(inst_type = "SWAP", sort_type = "pnl", state = "1", limit = 5),
    "?instType=SWAP&sortType=pnl&state=1&limit=5"
  )
  expect_equal(
    okxr::get_copy_trade_public_current_subpositions(unique_code = "leader-1", before = "200", limit = 10),
    "?uniqueCode=leader-1&before=200&limit=10"
  )
  expect_equal(
    okxr::get_copy_trade_public_subpositions_history(unique_code = "leader-1", after = "100", limit = 10),
    "?uniqueCode=leader-1&after=100&limit=10"
  )
  expect_equal(
    okxr::get_copy_trade_public_pnl(unique_code = "leader-1", last_days = "2", inst_type = "SWAP"),
    "?uniqueCode=leader-1&lastDays=2&instType=SWAP"
  )
  expect_equal(
    okxr::get_copy_trade_public_stats(unique_code = "leader-1", last_days = "4"),
    "?uniqueCode=leader-1&lastDays=4"
  )
  expect_equal(
    okxr::get_copy_trade_public_weekly_pnl(unique_code = "leader-1", inst_type = "SWAP"),
    "?uniqueCode=leader-1&instType=SWAP"
  )
  expect_equal(
    okxr::get_copy_trade_public_preference_currency(unique_code = "leader-1", inst_type = "SWAP"),
    "?uniqueCode=leader-1&instType=SWAP"
  )

  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")
  expect_equal(
    okxr::get_copy_trade_settings(unique_code = "leader-1", inst_type = "SWAP", config = cfg),
    "?uniqueCode=leader-1&instType=SWAP"
  )
  expect_equal(
    okxr::get_copy_trade_current_subpos(inst_type = "SWAP", inst_id = "BTC-USDT-SWAP", after = "100", limit = 10, config = cfg),
    "?instType=SWAP&instId=BTC-USDT-SWAP&after=100&limit=10"
  )
  expect_equal(
    okxr::get_copy_trade_historical_subpos(inst_type = "SWAP", inst_id = "BTC-USDT-SWAP", before = "200", limit = 10, config = cfg),
    "?instType=SWAP&instId=BTC-USDT-SWAP&before=200&limit=10"
  )
  expect_equal(
    okxr::get_copy_trade_instruments(inst_type = "SWAP", config = cfg),
    "?instType=SWAP"
  )
  expect_equal(
    okxr::get_copy_trade_config(config = cfg),
    ""
  )
  expect_equal(
    okxr::get_copy_trade_profit_sharing_details(inst_type = "SWAP", after = "100", limit = 10, config = cfg),
    "?instType=SWAP&after=100&limit=10"
  )
  expect_equal(
    okxr::get_copy_trade_unrealized_profit_sharing_details(inst_type = "SPOT", config = cfg),
    "?instType=SPOT"
  )
  expect_equal(
    okxr::get_copy_trade_total_profit_sharing(inst_type = "SWAP", config = cfg),
    "?instType=SWAP"
  )
  expect_equal(
    okxr::get_copy_trade_total_unrealized_profit_sharing(inst_type = "SWAP", config = cfg),
    "?instType=SWAP"
  )
  expect_equal(
    okxr::get_public_economic_calendar(region = "united_states", importance = "3", config = cfg),
    "?region=united_states&importance=3"
  )
  expect_equal(
    okxr::get_account_instruments(inst_type = "SPOT", inst_id = "BTC-USDT", config = cfg),
    "?instType=SPOT&instId=BTC-USDT"
  )
  expect_equal(
    okxr::get_account_subtypes(type = "1,2", config = cfg),
    "?type=1%2C2"
  )
  expect_equal(
    okxr::get_account_adjust_leverage_info(inst_type = "MARGIN", mgn_mode = "isolated", lever = 3, inst_id = "BTC-USDT", config = cfg),
    "?instType=MARGIN&mgnMode=isolated&lever=3&instId=BTC-USDT"
  )
  expect_equal(
    okxr::get_account_max_loan(mgn_mode = "cross", inst_id = "BTC-USDT", mgn_ccy = "BTC", config = cfg),
    "?mgnMode=cross&instId=BTC-USDT&mgnCcy=BTC"
  )
  expect_equal(
    okxr::get_account_position_risk(inst_type = "SWAP", config = cfg),
    "?instType=SWAP"
  )
  expect_equal(
    okxr::get_account_max_size(
      inst_id = "BTC-USDT",
      td_mode = "isolated",
      ccy = "BTC",
      leverage = 3,
      config = cfg
    ),
    "?instId=BTC-USDT&tdMode=isolated&ccy=BTC&leverage=3"
  )
  expect_equal(
    okxr::get_account_max_avail_size(
      inst_id = "BTC-USDT",
      td_mode = "cash",
      trade_quote_ccy = "USD",
      config = cfg
    ),
    "?instId=BTC-USDT&tdMode=cash&tradeQuoteCcy=USD"
  )
  expect_equal(
    okxr::get_account_trade_fee(inst_type = "SPOT", inst_id = "BTC-USDT", config = cfg),
    "?instType=SPOT&instId=BTC-USDT"
  )
  expect_equal(
    okxr::get_account_interest_rate(ccy = "BTC", config = cfg),
    "?ccy=BTC"
  )
  expect_equal(
    okxr::get_account_interest_accrued(ccy = "BTC", mgn_mode = "cross", limit = 10, config = cfg),
    "?ccy=BTC&mgnMode=cross&limit=10"
  )
  expect_equal(
    okxr::get_account_max_withdrawal(ccy = "BTC,ETH", config = cfg),
    "?ccy=BTC%2CETH"
  )
  expect_equal(
    okxr::get_account_risk_state(config = cfg),
    ""
  )
  expect_equal(
    okxr::get_account_interest_limits(type = "2", ccy = "BTC", config = cfg),
    "?type=2&ccy=BTC"
  )
  expect_equal(
    okxr::get_account_greeks(ccy = "BTC", config = cfg),
    "?ccy=BTC"
  )
  expect_equal(
    okxr::get_account_position_tiers(inst_type = "SWAP", inst_family = "BTC-USDT", config = cfg),
    "?instType=SWAP&instFamily=BTC-USDT"
  )
  expect_equal(
    okxr::get_account_collateral_assets(ccy = "BTC,ETH", collateral_enabled = TRUE, config = cfg),
    "?ccy=BTC%2CETH&collateralEnabled=TRUE"
  )
  expect_equal(
    okxr::get_account_mmp_config(inst_family = "BTC-USD", config = cfg),
    "?instFamily=BTC-USD"
  )
  expect_equal(
    okxr::get_account_move_positions_history(client_id = "move-1", state = "filled", limit = 5, config = cfg),
    "?clientId=move-1&limit=5&state=filled"
  )
  expect_equal(
    okxr::get_account_precheck_set_delta_neutral(stgy_type = 1, config = cfg),
    "?stgyType=1"
  )
  expect_equal(
    okxr::get_account_bills_history_archive(year = 2024, quarter = "Q1", type = "1,2", config = cfg),
    "?year=2024&quarter=Q1&type=1%2C2"
  )
  expect_equal(
    okxr::get_account_subaccount_balances(sub_acct = "desk_1", config = cfg),
    "?subAcct=desk_1"
  )
  expect_equal(
    okxr::get_account_subaccount_max_withdrawal(sub_acct = "desk_1", ccy = "BTC", config = cfg),
    "?subAcct=desk_1&ccy=BTC"
  )
  expect_equal(
    okxr::get_account_set_account_switch_precheck(acct_lv = 3, config = cfg),
    "?acctLv=3"
  )
  expect_equal(
    okxr::get_account_spot_borrow_repay_history(ccy = "USDT", type = "auto_repay", limit = 20, config = cfg),
    "?ccy=USDT&type=auto_repay&limit=20"
  )
  expect_equal(
    okxr::get_account_bills(ccy = "USDT", sub_type = "1", config = cfg),
    "?ccy=USDT&subType=1"
  )
  expect_equal(
    okxr::get_asset_deposit_history(ccy = "BTC", tx_id = "tx-1", state = "2", limit = 20, config = cfg),
    "?ccy=BTC&txId=tx-1&state=2&limit=20"
  )
  expect_equal(
    okxr::get_asset_withdrawal_history(ccy = "USDT", wd_id = "wd-1", client_id = "client-1", limit = 20, config = cfg),
    "?ccy=USDT&wdId=wd-1&clientId=client-1&limit=20"
  )
  expect_equal(
    okxr::get_asset_currencies(ccy = "BTC,ETH", config = cfg),
    "?ccy=BTC%2CETH"
  )
  expect_equal(
    okxr::get_asset_non_tradable_assets(ccy = "ETH", config = cfg),
    "?ccy=ETH"
  )
  expect_equal(
    okxr::get_asset_asset_valuation(ccy = "USD", config = cfg),
    "?ccy=USD"
  )
  expect_equal(
    okxr::get_asset_transfer_state(trans_id = "1", type = "0", config = cfg),
    "?transId=1&type=0"
  )
  expect_equal(
    okxr::get_asset_bills(ccy = "USDT", type = "2", limit = 10, config = cfg),
    "?ccy=USDT&type=2&limit=10"
  )
  expect_equal(
    okxr::get_asset_bills_history(ccy = "USDT", paging_type = "2", limit = 10, config = cfg),
    "?ccy=USDT&limit=10&pagingType=2"
  )
  expect_equal(
    okxr::get_asset_deposit_withdraw_status(wd_id = "200045249", config = cfg),
    "?wdId=200045249"
  )
  expect_equal(
    okxr::get_asset_exchange_list(),
    ""
  )
  expect_equal(
    okxr::get_asset_convert_currencies(config = cfg),
    ""
  )
  expect_equal(
    okxr::get_asset_convert_currency_pair(from_ccy = "USDT", to_ccy = "BTC", convert_mode = "1", config = cfg),
    "?fromCcy=USDT&toCcy=BTC&convertMode=1"
  )
  expect_equal(
    okxr::get_asset_convert_history(cl_t_req_id = "req-1", limit = 10, tag = "desk", config = cfg),
    "?clTReqId=req-1&limit=10&tag=desk"
  )
  expect_equal(
    okxr::get_trade_orders_history(inst_type = "SPOT", ord_type = "ioc", limit = 10, config = cfg),
    "?instType=SPOT&ordType=ioc&limit=10"
  )
  expect_equal(
    okxr::get_trade_orders_history_archive(inst_type = "SPOT", category = "delivery", config = cfg),
    "?instType=SPOT&category=delivery"
  )
  expect_equal(
    okxr::get_trade_easy_convert_currency_list(source = "funding", config = cfg),
    "?source=funding"
  )
  expect_equal(
    okxr::get_trade_easy_convert_history(limit = 20, config = cfg),
    "?limit=20"
  )
  expect_equal(
    okxr::get_trade_one_click_repay_currency_list(debt_type = "1", config = cfg),
    "?debtType=1"
  )
  expect_equal(
    okxr::get_trade_one_click_repay_history(limit = 20, config = cfg),
    "?limit=20"
  )
  expect_equal(
    okxr::get_trade_one_click_repay_currency_list_v2(config = cfg),
    ""
  )
  expect_equal(
    okxr::get_trade_one_click_repay_history_v2(limit = 20, config = cfg),
    "?limit=20"
  )
  expect_equal(
    okxr::get_trade_order_algo(algo_id = "123", config = cfg),
    "?algoId=123"
  )
  expect_equal(
    okxr::get_trade_orders_algo_pending(ord_type = "conditional", inst_id = "BTC-USDT", config = cfg),
    "?ordType=conditional&instId=BTC-USDT"
  )
  expect_equal(
    okxr::get_trade_orders_algo_history(ord_type = "conditional", state = "effective", config = cfg),
    "?ordType=conditional&state=effective"
  )
  expect_equal(
    okxr::get_trade_fills(inst_type = "SPOT", begin = "1000", end = "2000", limit = 10, config = cfg),
    "?instType=SPOT&limit=10&begin=1000&end=2000"
  )
  expect_equal(
    okxr::get_trade_fills_history(inst_type = "SPOT", limit = 10, config = cfg),
    "?instType=SPOT&limit=10"
  )
  expect_equal(
    okxr::get_trade_fills_history(inst_type = "SPOT", begin = "1000", end = "2000", limit = 10, config = cfg),
    "?instType=SPOT&limit=10&begin=1000&end=2000"
  )
  expect_equal(
    okxr::get_trade_account_rate_limit(config = cfg),
    ""
  )
})

test_that("post_trade_order preserves supplied client order id", {
  ns <- asNamespace("okxr")
  old_posts <- get(".posts", envir = ns)
  unlockBinding(".posts", ns)
  on.exit({
    assign(".posts", old_posts, envir = ns)
    lockBinding(".posts", ns)
  }, add = TRUE)
  assign(
    ".posts",
    list(
      trade_order = function(body_list, tz, config) body_list
    ),
    envir = ns
  )

  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")
  body <- okxr::post_trade_order(
    inst_id = "BTC-USDT",
    td_mode = "cash",
    side = "buy",
    ord_type = "market",
    sz = "1",
    cl_ord_id = "custom-id",
    config = cfg
  )

  expect_equal(body$clOrdId, "custom-id")
})

test_that("trade POST wrappers reject malformed identifier and batch inputs", {
  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")

  expect_error(
    okxr::post_trade_cancel_order(inst_id = "BTC-USDT", config = cfg),
    "Provide exactly one of `ord_id` or `cl_ord_id`"
  )
  expect_error(
    okxr::post_trade_batch_orders(orders = list(), config = cfg),
    "`orders` must be a non-empty list"
  )
  expect_error(
    okxr::post_trade_cancel_batch_orders(
      orders = list(list(inst_id = "BTC-USDT", ord_id = "1", cl_ord_id = "2")),
      config = cfg
    ),
    "Provide exactly one of `ord_id` or `cl_ord_id`"
  )
  expect_error(
    okxr::post_trade_amend_order(inst_id = "BTC-USDT", ord_id = "1", config = cfg),
    "must include at least one of"
  )
  expect_error(
    okxr::post_trade_amend_batch_orders(
      orders = list(list(inst_id = "BTC-USDT", ord_id = "1")),
      config = cfg
    ),
    "must include at least one of"
  )
  expect_error(
    okxr::post_trade_cancel_algos(
      orders = list(list(inst_id = "BTC-USDT", algo_id = "1", algo_cl_ord_id = "2")),
      config = cfg
    ),
    "Provide exactly one of `algo_id` or `algo_cl_ord_id`"
  )
  expect_error(
    okxr::post_trade_amend_algos(inst_id = "BTC-USDT", algo_id = "1", config = cfg),
    "must include at least one of"
  )
})

test_that("trade POST wrappers build expected request bodies", {
  ns <- asNamespace("okxr")
  old_posts <- get(".posts", envir = ns)
  unlockBinding(".posts", ns)
  on.exit({
    assign(".posts", old_posts, envir = ns)
    lockBinding(".posts", ns)
  }, add = TRUE)
  assign(
    ".posts",
    list(
      trade_cancel_order = function(body_list, tz, config) body_list,
      trade_batch_orders = function(body_list, tz, config) body_list,
      trade_cancel_batch_orders = function(body_list, tz, config) body_list,
      trade_amend_order = function(body_list, tz, config) body_list,
      trade_amend_batch_orders = function(body_list, tz, config) body_list,
      trade_order_precheck = function(body_list, tz, config) body_list,
      trade_cancel_all_after = function(body_list, tz, config) body_list,
      trade_cancel_algos = function(body_list, tz, config) body_list,
      trade_amend_algos = function(body_list, tz, config) body_list,
      trade_mass_cancel = function(body_list, tz, config) body_list
    ),
    envir = ns
  )

  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")

  cancel_order_body <- okxr::post_trade_cancel_order(
    inst_id = "BTC-USDT",
    cl_ord_id = "cancel-client-1",
    config = cfg
  )
  expect_equal(cancel_order_body$instId, "BTC-USDT")
  expect_equal(cancel_order_body$clOrdId, "cancel-client-1")

  batch_body <- okxr::post_trade_batch_orders(
    orders = list(
      list(inst_id = "BTC-USDT", td_mode = "cash", side = "buy", ord_type = "limit", px = "2.15", sz = "2", cl_ord_id = "b15"),
      list(inst_id = "BTC-USDT", td_mode = "cash", side = "buy", ord_type = "limit", px = "2.15", sz = "2", cl_ord_id = "b16")
    ),
    config = cfg
  )
  expect_equal(batch_body[[1]]$clOrdId, "b15")
  expect_equal(batch_body[[2]]$clOrdId, "b16")

  cancel_batch_body <- okxr::post_trade_cancel_batch_orders(
    orders = list(
      list(inst_id = "BTC-USDT", ord_id = "123"),
      list(inst_id = "BTC-USDT", cl_ord_id = "client-2")
    ),
    config = cfg
  )
  expect_equal(cancel_batch_body[[1]]$ordId, "123")
  expect_equal(cancel_batch_body[[2]]$clOrdId, "client-2")

  amend_body <- okxr::post_trade_amend_order(
    inst_id = "BTC-USDT",
    ord_id = "123",
    req_id = "amend-1",
    new_sz = "2",
    cxl_on_fail = TRUE,
    config = cfg
  )
  expect_equal(amend_body$reqId, "amend-1")
  expect_equal(amend_body$cxlOnFail, "true")

  amend_batch_body <- okxr::post_trade_amend_batch_orders(
    orders = list(
      list(inst_id = "BTC-USDT", ord_id = "123", new_sz = "2"),
      list(inst_id = "BTC-USDT", cl_ord_id = "client-2", new_px = "2.20")
    ),
    config = cfg
  )
  expect_equal(amend_batch_body[[1]]$newSz, "2")
  expect_equal(amend_batch_body[[2]]$newPx, "2.20")

  precheck_body <- okxr::post_trade_order_precheck(
    inst_id = "BTC-USDT",
    td_mode = "cash",
    side = "buy",
    ord_type = "limit",
    px = "2.15",
    sz = "2",
    reduce_only = TRUE,
    config = cfg
  )
  expect_equal(precheck_body$reduceOnly, "true")
  expect_equal(precheck_body$ordType, "limit")

  caa_body <- okxr::post_trade_cancel_all_after(
    time_out = 60,
    tag = "desk1",
    config = cfg
  )
  expect_equal(caa_body$timeOut, "60")
  expect_equal(caa_body$tag, "desk1")

  cancel_algos_body <- okxr::post_trade_cancel_algos(
    orders = list(
      list(inst_id = "BTC-USDT", algo_id = "algo-1"),
      list(inst_id = "BTC-USDT", algo_cl_ord_id = "algo-client-2")
    ),
    config = cfg
  )
  expect_equal(cancel_algos_body[[1]]$algoId, "algo-1")
  expect_equal(cancel_algos_body[[2]]$algoClOrdId, "algo-client-2")

  amend_algos_body <- okxr::post_trade_amend_algos(
    inst_id = "BTC-USDT",
    algo_id = "algo-1",
    req_id = "algo-amend-1",
    new_sz = "2",
    cxl_on_fail = TRUE,
    new_trigger_px = "62000",
    new_ord_px = "-1",
    config = cfg
  )
  expect_equal(amend_algos_body$algoId, "algo-1")
  expect_equal(amend_algos_body$reqId, "algo-amend-1")
  expect_equal(amend_algos_body$cxlOnFail, "true")
  expect_equal(amend_algos_body$newTriggerPx, "62000")

  mass_cancel_body <- okxr::post_trade_mass_cancel(
    inst_type = "OPTION",
    inst_family = "BTC-USD",
    lock_interval = "500",
    config = cfg
  )
  expect_equal(mass_cancel_body$instType, "OPTION")
  expect_equal(mass_cancel_body$instFamily, "BTC-USD")
  expect_equal(mass_cancel_body$lockInterval, "500")
})

test_that("account POST wrappers build expected request bodies", {
  ns <- asNamespace("okxr")
  old_posts <- get(".posts", envir = ns)
  unlockBinding(".posts", ns)
  on.exit({
    assign(".posts", old_posts, envir = ns)
    lockBinding(".posts", ns)
  }, add = TRUE)
  assign(
    ".posts",
    list(
      account_set_position_mode = function(body_list, tz, config) body_list,
      account_set_fee_type = function(body_list, tz, config) body_list,
      account_set_greeks = function(body_list, tz, config) body_list,
      account_set_auto_repay = function(body_list, tz, config) body_list,
      account_set_auto_loan = function(body_list, tz, config) body_list,
      account_set_account_level = function(body_list, tz, config) body_list,
      account_set_collateral_assets = function(body_list, tz, config) body_list
    ),
    envir = ns
  )

  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")

  pos_mode_body <- okxr::post_account_set_position_mode(
    pos_mode = "long_short_mode",
    config = cfg
  )
  expect_equal(pos_mode_body$posMode, "long_short_mode")

  fee_type_body <- okxr::post_account_set_fee_type(
    fee_type = "1",
    config = cfg
  )
  expect_equal(fee_type_body$feeType, "1")

  greeks_body <- okxr::post_account_set_greeks(
    greeks_type = "PA",
    config = cfg
  )
  expect_equal(greeks_body$greeksType, "PA")

  auto_repay_body <- okxr::post_account_set_auto_repay(
    auto_repay = TRUE,
    config = cfg
  )
  expect_identical(auto_repay_body$autoRepay, TRUE)

  auto_loan_body <- okxr::post_account_set_auto_loan(
    auto_loan = FALSE,
    config = cfg
  )
  expect_identical(auto_loan_body$autoLoan, FALSE)

  acct_lv_body <- okxr::post_account_set_account_level(
    acct_lv = "3",
    config = cfg
  )
  expect_equal(acct_lv_body$acctLv, "3")

  collateral_body <- okxr::post_account_set_collateral_assets(
    type = "custom",
    collateral_enabled = FALSE,
    ccy_list = c("BTC", "ETH"),
    config = cfg
  )
  expect_equal(collateral_body$type, "custom")
  expect_identical(collateral_body$collateralEnabled, FALSE)
  expect_equal(collateral_body$ccyList, c("BTC", "ETH"))
})

test_that("account operational POST wrappers build expected request bodies", {
  ns <- asNamespace("okxr")
  old_posts <- get(".posts", envir = ns)
  unlockBinding(".posts", ns)
  on.exit({
    assign(".posts", old_posts, envir = ns)
    lockBinding(".posts", ns)
  }, add = TRUE)
  assign(
    ".posts",
    list(
      account_position_margin_balance = function(body_list, tz, config) body_list,
      account_spot_manual_borrow_repay = function(body_list, tz, config) body_list,
      account_account_level_switch_preset = function(body_list, tz, config) body_list,
      account_mmp_reset = function(body_list, tz, config) body_list,
      account_mmp_config = function(body_list, tz, config) body_list,
      account_move_positions = function(body_list, tz, config) body_list
    ),
    envir = ns
  )

  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")

  expect_error(
    okxr::post_account_move_positions(from_acct = "0", to_acct = "sub1", legs = list(), client_id = "move-1", config = cfg),
    "`legs` must be a non-empty list"
  )

  margin_body <- okxr::post_account_position_margin_balance(
    inst_id = "BTC-USDT-SWAP",
    pos_side = "short",
    type = "add",
    amt = "1",
    ccy = "BTC",
    config = cfg
  )
  expect_equal(margin_body$instId, "BTC-USDT-SWAP")
  expect_equal(margin_body$posSide, "short")
  expect_equal(margin_body$type, "add")
  expect_equal(margin_body$ccy, "BTC")

  borrow_body <- okxr::post_account_spot_manual_borrow_repay(
    ccy = "USDT",
    side = "borrow",
    amt = "100",
    config = cfg
  )
  expect_equal(borrow_body$ccy, "USDT")
  expect_equal(borrow_body$side, "borrow")
  expect_equal(borrow_body$amt, "100")

  preset_body <- okxr::post_account_account_level_switch_preset(
    acct_lv = "2",
    lever = "10",
    risk_offset_type = "1",
    config = cfg
  )
  expect_equal(preset_body$acctLv, "2")
  expect_equal(preset_body$lever, "10")
  expect_equal(preset_body$riskOffsetType, "1")

  mmp_reset_body <- okxr::post_account_mmp_reset(
    inst_family = "BTC-USD",
    inst_type = "OPTION",
    config = cfg
  )
  expect_equal(mmp_reset_body$instFamily, "BTC-USD")
  expect_equal(mmp_reset_body$instType, "OPTION")

  mmp_config_body <- okxr::post_account_mmp_config(
    inst_family = "BTC-USD",
    time_interval = "5000",
    frozen_interval = "2000",
    qty_limit = "100",
    config = cfg
  )
  expect_equal(mmp_config_body$instFamily, "BTC-USD")
  expect_equal(mmp_config_body$timeInterval, "5000")
  expect_equal(mmp_config_body$frozenInterval, "2000")
  expect_equal(mmp_config_body$qtyLimit, "100")

  legs <- list(
    list(
      from = list(posId = "pos-1", side = "sell", sz = "1"),
      to = list(posSide = "net", tdMode = "cross")
    )
  )
  move_body <- okxr::post_account_move_positions(
    from_acct = "0",
    to_acct = "sub1",
    legs = legs,
    client_id = "move-1",
    config = cfg
  )
  expect_equal(move_body$fromAcct, "0")
  expect_equal(move_body$toAcct, "sub1")
  expect_equal(move_body$clientId, "move-1")
  expect_equal(move_body$legs[[1]]$from$posId, "pos-1")
  expect_equal(move_body$legs[[1]]$to$tdMode, "cross")
})

test_that("asset POST wrappers build expected request bodies", {
  ns <- asNamespace("okxr")
  old_posts <- get(".posts", envir = ns)
  unlockBinding(".posts", ns)
  on.exit({
    assign(".posts", old_posts, envir = ns)
    lockBinding(".posts", ns)
  }, add = TRUE)
  assign(
    ".posts",
    list(
      asset_transfer = function(body_list, tz, config) body_list,
      asset_withdrawal = function(body_list, tz, config) body_list,
      asset_cancel_withdrawal = function(body_list, tz, config) body_list,
      asset_convert_estimate_quote = function(body_list, tz, config) body_list,
      asset_convert_trade = function(body_list, tz, config) body_list
    ),
    envir = ns
  )

  cfg <- list(api_key = "key", secret_key = "secret", passphrase = "pass")

  expect_error(
    okxr::post_asset_transfer(ccy = "USDT", amt = "100", from = "6", to = "6", config = cfg),
    "`from` and `to` must be different account codes"
  )

  transfer_body <- okxr::post_asset_transfer(
    ccy = "USDT",
    amt = "100",
    from = "6",
    to = "18",
    type = "1",
    sub_acct = "desk-sub",
    loan_trans = TRUE,
    omit_pos_risk = FALSE,
    client_id = "transfer-1",
    config = cfg
  )
  expect_equal(transfer_body$ccy, "USDT")
  expect_equal(transfer_body$from, "6")
  expect_equal(transfer_body$to, "18")
  expect_identical(transfer_body$loanTrans, TRUE)
  expect_identical(transfer_body$omitPosRisk, FALSE)
  expect_equal(transfer_body$clientId, "transfer-1")

  rcvr_info <- list(
    walletType = "exchange",
    exchId = "binance",
    rcvrFirstName = "Lily",
    rcvrLastName = "Li"
  )
  withdrawal_body <- okxr::post_asset_withdrawal(
    ccy = "USDT",
    amt = "50",
    dest = "4",
    to_addr = "0xabc",
    chain = "USDT-ERC20",
    to_addr_type = "1",
    area_code = "852",
    rcvr_info = rcvr_info,
    client_id = "withdraw-1",
    config = cfg
  )
  expect_equal(withdrawal_body$ccy, "USDT")
  expect_equal(withdrawal_body$toAddr, "0xabc")
  expect_equal(withdrawal_body$chain, "USDT-ERC20")
  expect_equal(withdrawal_body$rcvrInfo$exchId, "binance")
  expect_equal(withdrawal_body$clientId, "withdraw-1")

  cancel_withdrawal_body <- okxr::post_asset_cancel_withdrawal(
    wd_id = "wd-1",
    config = cfg
  )
  expect_equal(cancel_withdrawal_body$wdId, "wd-1")

  estimate_quote_body <- okxr::post_asset_convert_estimate_quote(
    base_ccy = "BTC",
    quote_ccy = "USDT",
    side = "sell",
    rfq_sz = "0.1",
    rfq_sz_ccy = "BTC",
    cl_q_req_id = "quote-1",
    tag = "desk1",
    convert_mode = "cash",
    config = cfg
  )
  expect_equal(estimate_quote_body$baseCcy, "BTC")
  expect_equal(estimate_quote_body$quoteCcy, "USDT")
  expect_equal(estimate_quote_body$rfqSz, "0.1")
  expect_equal(estimate_quote_body$clQReqId, "quote-1")

  convert_trade_body <- okxr::post_asset_convert_trade(
    quote_id = "quote-id-1",
    base_ccy = "BTC",
    quote_ccy = "USDT",
    side = "sell",
    sz = "0.1",
    sz_ccy = "BTC",
    cl_t_req_id = "trade-1",
    tag = "desk1",
    convert_mode = "cash",
    config = cfg
  )
  expect_equal(convert_trade_body$quoteId, "quote-id-1")
  expect_equal(convert_trade_body$baseCcy, "BTC")
  expect_equal(convert_trade_body$szCcy, "BTC")
  expect_equal(convert_trade_body$clTReqId, "trade-1")
})

#' Get copy trading settings
#'
#' Retrieve your account's copy trading configuration.
#'
#' @details
#' Wraps `/api/v5/copytrading/copy-settings`. Returns one row with the
#' current copy mode and copy state for the given `unique_code`.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param uniqueCode Deprecated alias for `unique_code`.
#' @param instType Deprecated alias for `inst_type`.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with fields like `copyMode` and `copyState`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_copy_trade_settings(unique_code = "1129E65755274C36", config = cfg)
#' }
#'
#' @seealso [get_copy_trade_my_leaders()], [get_copy_trade_current_subpos()]
#' @note Since okxr 0.1.2
#' @export
get_copy_trade_settings <- function(unique_code, inst_type = NULL, config, tz = .okx_default_tz, uniqueCode = unique_code, instType = inst_type) {
  query_string <- .okx_build_query(uniqueCode = uniqueCode, instType = instType)
  .gets$copy_trade_settings(query_string = query_string, tz = tz, config = config)
}

#' Get my lead traders
#'
#' Retrieve the list of lead traders you are currently copying.
#'
#' @details
#' Wraps `/api/v5/copytrading/current-lead-traders`. Returns one row per
#' lead trader followed by your account.
#'
#' @param inst_type Character or `NULL`. Filter by instrument type
#'   (e.g., `"SWAP"`, `"MARGIN"`, `"SPOT"`). If `NULL`, returns all.
#' @param instType Deprecated alias for `inst_type`.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with fields such as `nickName` and `uniqueCode`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_copy_trade_my_leaders(inst_type = "SWAP", config = cfg)
#' }
#'
#' @seealso [get_copy_trade_settings()], [get_copy_trade_current_subpos()]
#' @note Since okxr 0.1.2
#' @export
get_copy_trade_my_leaders <- function(inst_type = NULL, config, tz = .okx_default_tz, instType = inst_type) {
  query_string <- .okx_build_query(instType = instType)
  .gets$copy_trade_my_leaders(query_string = query_string, tz = tz, config = config)
}

#' Get current copy trading subpositions
#'
#' Retrieve your currently active subpositions under copy trading.
#'
#' @details
#' Wraps `/api/v5/copytrading/current-subpositions`. Returns one row per
#' subposition, associated with the relevant lead trader.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param inst_id Character or `NULL`. Instrument ID filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with fields like `instId` and `uniqueCode`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' get_copy_trade_current_subpos(config = cfg)
#' }
#'
#' @seealso [get_copy_trade_historical_subpos()]
#' @note Since okxr 0.1.2
#' @export
get_copy_trade_current_subpos <- function(inst_type = NULL, inst_id = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instId = inst_id,
    after = after,
    before = before,
    limit = limit
  )
  .gets$copy_trade_current_subpos(query_string = query_string, tz = tz, config = config)
}

#' Get historical copy trading subpositions
#'
#' Retrieve your historical copy trading subpositions.
#'
#' @details
#' Wraps `/api/v5/copytrading/subpositions-history`. Returns one row per
#' closed or historical subposition.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param inst_id Character or `NULL`. Instrument ID filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return
#' A `data.frame` with fields like `instId` and `uniqueCode`.
#'
#' @examples
#' \dontrun{
#' cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
#' hist <- get_copy_trade_historical_subpos(config = cfg)
#' head(hist)
#' }
#'
#' @seealso [get_copy_trade_current_subpos()]
#' @note Since okxr 0.1.2
#' @export
get_copy_trade_historical_subpos <- function(inst_type = NULL, inst_id = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    instId = inst_id,
    after = after,
    before = before,
    limit = limit
  )
  .gets$copy_trade_historical_subpos(query_string = query_string, tz = tz, config = config)
}

#' Get copy trading instruments
#'
#' Retrieve instruments currently available for copy trading.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per instrument and an `enabled` flag.
#' @export
get_copy_trade_instruments <- function(
  inst_type = NULL,
  config,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$copy_trade_instruments(query_string = query_string, tz = tz, config = config)
}

#' Get copy trading config
#'
#' Retrieve your account-level copy trading configuration.
#'
#' @param config List. API credentials/config, typically containing
#'   `api_key`, `secret_key`, and `passphrase`.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with top-level copy trading configuration fields.
#'   Nested `details` are returned as a JSON string column.
#' @export
get_copy_trade_config <- function(config, tz = .okx_default_tz) {
  .gets$copy_trade_config(query_string = "", tz = tz, config = config)
}

#' Get public copy trading config
#'
#' Retrieve public copy trading limits and ratio bounds.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with public copy trading configuration fields.
#' @export
get_copy_trade_public_config <- function(
  inst_type = NULL,
  tz = .okx_default_tz
) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$copy_trade_public_config(query_string = query_string, tz = tz, config = NULL)
}

#' Get public copy trader summary
#'
#' Retrieve public copy trader summary metrics for a lead trader.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with summary metrics and a JSON-string `copyTraders`
#'   column for nested trader details.
#' @export
get_copy_trade_public_copy_traders <- function(unique_code, inst_type = NULL, limit = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(uniqueCode = unique_code, instType = inst_type, limit = limit)
  .gets$copy_trade_public_copy_traders(query_string = query_string, tz = tz, config = NULL)
}

#' Get public lead trader ranks
#'
#' Retrieve ranked public lead trader summaries.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param sort_type Character or `NULL`. Rank sort selector.
#' @param state Character or `NULL`. Lead trader state filter.
#' @param min_lead_days Character or `NULL`. Minimum lead-days selector.
#' @param min_assets Character or `NULL`. Minimum assets filter.
#' @param max_assets Character or `NULL`. Maximum assets filter.
#' @param min_aum Character or `NULL`. Minimum assets-under-management filter.
#' @param max_aum Character or `NULL`. Maximum assets-under-management filter.
#' @param data_ver Character or `NULL`. Data version selector used for pagination.
#' @param page Character or `NULL`. Page number.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with top-level ranking metadata and a JSON-string
#'   `ranks` column for nested leader rows.
#' @export
get_copy_trade_public_lead_traders <- function(inst_type = NULL, sort_type = NULL, state = NULL, min_lead_days = NULL, min_assets = NULL, max_assets = NULL, min_aum = NULL, max_aum = NULL, data_ver = NULL, page = NULL, limit = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    instType = inst_type,
    sortType = sort_type,
    state = state,
    minLeadDays = min_lead_days,
    minAssets = min_assets,
    maxAssets = max_assets,
    minAum = min_aum,
    maxAum = max_aum,
    dataVer = data_ver,
    page = page,
    limit = limit
  )
  .gets$copy_trade_public_lead_traders(query_string = query_string, tz = tz, config = NULL)
}

#' Get public current copy trading subpositions
#'
#' Retrieve public current subpositions for a lead trader.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per current public subposition.
#' @export
get_copy_trade_public_current_subpositions <- function(unique_code, inst_type = NULL, after = NULL, before = NULL, limit = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    uniqueCode = unique_code,
    instType = inst_type,
    after = after,
    before = before,
    limit = limit
  )
  .gets$copy_trade_public_current_subpositions(query_string = query_string, tz = tz, config = NULL)
}

#' Get public historical copy trading subpositions
#'
#' Retrieve public historical subpositions for a lead trader.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with one row per historical public subposition.
#' @export
get_copy_trade_public_subpositions_history <- function(unique_code, inst_type = NULL, after = NULL, before = NULL, limit = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(
    uniqueCode = unique_code,
    instType = inst_type,
    after = after,
    before = before,
    limit = limit
  )
  .gets$copy_trade_public_subpositions_history(query_string = query_string, tz = tz, config = NULL)
}

#' Get public copy trading pnl
#'
#' Retrieve public pnl time windows for a lead trader.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param last_days Character or numeric. OKX lookback selector.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with pnl windows and ratios.
#' @export
get_copy_trade_public_pnl <- function(unique_code, last_days, inst_type = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(uniqueCode = unique_code, lastDays = last_days, instType = inst_type)
  .gets$copy_trade_public_pnl(query_string = query_string, tz = tz, config = NULL)
}

#' Get public copy trading stats
#'
#' Retrieve public copy trading performance stats for a lead trader.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param last_days Character or numeric. OKX lookback selector.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with copy trading summary statistics.
#' @export
get_copy_trade_public_stats <- function(unique_code, last_days, inst_type = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(uniqueCode = unique_code, lastDays = last_days, instType = inst_type)
  .gets$copy_trade_public_stats(query_string = query_string, tz = tz, config = NULL)
}

#' Get public copy trading weekly pnl
#'
#' Retrieve public weekly pnl series for a lead trader.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with weekly pnl windows and ratios.
#' @export
get_copy_trade_public_weekly_pnl <- function(unique_code, inst_type = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(uniqueCode = unique_code, instType = inst_type)
  .gets$copy_trade_public_weekly_pnl(query_string = query_string, tz = tz, config = NULL)
}

#' Get public preference currencies
#'
#' Retrieve the most frequently traded currencies for a lead trader.
#'
#' @param unique_code Character. Lead trader unique code.
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with preferred currencies and their ratios.
#' @export
get_copy_trade_public_preference_currency <- function(unique_code, inst_type = NULL, tz = .okx_default_tz) {
  query_string <- .okx_build_query(uniqueCode = unique_code, instType = inst_type)
  .gets$copy_trade_public_preference_currency(query_string = query_string, tz = tz, config = NULL)
}

#' Get profit sharing details
#'
#' Retrieve realized profit sharing detail rows.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param after Character or `NULL`. Pagination cursor for earlier records.
#' @param before Character or `NULL`. Pagination cursor for newer records.
#' @param limit Integer or `NULL`. Number of rows to request.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with realized profit sharing rows.
#' @export
get_copy_trade_profit_sharing_details <- function(inst_type = NULL, after = NULL, before = NULL, limit = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type, after = after, before = before, limit = limit)
  .gets$copy_trade_profit_sharing_details(query_string = query_string, tz = tz, config = config)
}

#' Get unrealized profit sharing details
#'
#' Retrieve unrealized profit sharing detail rows.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with unrealized profit sharing rows.
#' @export
get_copy_trade_unrealized_profit_sharing_details <- function(inst_type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$copy_trade_unrealized_profit_sharing_details(query_string = query_string, tz = tz, config = config)
}

#' Get total profit sharing
#'
#' Retrieve total realized profit sharing by instrument type.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with total realized profit sharing rows.
#' @export
get_copy_trade_total_profit_sharing <- function(inst_type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$copy_trade_total_profit_sharing(query_string = query_string, tz = tz, config = config)
}

#' Get total unrealized profit sharing
#'
#' Retrieve total unrealized profit sharing by instrument type.
#'
#' @param inst_type Character or `NULL`. Instrument type filter.
#' @param config List. API credentials/config.
#' @param tz Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.
#'
#' @return A `data.frame` with total unrealized profit sharing rows.
#' @export
get_copy_trade_total_unrealized_profit_sharing <- function(inst_type = NULL, config, tz = .okx_default_tz) {
  query_string <- .okx_build_query(instType = inst_type)
  .gets$copy_trade_total_unrealized_profit_sharing(query_string = query_string, tz = tz, config = config)
}

#' Create a POST request function from an API spec
#'
#' Converts a single entry from \code{.api_POST_specs} into a function that sends
#' a POST request to the OKX API and returns the parsed response using the provided schema.
#'
#' @param api A list containing the POST API spec. Must include:
#'   \code{okx_path}, \code{schema}, and optional \code{mode}.
#'
#' @return A function with signature \code{(body_list, tz, config)} that:
#' \itemize{
#'   \item Converts the body list to JSON,
#'   \item Signs and sends the POST request,
#'   \item Parses and returns the response using the schema-aware parser.
#' }
#' @keywords internal
.make_post_function <- function(api) {
  parser <- .make_parser(api$parser_schema, mode = api$parser_mode %||% "named")

  function(body_list, tz, config, raw_data = getOption("okxr.raw_data", FALSE)) {
    res <- .execute_post_action(api$okx_path, body_list, config)
    if (is.null(res)) {
      return(NULL)
    }

    parsed_res <- parser(res = res, tz = tz)
    .okx_extract_result(parsed_res, raw_data = raw_data)
  }
}

#' @title POST request function list
#' @description A list of POST endpoint functions generated from \code{.api_POST_specs}.
#' Each function has the form \code{(body_list, tz, config)} and returns a parsed result.
#' @keywords internal
.posts <- lapply(.api_POST_specs, .make_post_function)

#---- Trade: POST Wrappers ----

.okx_compact_body <- function(body_list) {
  body_list[!vapply(body_list, is.null, logical(1))]
}

.okx_trade_order_body <- function(
  inst_id,
  td_mode,
  side,
  ord_type,
  sz,
  ccy = NULL,
  cl_ord_id = NULL,
  tag = NULL,
  pos_side = NULL,
  px = NULL,
  reduce_only = NULL,
  tgt_ccy = NULL,
  ban_amend = NULL,
  px_amend_type = NULL,
  trade_quote_ccy = NULL,
  stp_mode = NULL,
  attach_algo_ords = NULL,
  px_usd = NULL,
  px_vol = NULL,
  speed_bump = NULL,
  outcome = NULL,
  is_elp_taker_access = NULL
) {
  .okx_compact_body(list(
    instId = inst_id,
    tdMode = td_mode,
    side = side,
    ordType = ord_type,
    ccy = ccy,
    clOrdId = cl_ord_id,
    tag = tag,
    posSide = pos_side,
    px = px,
    sz = sz,
    reduceOnly = if (is.null(reduce_only)) NULL else tolower(as.character(reduce_only)),
    tgtCcy = tgt_ccy,
    banAmend = if (is.null(ban_amend)) NULL else tolower(as.character(ban_amend)),
    pxAmendType = px_amend_type,
    tradeQuoteCcy = trade_quote_ccy,
    stpMode = stp_mode,
    attachAlgoOrds = attach_algo_ords,
    pxUsd = px_usd,
    pxVol = px_vol,
    speedBump = speed_bump,
    outcome = outcome,
    isElpTakerAccess = if (is.null(is_elp_taker_access)) NULL else tolower(as.character(is_elp_taker_access))
  ))
}

.okx_trade_amend_body <- function(
  inst_id,
  ord_id = NULL,
  cl_ord_id = NULL,
  req_id = NULL,
  new_sz = NULL,
  new_px = NULL,
  cxl_on_fail = NULL,
  new_px_usd = NULL,
  new_px_vol = NULL,
  px_amend_type = NULL,
  attach_algo_ords = NULL,
  speed_bump = NULL
) {
  .okx_compact_body(list(
    instId = inst_id,
    ordId = ord_id,
    clOrdId = cl_ord_id,
    reqId = req_id,
    newSz = new_sz,
    newPx = new_px,
    cxlOnFail = if (is.null(cxl_on_fail)) NULL else tolower(as.character(cxl_on_fail)),
    newPxUsd = new_px_usd,
    newPxVol = new_px_vol,
    pxAmendType = px_amend_type,
    attachAlgoOrds = attach_algo_ords,
    speedBump = speed_bump
  ))
}

.okx_trade_algo_cancel_body <- function(
  inst_id,
  algo_id = NULL,
  algo_cl_ord_id = NULL
) {
  .okx_compact_body(list(
    instId = inst_id,
    algoId = algo_id,
    algoClOrdId = algo_cl_ord_id
  ))
}

.okx_trade_algo_amend_body <- function(
  inst_id,
  algo_id = NULL,
  algo_cl_ord_id = NULL,
  cxl_on_fail = NULL,
  req_id = NULL,
  new_sz = NULL,
  new_tp_trigger_px = NULL,
  new_tp_ord_px = NULL,
  new_sl_trigger_px = NULL,
  new_sl_ord_px = NULL,
  new_tp_trigger_px_type = NULL,
  new_sl_trigger_px_type = NULL,
  new_trigger_px = NULL,
  new_ord_px = NULL,
  new_trigger_px_type = NULL,
  attach_algo_ords = NULL
) {
  .okx_compact_body(list(
    instId = inst_id,
    algoId = algo_id,
    algoClOrdId = algo_cl_ord_id,
    cxlOnFail = if (is.null(cxl_on_fail)) NULL else tolower(as.character(cxl_on_fail)),
    reqId = req_id,
    newSz = new_sz,
    newTpTriggerPx = new_tp_trigger_px,
    newTpOrdPx = new_tp_ord_px,
    newSlTriggerPx = new_sl_trigger_px,
    newSlOrdPx = new_sl_ord_px,
    newTpTriggerPxType = new_tp_trigger_px_type,
    newSlTriggerPxType = new_sl_trigger_px_type,
    newTriggerPx = new_trigger_px,
    newOrdPx = new_ord_px,
    newTriggerPxType = new_trigger_px_type,
    attachAlgoOrds = attach_algo_ords
  ))
}

#' Place a Trade Order
#'
#' Submits a trade order to the OKX exchange.
#'
#' @param inst_id Instrument ID (e.g., \code{"BTC-USDT"}).
#' @param td_mode Trade mode: \code{"cross"} or \code{"isolated"}.
#' @param side Order side: \code{"buy"} or \code{"sell"}.
#' @param ord_type Order type: \code{"limit"}, \code{"market"}, etc.
#' @param sz Size of the order (quantity to buy/sell).
#' @param pos_side Optional. Position side: \code{"long"} or \code{"short"}.
#' @param px Optional. Price (required for limit orders).
#' @param reduce_only Optional. Logical flag to indicate a reduce-only order.
#' @param tgt_ccy Optional. Quote currency (e.g., \code{"base"}, \code{"quote"}).
#' @param cl_ord_id Optional. Custom client order ID (auto-generated if NULL).
#' @param tag Optional. Tag used for identifying the strategy or bot.
#' @param config A list with API credentials: \code{api_key}, \code{secret_key}, \code{passphrase}.
#' @param tz Timezone for parsing any timestamps (default: \code{"Asia/Hong_Kong"}).
#'
#' @return A \code{data.frame} containing fields like order ID, client order ID, and timestamp.
#'
#' @export
post_trade_order <- function(
  inst_id,
  td_mode,
  side,
  ord_type,
  sz,
  pos_side = NULL,
  px = NULL,
  reduce_only = NULL,
  tgt_ccy = NULL,
  cl_ord_id = NULL,
  tag = NULL,
  config,
  tz = .okx_default_tz
) {
  if (is.null(cl_ord_id)) {
    cl_ord_id <- .okx_generate_client_order_id()
  }

  body_list <- .okx_trade_order_body(
    inst_id = inst_id,
    td_mode = td_mode,
    side = side,
    ord_type = ord_type,
    sz = sz,
    pos_side = pos_side,
    px = px,
    reduce_only = reduce_only,
    tgt_ccy = tgt_ccy,
    cl_ord_id = cl_ord_id,
    tag = tag
  )

  .posts$trade_order(body_list = body_list, tz = tz, config = config)
}

#' Cancel a Trade Order
#'
#' Submits a cancellation request for a previously placed trade order.
#'
#' @param inst_id Instrument ID (e.g., \code{"BTC-USDT"}).
#' @param ord_id Optional OKX order ID to cancel.
#' @param cl_ord_id Optional client order ID to cancel. Provide this or
#'   `ord_id`.
#' @param config A list with API credentials: \code{api_key}, \code{secret_key}, \code{passphrase}.
#' @param tz Timezone for parsing any timestamps (default: \code{"Asia/Hong_Kong"}).
#'
#' @return A \code{data.frame} containing cancellation result and timestamp.
#'
#' @export
post_trade_cancel_order <- function(inst_id, ord_id = NULL, cl_ord_id = NULL, config, tz = .okx_default_tz) {
  .okx_assert_exactly_one_present(ord_id, cl_ord_id, names = c("ord_id", "cl_ord_id"))
  .posts$trade_cancel_order(
    body_list = .okx_compact_body(list(instId = inst_id, ordId = ord_id, clOrdId = cl_ord_id)),
    tz = tz,
    config = config
  )
}

#' Close a Position
#'
#' Submits a request to close a position for a given instrument and position side.
#'
#' @param inst_id Instrument ID (e.g., \code{"BTC-USDT"}).
#' @param mgn_mode Margin mode: \code{"cross"} or \code{"isolated"}.
#' @param pos_side Position side to close: \code{"long"} or \code{"short"}.
#' @param tz Timezone for parsing any timestamps (default: \code{"Asia/Hong_Kong"}).
#' @param config A list with API credentials: \code{api_key}, \code{secret_key}, \code{passphrase}.
#'
#' @return A \code{data.frame} with close position confirmation details.
#'
#' @export
post_trade_close_position <- function(inst_id, mgn_mode, pos_side, tz = .okx_default_tz, config) {
  .posts$trade_close_position(body_list = list(instId = inst_id, mgnMode = mgn_mode, posSide = pos_side), tz = tz, config = config)
}

#' Place Multiple Trade Orders
#'
#' Submit multiple trade orders in one request.
#'
#' @param orders List of order specs using the snake_case names from
#'   [post_trade_order()].
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` with one row per submitted order result.
#' @export
post_trade_batch_orders <- function(orders, config, tz = .okx_default_tz) {
  .okx_assert_non_empty_list(orders, "orders")
  body_list <- lapply(
    seq_along(orders),
    function(i) {
      order <- orders[[i]]
      .okx_assert_has_fields(order, c("inst_id", "td_mode", "side", "ord_type", "sz"), paste0("orders[[", i, "]]"))
      if (is.null(order$cl_ord_id)) {
        order$cl_ord_id <- .okx_generate_client_order_id()
      }
      do.call(.okx_trade_order_body, order)
    }
  )
  .posts$trade_batch_orders(body_list = body_list, tz = tz, config = config)
}

#' Cancel Multiple Trade Orders
#'
#' Submit a batch cancellation request for incomplete orders.
#'
#' @param orders List of cancellation specification lists containing `inst_id`
#'   plus either `ord_id` or `cl_ord_id`.
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` with one row per cancellation result.
#' @export
post_trade_cancel_batch_orders <- function(
  orders,
  config,
  tz = .okx_default_tz
) {
  .okx_assert_non_empty_list(orders, "orders")
  body_list <- lapply(
    seq_along(orders),
    function(i) {
      order <- orders[[i]]
      .okx_assert_has_fields(order, "inst_id", paste0("orders[[", i, "]]"))
      .okx_assert_exactly_one_present(order$ord_id, order$cl_ord_id, names = c("ord_id", "cl_ord_id"))
      .okx_compact_body(list(
        instId = order$inst_id,
        ordId = order$ord_id,
        clOrdId = order$cl_ord_id
      ))
    }
  )
  .posts$trade_cancel_batch_orders(body_list = body_list, tz = tz, config = config)
}

#' Amend a Trade Order
#'
#' Submit an amendment request for an incomplete order.
#'
#' @param inst_id Instrument ID.
#' @param ord_id Order ID. Optional if `cl_ord_id` is supplied.
#' @param cl_ord_id Client order ID. Optional if `ord_id` is supplied.
#' @param req_id Optional client amendment request ID.
#' @param new_sz Optional new total order size.
#' @param new_px Optional new price.
#' @param cxl_on_fail Optional logical. Whether to cancel the order if the
#'   amendment fails.
#' @param new_px_usd Optional new option order USD price.
#' @param new_px_vol Optional new option order implied volatility price.
#' @param px_amend_type Optional price amendment mode.
#' @param attach_algo_ords Optional attached TP/SL amendment list.
#' @param speed_bump Optional event-contract speed bump.
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` describing the amendment request result.
#' @export
post_trade_amend_order <- function(inst_id, ord_id = NULL, cl_ord_id = NULL, req_id = NULL, new_sz = NULL, new_px = NULL, cxl_on_fail = NULL, new_px_usd = NULL, new_px_vol = NULL, px_amend_type = NULL, attach_algo_ords = NULL, speed_bump = NULL, config, tz = .okx_default_tz) {
  .okx_assert_exactly_one_present(ord_id, cl_ord_id, names = c("ord_id", "cl_ord_id"))
  .okx_assert_any_field_present(
    list(
      new_sz = new_sz,
      new_px = new_px,
      cxl_on_fail = cxl_on_fail,
      new_px_usd = new_px_usd,
      new_px_vol = new_px_vol,
      px_amend_type = px_amend_type,
      attach_algo_ords = attach_algo_ords,
      speed_bump = speed_bump
    ),
    c("new_sz", "new_px", "cxl_on_fail", "new_px_usd", "new_px_vol", "px_amend_type", "attach_algo_ords", "speed_bump"),
    "amendment request"
  )
  body_list <- .okx_trade_amend_body(
    inst_id = inst_id,
    ord_id = ord_id,
    cl_ord_id = cl_ord_id,
    req_id = req_id,
    new_sz = new_sz,
    new_px = new_px,
    cxl_on_fail = cxl_on_fail,
    new_px_usd = new_px_usd,
    new_px_vol = new_px_vol,
    px_amend_type = px_amend_type,
    attach_algo_ords = attach_algo_ords,
    speed_bump = speed_bump
  )
  .posts$trade_amend_order(body_list = body_list, tz = tz, config = config)
}

#' Amend Multiple Trade Orders
#'
#' Submit multiple amendment requests in one request.
#'
#' @param orders List of amendment specs using the names from
#'   [post_trade_amend_order()].
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` with one row per amendment result.
#' @export
post_trade_amend_batch_orders <- function(
  orders,
  config,
  tz = .okx_default_tz
) {
  .okx_assert_non_empty_list(orders, "orders")
  body_list <- lapply(
    seq_along(orders),
    function(i) {
      order <- orders[[i]]
      .okx_assert_has_fields(order, "inst_id", paste0("orders[[", i, "]]"))
      .okx_assert_exactly_one_present(order$ord_id, order$cl_ord_id, names = c("ord_id", "cl_ord_id"))
      .okx_assert_any_field_present(
        order,
        c("new_sz", "new_px", "cxl_on_fail", "new_px_usd", "new_px_vol", "px_amend_type", "attach_algo_ords", "speed_bump"),
        paste0("orders[[", i, "]]")
      )
      do.call(.okx_trade_amend_body, order)
    }
  )
  .posts$trade_amend_batch_orders(body_list = body_list, tz = tz, config = config)
}

#' Precheck a Trade Order
#'
#' Submit an order precheck request without placing the order.
#'
#' @param inst_id Instrument ID.
#' @param td_mode Trade mode.
#' @param side Order side.
#' @param ord_type Order type.
#' @param sz Order size.
#' @param ccy Optional margin currency.
#' @param cl_ord_id Optional client order ID.
#' @param tag Optional order tag.
#' @param pos_side Optional position side.
#' @param px Optional order price.
#' @param reduce_only Optional logical reduce-only flag.
#' @param tgt_ccy Optional target currency mode.
#' @param attach_algo_ords Optional attached TP/SL list.
#' @param speed_bump Optional event-contract speed bump.
#' @param outcome Optional event-contract outcome.
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` with projected account metrics after the precheck.
#' @export
post_trade_order_precheck <- function(inst_id, td_mode, side, ord_type, sz, ccy = NULL, cl_ord_id = NULL, tag = NULL, pos_side = NULL, px = NULL, reduce_only = NULL, tgt_ccy = NULL, attach_algo_ords = NULL, speed_bump = NULL, outcome = NULL, config, tz = .okx_default_tz) {
  body_list <- .okx_trade_order_body(
    inst_id = inst_id,
    td_mode = td_mode,
    side = side,
    ord_type = ord_type,
    sz = sz,
    ccy = ccy,
    cl_ord_id = cl_ord_id,
    tag = tag,
    pos_side = pos_side,
    px = px,
    reduce_only = reduce_only,
    tgt_ccy = tgt_ccy,
    attach_algo_ords = attach_algo_ords,
    speed_bump = speed_bump,
    outcome = outcome
  )
  .posts$trade_order_precheck(body_list = body_list, tz = tz, config = config)
}

#' Set Cancel-All-After
#'
#' Set or disable the cancel-all-after countdown.
#'
#' @param time_out Character or numeric. Countdown in seconds. `0` disables it.
#' @param tag Optional cancel-all-after tag scope.
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` with the configured trigger time and tag.
#' @export
post_trade_cancel_all_after <- function(
  time_out,
  tag = NULL,
  config,
  tz = .okx_default_tz
) {
  body_list <- .okx_compact_body(list(timeOut = as.character(time_out), tag = tag))
  .posts$trade_cancel_all_after(body_list = body_list, tz = tz, config = config)
}

#' Cancel Multiple Algo Orders
#'
#' Cancel up to 10 unfilled algo orders in one request.
#'
#' @param orders List of cancellation specification lists containing `inst_id`
#'   plus either `algo_id` or `algo_cl_ord_id`.
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` with one row per algo cancellation result.
#' @export
post_trade_cancel_algos <- function(orders, config, tz = .okx_default_tz) {
  .okx_assert_non_empty_list(orders, "orders")
  body_list <- lapply(
    seq_along(orders),
    function(i) {
      order <- orders[[i]]
      .okx_assert_has_fields(order, "inst_id", paste0("orders[[", i, "]]"))
      .okx_assert_exactly_one_present(order$algo_id, order$algo_cl_ord_id, names = c("algo_id", "algo_cl_ord_id"))
      do.call(.okx_trade_algo_cancel_body, order)
    }
  )
  .posts$trade_cancel_algos(body_list = body_list, tz = tz, config = config)
}

#' Amend an Algo Order
#'
#' Amend a supported unfilled algo order.
#'
#' @param inst_id Instrument ID.
#' @param algo_id Algo order ID. Optional if `algo_cl_ord_id` is supplied.
#' @param algo_cl_ord_id Client-supplied algo ID. Optional if `algo_id` is supplied.
#' @param cxl_on_fail Optional logical. Whether to cancel the order if the
#'   amendment fails.
#' @param req_id Optional client amendment request ID.
#' @param new_sz Optional new quantity after amendment.
#' @param new_tp_trigger_px Optional new take-profit trigger price.
#' @param new_tp_ord_px Optional new take-profit order price.
#' @param new_sl_trigger_px Optional new stop-loss trigger price.
#' @param new_sl_ord_px Optional new stop-loss order price.
#' @param new_tp_trigger_px_type Optional new take-profit trigger price type.
#' @param new_sl_trigger_px_type Optional new stop-loss trigger price type.
#' @param new_trigger_px Optional new trigger price for trigger orders.
#' @param new_ord_px Optional new order price for trigger orders.
#' @param new_trigger_px_type Optional new trigger price type for trigger orders.
#' @param attach_algo_ords Optional attached TP/SL amendment list.
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` describing the algo amendment result.
#' @export
post_trade_amend_algos <- function(inst_id, algo_id = NULL, algo_cl_ord_id = NULL, cxl_on_fail = NULL, req_id = NULL, new_sz = NULL, new_tp_trigger_px = NULL, new_tp_ord_px = NULL, new_sl_trigger_px = NULL, new_sl_ord_px = NULL, new_tp_trigger_px_type = NULL, new_sl_trigger_px_type = NULL, new_trigger_px = NULL, new_ord_px = NULL, new_trigger_px_type = NULL, attach_algo_ords = NULL, config, tz = .okx_default_tz) {
  .okx_assert_exactly_one_present(algo_id, algo_cl_ord_id, names = c("algo_id", "algo_cl_ord_id"))
  .okx_assert_any_field_present(
    list(
      cxl_on_fail = cxl_on_fail,
      new_sz = new_sz,
      new_tp_trigger_px = new_tp_trigger_px,
      new_tp_ord_px = new_tp_ord_px,
      new_sl_trigger_px = new_sl_trigger_px,
      new_sl_ord_px = new_sl_ord_px,
      new_tp_trigger_px_type = new_tp_trigger_px_type,
      new_sl_trigger_px_type = new_sl_trigger_px_type,
      new_trigger_px = new_trigger_px,
      new_ord_px = new_ord_px,
      new_trigger_px_type = new_trigger_px_type,
      attach_algo_ords = attach_algo_ords
    ),
    c("cxl_on_fail", "new_sz", "new_tp_trigger_px", "new_tp_ord_px", "new_sl_trigger_px", "new_sl_ord_px", "new_tp_trigger_px_type", "new_sl_trigger_px_type", "new_trigger_px", "new_ord_px", "new_trigger_px_type", "attach_algo_ords"),
    "algo amendment request"
  )
  body_list <- .okx_trade_algo_amend_body(
    inst_id = inst_id,
    algo_id = algo_id,
    algo_cl_ord_id = algo_cl_ord_id,
    cxl_on_fail = cxl_on_fail,
    req_id = req_id,
    new_sz = new_sz,
    new_tp_trigger_px = new_tp_trigger_px,
    new_tp_ord_px = new_tp_ord_px,
    new_sl_trigger_px = new_sl_trigger_px,
    new_sl_ord_px = new_sl_ord_px,
    new_tp_trigger_px_type = new_tp_trigger_px_type,
    new_sl_trigger_px_type = new_sl_trigger_px_type,
    new_trigger_px = new_trigger_px,
    new_ord_px = new_ord_px,
    new_trigger_px_type = new_trigger_px_type,
    attach_algo_ords = attach_algo_ords
  )
  .posts$trade_amend_algos(body_list = body_list, tz = tz, config = config)
}

#' Mass Cancel MMP Orders
#'
#' Cancel all MMP pending orders for an options instrument family.
#'
#' @param inst_type Instrument type. Currently `OPTION`.
#' @param inst_family Instrument family.
#' @param lock_interval Optional lock interval in milliseconds.
#' @param config A list with API credentials.
#' @param tz Timezone for parsing response timestamps.
#'
#' @return A `data.frame` with the request result.
#' @export
post_trade_mass_cancel <- function(inst_type, inst_family, lock_interval = NULL, config, tz = .okx_default_tz) {
  body_list <- .okx_compact_body(list(
    instType = inst_type,
    instFamily = inst_family,
    lockInterval = lock_interval
  ))
  .posts$trade_mass_cancel(body_list = body_list, tz = tz, config = config)
}

#' Build a typed OKX JSON response parser (named or positional) to data.table
#'
#' Constructs and returns a parser function that converts an OKX REST API JSON
#' response into a typed \code{data.table}, using a provided field schema and a
#' parsing \code{mode}. Supports both key-based (\code{"named"}) and index-based
#' (\code{"positional"}) endpoints. For \code{"named"} endpoints returning a
#' single object, the parser wraps it as a one-row table.
#'
#' @param schema A \code{data.frame} with \code{okx}, \code{formal}, and
#'   \code{type} columns. Supported types are time, numeric, integer, string,
#'   and logical.
#' @param mode Parser mode: \code{"named"}, \code{"positional"}, or
#'   \code{"vector"}.
#'
#' @return A function with signature \code{function(res, tz)} where:
#'   \describe{
#'     \item{\code{res}}{An \code{httr::response}. The body must decode to a list
#'       with \code{$code}, \code{$msg}, and \code{$data}.}
#'     \item{\code{tz}}{Time zone for \code{"time"} fields. Millisecond values
#'       are converted with \code{as.POSIXct()}.}
#'   }
#'   The returned parser yields a \code{data.table} with column names from
#'   \code{schema$okx} and attaches variable labels as
#'   \code{attr(DT, "var_labels")} (a named character vector \code{formal} by \code{okx}).
#'   Returns \code{NULL} if the API \code{code} is not \code{"0"} or if \code{$data}
#'   is empty.
#'
#' @details
#' \itemize{
#'   \item \strong{Typing}: Columns are preallocated per \code{schema$type}. Time fields
#'   are interpreted as UNIX \emph{milliseconds}.
#'   \item \strong{Modes}:
#'     \itemize{
#'       \item \code{"named"} - fields accessed via \code{okx} keys; a single object
#'         in \code{$data} is wrapped to one row.
#'       \item \code{"positional"} - fields accessed by index order of \code{schema}.
#'       \item \code{"vector"} - scalar or vector payloads returned in one column.
#'     }
#'   \item \strong{Attributes}: \code{attr(DT, "var_labels")} maps \code{okx} to \code{formal}.
#' }
#'
#' @section Errors & warnings:
#' If \code{parsed$code != "0"}, a warning with \code{parsed$msg} is emitted and
#' \code{NULL} is returned.
#' @importFrom httr content
#' @importFrom data.table as.data.table
#' @keywords internal
.make_parser <- function(schema, mode = c("named", "positional", "vector")) {
  mode <- match.arg(mode)

  function(res, tz) {
    if (is.null(res)) {
      return(NULL)
    }

    parsed <- tryCatch(
      httr::content(res, as = "parsed", type = "application/json"),
      error = function(err) {
        warning("Response parsing failed: ", conditionMessage(err), call. = FALSE)
        NULL
      }
    )
    if (is.null(parsed)) {
      return(NULL)
    }

    if (parsed$code != "0") {
      warning("Request failed: ", parsed$msg %||% "unknown OKX API error", call. = FALSE)
      return(NULL)
    }

    data_list <- parsed$data
    if (length(data_list) == 0) return(NULL)

    if (mode == "vector") {
      okx_key <- schema$okx[[1]]
      formal_name <- schema$formal[[1]]
      values <- unlist(data_list, use.names = FALSE)
      DT <- data.table::as.data.table(stats::setNames(list(as.character(values)), okx_key))
      attr(DT, "var_labels") <- stats::setNames(formal_name, okx_key)
      return(list(data_raw = data_list, data_dt = DT))
    }

    # If it's a named single-entry, wrap in a list
    if (mode == "named" && is.list(data_list) && !is.list(data_list[[1]])) {
      data_list <- list(data_list)
    }

    okx_keys  <- schema$okx
    col_names <- schema$formal
    col_types <- schema$type

    .extract_column <- function(index) {
      raw_vals <- lapply(
        data_list,
        function(entry) {
          if (mode == "named") entry[[okx_keys[[index]]]] else entry[[index]]
        }
      )

      type <- col_types[[index]]
      switch(
        type,
        time = {
          seconds <- vapply(
            raw_vals,
            function(raw_val) {
              if (is.null(raw_val)) {
                return(NA_real_)
              }
              suppressWarnings(as.numeric(raw_val) / 1000)
            },
            numeric(1)
          )
          as.POSIXct(seconds, origin = "1970-01-01", tz = tz)
        },
        numeric = vapply(
          raw_vals,
          function(raw_val) {
            if (is.null(raw_val)) NA_real_ else suppressWarnings(as.numeric(raw_val))
          },
          numeric(1)
        ),
        integer = vapply(
          raw_vals,
          function(raw_val) {
            if (is.null(raw_val)) NA_integer_ else suppressWarnings(as.integer(raw_val))
          },
          integer(1)
        ),
        string = vapply(
          raw_vals,
          function(raw_val) {
            if (is.null(raw_val)) {
              return(NA_character_)
            }

            if (is.list(raw_val)) {
              return(as.character(jsonlite::toJSON(raw_val, auto_unbox = TRUE)))
            }

            as.character(raw_val)
          },
          character(1)
        ),
        logical = vapply(
          raw_vals,
          function(raw_val) {
            if (is.null(raw_val)) {
              return(NA)
            }

            if (is.logical(raw_val)) {
              return(raw_val)
            }

            raw_text <- toupper(as.character(raw_val))
            if (identical(raw_text, "TRUE")) return(TRUE)
            if (identical(raw_text, "FALSE")) return(FALSE)
            NA
          },
          logical(1)
        ),
        raw_vals
      )
    }

    cols <- stats::setNames(lapply(seq_along(okx_keys), .extract_column), okx_keys)
    DT <- data.table::as.data.table(cols)

    attr(DT, "var_labels") <- stats::setNames(col_names, okx_keys)
    list(
      data_raw = data_list,
      data_dt = DT
    )
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_deposit_history}
\alias{get_asset_deposit_history}
\title{Get asset deposit history}
\usage{
get_asset_deposit_history(
  ccy = NULL,
  dep_id = NULL,
  from_wd_id = NULL,
  tx_id = NULL,
  type = NULL,
  state = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter.}

\item{dep_id}{Character or `NULL`. Deposit ID filter.}

\item{from_wd_id}{Character or `NULL`. Source withdrawal ID filter.}

\item{tx_id}{Character or `NULL`. Transaction hash filter.}

\item{type}{Character or `NULL`. Deposit type filter.}

\item{state}{Character or `NULL`. Deposit state filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key}, and \code{passphrase}.}

\item{tz}{Timezone string (default: \code{"Asia/Hong_Kong"}).}
}
\value{
A \code{data.frame} with deposit history rows and detailed transfer metadata.
}
\description{
Retrieves a record of all asset deposits made to your account.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_balance}
\alias{get_account_balance}
\title{Get account balance}
\usage{
get_account_balance(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`. May also include `base_url`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with account balance and margin metrics (e.g.,
`totalEq`, `isoEq`, `adjEq`, `availEq`, `ordFroz`, `imr`, `mmr`, `upl`,
`mgnRatio`, …). Timestamp columns (`uTime`) are `POSIXct`.
}
\description{
Retrieve account-level margin and equity information for your OKX account.
}
\details{
This wraps `/api/v5/account/balance`. Returns one row per account-level
equity snapshot. Timestamps are parsed into `POSIXct` in the given `tz`.
}
\note{
Since okxr 0.1.1
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
bal <- get_account_balance(config = cfg)
head(bal)
}

}
\seealso{
[get_account_positions()], [get_account_leverage_info()]

Other okxr-account: 
\code{\link{get_account_config}()},
\code{\link{get_account_leverage_info}()},
\code{\link{get_account_positions_history}()},
\code{\link{get_account_positions}()}
}
\concept{okxr-account}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_parser.R
\name{.make_parser}
\alias{.make_parser}
\title{Build a typed OKX JSON response parser (named or positional) → data.table}
\usage{
.make_parser(schema, mode = c("named", "positional"))
}
\arguments{
\item{schema}{A \code{data.frame} describing the response fields with columns:
\describe{
  \item{\code{okx}}{Field name in the raw JSON response (used as output column names).}
  \item{\code{formal}}{Human-readable label (stored in \code{attr(, "var_labels")}).}
  \item{\code{type}}{One of \code{"time"}, \code{"numeric"}, \code{"integer"},
    \code{"string"}, \code{"logical"}.}
}}

\item{mode}{Parsing mode; either \code{"named"} (default) for key-based access,
or \code{"positional"} for index-based access.}
}
\value{
A function with signature \code{function(res, tz)} where:
  \describe{
    \item{\code{res}}{An \code{httr::response}. The body must decode to a list
      with \code{$code}, \code{$msg}, and \code{$data}.}
    \item{\code{tz}}{Timezone string used for \code{"time"} fields. Millisecond
      timestamps are converted via \code{as.POSIXct(ms/1000, tz=tz)}.}
  }
  The returned parser yields a \code{data.table} with column names from
  \code{schema$okx} and attaches variable labels as
  \code{attr(DT, "var_labels")} (a named character vector \code{formal} by \code{okx}).
  Returns \code{NULL} if the API \code{code} is not \code{"0"} or if \code{$data}
  is empty.
}
\description{
Constructs and returns a parser function that converts an OKX REST API JSON
response into a typed \code{data.table}, using a provided field schema and a
parsing \code{mode}. Supports both key-based (\code{"named"}) and index-based
(\code{"positional"}) endpoints. For \code{"named"} endpoints returning a
single object, the parser wraps it as a one-row table.
}
\details{
\itemize{
  \item \strong{Typing}: Columns are preallocated per \code{schema$type}. Time fields
  are interpreted as UNIX \emph{milliseconds}.
  \item \strong{Modes}:
    \itemize{
      \item \code{"named"} — fields accessed via \code{okx} keys; a single object
        in \code{$data} is wrapped to one row.
      \item \code{"positional"} — fields accessed by index order of \code{schema}.
    }
  \item \strong{Attributes}: \code{attr(DT, "var_labels")} maps \code{okx} → \code{formal}.
}
}
\section{Errors & warnings}{

If \code{parsed$code != "0"}, a warning with \code{parsed$msg} is emitted and
\code{NULL} is returned.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_auth.R
\name{.build_request}
\alias{.build_request}
\title{Build a full OKX request object}
\usage{
.build_request(
  httr_method,
  base_url,
  api_path,
  query_string,
  config = NULL,
  body_json = "",
  auth = TRUE
)
}
\arguments{
\item{httr_method}{HTTP method ("GET" or "POST").}

\item{base_url}{Base URL of the OKX API.}

\item{api_path}{API path (e.g., "/api/v5/account/balance").}

\item{query_string}{Query string starting with "?" or empty.}

\item{config}{List with API credentials.}

\item{body_json}{Optional JSON string for POST body.}

\item{auth}{Logical. Whether to sign the request with OKX credentials.}
}
\value{
A list with elements: `method`, `url`, `full_path`, `headers`, and `body_json`.
}
\description{
Assemble URL, headers, and body for an OKX API call.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_position_tiers}
\alias{get_account_position_tiers}
\title{Get account position tiers}
\usage{
get_account_position_tiers(
  inst_type,
  inst_family,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type: `"SWAP"`, `"FUTURES"`, or
`"OPTION"`.}

\item{inst_family}{Character. One instrument family or a comma-separated list
of up to five families.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with instrument-family position limits.
}
\description{
Retrieve portfolio-margin position limits for one or more instrument
families.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_labels.R
\name{get_var_label}
\alias{get_var_label}
\title{Retrieve variable labels from OKX data frames}
\usage{
get_var_label(df, var = NULL, default = NA_character_)
}
\arguments{
\item{df}{A \code{data.frame} with \code{"var_labels"} attribute (as returned by OKX API parsers).}

\item{var}{Optional variable name (character) or index (numeric). If \code{NULL}, all labels are returned.}

\item{default}{Value to return if the variable has no label (default: \code{NA_character_}).}
}
\value{
A character label if \code{var} is provided, or a named character vector of all labels if \code{var = NULL}.
}
\description{
Returns human-readable labels attached to a data frame produced by OKX API parsers.
You can retrieve all labels or a label for a specific variable by name or index.
}
\examples{
df <- data.frame(ordId = "123", px = 10)
attr(df, "var_labels") <- c(ordId = "Order ID", px = "Price")

get_var_label(df, "ordId")
get_var_label(df, 2)
get_var_label(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_amend_batch_orders}
\alias{post_trade_amend_batch_orders}
\title{Amend Multiple Trade Orders}
\usage{
post_trade_amend_batch_orders(
  orders,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{orders}{List of amendment specs.
See \code{post_trade_amend_order()}.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with one row per amendment result.
}
\description{
Submit multiple amendment requests in one request.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_order}
\alias{get_trade_order}
\title{Get trade order details}
\usage{
get_trade_order(
  inst_id,
  ord_id = NULL,
  cl_ord_id = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"ETH-USDT-SWAP"` (perps),
`"BTC-USDT"` (spot), or `"BTC-USD-240927"` (dated futures).}

\item{ord_id}{Character, optional. The OKX order ID. Provide this **or** `cl_ord_id`.}

\item{cl_ord_id}{Character, optional. Your client order ID. Provide this **or** `ord_id`.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`. May also include `base_url`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` (one row) with order details following OKX schema (e.g.,
`ordId`, `clOrdId`, `instId`, `ordType`, `px`, `sz`, `side`, `posSide`,
`tdMode`, `accFillSz`, `fillPx`, `fillSz`, `fillTime`, `avgPx`, `state`,
`lever`, etc.). Timestamp columns are `POSIXct` in `tz`.
}
\description{
Retrieve detailed information about a specific OKX order by either the
exchange-assigned `ord_id` or your client-assigned `cl_ord_id`.
}
\details{
You must provide exactly one identifier: `ord_id` **or** `cl_ord_id`.
Timestamps in the response are converted to `POSIXct` in the supplied `tz`.
}
\note{
Since okxr 0.1.1
}
\section{Common errors}{

- `Either 'ord_id' or 'cl_ord_id' must be provided.` (client-side)
- HTTP 401 Unauthorized (missing/invalid credentials)
- OKX `code` like `51000` invalid sign or `51603` order not found
}

\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
get_trade_order(
  inst_id = "ETH-USDT-SWAP",
  ord_id  = "1234567890",
  config  = cfg
)
}

}
\seealso{
[get_trade_orders_pending()], [get_trade_orders_history_7d()]

Other okxr-trade: 
\code{\link{get_trade_orders_history_7d}()},
\code{\link{get_trade_orders_pending}()}
}
\concept{okxr-trade}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_history_trades}
\alias{get_market_history_trades}
\title{Get historical public trades}
\usage{
get_market_history_trades(
  inst_id,
  type = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT"`.}

\item{type}{Character or `NULL`. Pagination type, using OKX values.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with historical public trades.
}
\description{
Retrieve public trade history for an instrument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_risk_state}
\alias{get_account_risk_state}
\title{Get account risk state}
\usage{
get_account_risk_state(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with the account risk flag and JSON-encoded
  risk-unit arrays.
}
\description{
Retrieve portfolio-margin account risk flags and affected risk units.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_interest_accrued}
\alias{get_account_interest_accrued}
\title{Get account interest accrued history}
\usage{
get_account_interest_accrued(
  type = NULL,
  ccy = NULL,
  inst_id = NULL,
  mgn_mode = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{type}{Character or `NULL`. Loan type. Currently `"2"` for market loans.}

\item{ccy}{Character or `NULL`. Loan currency.}

\item{inst_id}{Character or `NULL`. Instrument ID.}

\item{mgn_mode}{Character or `NULL`. Margin mode.}

\item{after}{Character or `NULL`. Pagination cursor for earlier rows.}

\item{before}{Character or `NULL`. Pagination cursor for newer rows.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per accrued-interest event.
}
\description{
Retrieve accrued borrowing interest records for the past year.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_underlying}
\alias{get_public_underlying}
\title{Get underlying list}
\usage{
get_public_underlying(
  inst_type,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type: `"SWAP"`, `"FUTURES"`, or
`"OPTION"`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Unused except for interface consistency.}
}
\value{
A one-column `data.frame` with `uly`.
}
\description{
Retrieve available underlyings for derivatives instruments.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_one_click_repay_history_v2}
\alias{get_trade_one_click_repay_history_v2}
\title{Get one-click repay history v2}
\usage{
get_trade_one_click_repay_history_v2(
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of results to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with one-click repay history rows.
}
\description{
Retrieve new one-click repay history.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_leverage_info}
\alias{get_account_leverage_info}
\title{Get account leverage settings}
\usage{
get_account_leverage_info(inst_id, mgn_mode, config, tz = .okx_default_tz)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT"`.}

\item{mgn_mode}{Character. Margin mode. One of `"cross"` or `"isolated"`.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with columns `instId`, `mgnMode`, `posSide`, and `lever`.
}
\description{
Retrieve leverage configuration for a given instrument and margin mode.
}
\details{
Wraps `/api/v5/account/leverage-info`. Requires both `inst_id` and
`mgn_mode`. Returns current leverage values (numeric).
}
\note{
Since okxr 0.1.1
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
get_account_leverage_info(
  inst_id = "BTC-USDT",
  mgn_mode = "cross",
  config = cfg
)
}

}
\seealso{
[get_account_balance()], [get_account_positions()]

Other okxr-account: 
\code{\link{get_account_balance}()},
\code{\link{get_account_config}()},
\code{\link{get_account_positions_history}()},
\code{\link{get_account_positions}()}
}
\concept{okxr-account}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_instruments}
\alias{get_account_instruments}
\title{Get account-available instruments}
\usage{
get_account_instruments(
  inst_type,
  uly = NULL,
  inst_family = NULL,
  inst_id = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
`"SWAP"`, `"FUTURES"`, `"OPTION"`.}

\item{uly}{Character or `NULL`. Underlying, where applicable.}

\item{inst_family}{Character or `NULL`. Instrument family filter.}

\item{inst_id}{Character or `NULL`. Specific instrument ID filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with account-available instrument metadata, including
  identifiers, currencies, tick size, lot size, leverage, expiry/listing
  timestamps, and state where returned by OKX.
}
\description{
Retrieve available instruments for the current account and instrument type.
}
\details{
Wraps `/api/v5/account/instruments`. This endpoint is account-scoped and may
return a subset of instruments available to the authenticated account.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_history_candles}
\alias{get_market_history_candles}
\title{Get historical market candles}
\usage{
get_market_history_candles(
  inst_id,
  bar,
  before = NULL,
  limit = 100L,
  config = NULL,
  tz = .okx_default_tz,
  standardize_names = TRUE
)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT"`.}

\item{bar}{Character. Candlestick granularity, e.g. `"1m"`, `"5m"`, `"1H"`.}

\item{before}{Character or `NULL`. Timestamp string like
`"\%Y-\%m-\%d \%H:\%M:\%S"`. If `NULL`, fetch recent history.}

\item{limit}{Integer. Number of bars to retrieve. Default `100L`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}

\item{standardize_names}{Logical. If `TRUE` (default), renames columns to
`timestamp`, `open`, `high`, `low`, `close`, `volume`, `volQuote`.}
}
\value{
A `data.frame` of candlestick bars. If `standardize_names = TRUE`,
column names are normalized. Timestamps are `POSIXct` in `tz`.
}
\description{
Retrieve candlestick data before a specific datetime.
}
\details{
Wraps `/api/v5/market/history-candles`. If `before` is supplied, it is
converted to milliseconds since epoch (in `tz`) and sent as `after=...`
(per OKX semantics: *return data before this time*).
}
\note{
Since okxr 0.1.1
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
get_market_history_candles(
  "ETH-USDT-SWAP", bar = "1H",
  before = "2025-08-20 00:00:00", config = cfg
)
}

}
\seealso{
[get_market_candles()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_auto_repay}
\alias{post_account_set_auto_repay}
\title{Set Account Auto Repay}
\usage{
post_account_set_auto_repay(
  auto_repay,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{auto_repay}{Logical. Whether auto repay should be enabled.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied auto-repay setting.
}
\description{
Enable or disable spot-mode auto repay.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_my_leaders}
\alias{get_copy_trade_my_leaders}
\title{Get my lead traders}
\usage{
get_copy_trade_my_leaders(
  inst_type = NULL,
  config,
  tz = .okx_default_tz,
  instType = inst_type
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Filter by instrument type
(e.g., `"SWAP"`, `"MARGIN"`, `"SPOT"`). If `NULL`, returns all.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}

\item{instType}{Deprecated alias for `inst_type`.}
}
\value{
A `data.frame` with fields such as `nickName` and `uniqueCode`.
}
\description{
Retrieve the list of lead traders you are currently copying.
}
\details{
Wraps `/api/v5/copytrading/current-lead-traders`. Returns one row per
lead trader followed by your account.
}
\note{
Since okxr 0.1.2
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
get_copy_trade_my_leaders(inst_type = "SWAP", config = cfg)
}

}
\seealso{
[get_copy_trade_settings()], [get_copy_trade_current_subpos()]

Other okxr-copytrading: 
\code{\link{get_copy_trade_current_subpos}()},
\code{\link{get_copy_trade_historical_subpos}()},
\code{\link{get_copy_trade_settings}()}
}
\concept{okxr-copytrading}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_auth.R
\name{.execute_post_action}
\alias{.execute_post_action}
\title{Execute a POST request to OKX}
\usage{
.execute_post_action(api_path, body_list, config)
}
\arguments{
\item{api_path}{API path (e.g., "/api/v5/trade/order").}

\item{body_list}{List to be converted to JSON for the request body.}

\item{config}{List with API credentials.}
}
\value{
An `httr` response object, or `NULL` if the request fails.
}
\description{
Perform a signed POST call to the OKX API.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_one_click_repay_currency_list_v2}
\alias{get_trade_one_click_repay_currency_list_v2}
\title{Get one-click repay currency list v2}
\usage{
get_trade_one_click_repay_currency_list_v2(
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with debt and repay currency metadata.
}
\description{
Retrieve repayable currencies for the new one-click repay endpoint.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_interest_rate}
\alias{get_account_interest_rate}
\title{Get account borrowing interest rates}
\usage{
get_account_interest_rate(
  ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter, e.g. `"BTC"`.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `ccy` and `interestRate`.
}
\description{
Retrieve the current leveraged currency borrowing market interest rate for
one currency or for all eligible currencies.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_get_generator.R
\docType{data}
\name{.gets}
\alias{.gets}
\title{GET request function list}
\format{
An object of class \code{list} of length 35.
}
\usage{
.gets
}
\description{
A list of endpoint functions automatically generated from \code{.api_GET_specs}.
Each function supports \code{tz}, \code{config}, and additional endpoint-specific arguments.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_discount_rate_interest_free_quota}
\alias{get_public_discount_rate_interest_free_quota}
\title{Get collateral discount rate and interest-free quota}
\usage{
get_public_discount_rate_interest_free_quota(
  ccy = NULL,
  discount_lv = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter, e.g. `"BTC"`.}

\item{discount_lv}{Character or `NULL`. Discount level filter.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with top-level discount and interest-free quota
  fields; nested tier details are JSON-encoded in `details`.
}
\description{
Retrieve public collateral discount-rate tiers and interest-free quota
information for supported currencies.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_block_trades}
\alias{get_public_block_trades}
\title{Get public block trades}
\usage{
get_public_block_trades(
  inst_id,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with block trade fields such as price, size, trade
  side, volatility, and timestamps.
}
\description{
Retrieve recent single-leg public block trades for an instrument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_greeks}
\alias{get_account_greeks}
\title{Get account Greeks}
\usage{
get_account_greeks(ccy = NULL, config, tz = .okx_default_tz)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per currency and Greek metrics in both
  Black-Scholes and coin terms.
}
\description{
Retrieve currency-level Greeks across the account.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_order_precheck}
\alias{post_trade_order_precheck}
\title{Precheck a Trade Order}
\usage{
post_trade_order_precheck(
  inst_id,
  td_mode,
  side,
  ord_type,
  sz,
  ccy = NULL,
  cl_ord_id = NULL,
  tag = NULL,
  pos_side = NULL,
  px = NULL,
  reduce_only = NULL,
  tgt_ccy = NULL,
  attach_algo_ords = NULL,
  speed_bump = NULL,
  outcome = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Instrument ID.}

\item{td_mode}{Trade mode.}

\item{side}{Order side.}

\item{ord_type}{Order type.}

\item{sz}{Order size.}

\item{ccy}{Optional margin currency.}

\item{cl_ord_id}{Optional client order ID.}

\item{tag}{Optional order tag.}

\item{pos_side}{Optional position side.}

\item{px}{Optional order price.}

\item{reduce_only}{Optional logical reduce-only flag.}

\item{tgt_ccy}{Optional target currency mode.}

\item{attach_algo_ords}{Optional attached TP/SL list.}

\item{speed_bump}{Optional event-contract speed bump.}

\item{outcome}{Optional event-contract outcome.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with projected account metrics after the precheck.
}
\description{
Submit an order precheck request without placing the order.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_auth.R
\name{.execute_get_action}
\alias{.execute_get_action}
\title{Execute a GET request to OKX}
\usage{
.execute_get_action(api_path, query_string, config = NULL, auth = TRUE)
}
\arguments{
\item{api_path}{API path (e.g., "/api/v5/account/balance").}

\item{query_string}{Query string starting with "?" or empty.}

\item{config}{List with API credentials.}

\item{auth}{Logical. Whether to sign the request with OKX credentials.}
}
\value{
An `httr` response object, or `NULL` if the request fails.
}
\description{
Perform a signed GET call to the OKX API.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_cancel_batch_orders}
\alias{post_trade_cancel_batch_orders}
\title{Cancel Multiple Trade Orders}
\usage{
post_trade_cancel_batch_orders(
  orders,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{orders}{List of cancellation specification lists containing `inst_id`
plus either `ord_id` or `cl_ord_id`.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with one row per cancellation result.
}
\description{
Submit a batch cancellation request for incomplete orders.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_unrealized_profit_sharing_details}
\alias{get_copy_trade_unrealized_profit_sharing_details}
\title{Get unrealized profit sharing details}
\usage{
get_copy_trade_unrealized_profit_sharing_details(
  inst_type = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with unrealized profit sharing rows.
}
\description{
Retrieve unrealized profit sharing detail rows.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_post_generator.R
\name{.make_post_function}
\alias{.make_post_function}
\title{Create a POST request function from an API spec}
\usage{
.make_post_function(api)
}
\arguments{
\item{api}{A list containing the POST API spec. Must include:
\code{okx_path}, \code{schema}, and optional \code{mode}.}
}
\value{
A function with signature \code{(body_list, tz, config)} that:
\itemize{
  \item Converts the body list to JSON,
  \item Signs and sends the POST request,
  \item Parses and returns the response using the schema-aware parser.
}
}
\description{
Converts a single entry from \code{.api_POST_specs} into a function that sends
a POST request to the OKX API and returns the parsed response using the provided schema.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_precheck_set_delta_neutral}
\alias{get_account_precheck_set_delta_neutral}
\title{Precheck delta-neutral strategy switch}
\usage{
get_account_precheck_set_delta_neutral(
  stgy_type,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{stgy_type}{Character or numeric. Strategy type. `"0"` for general or
`"1"` for delta neutral.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with JSON-encoded unmatched information.
}
\description{
Retrieve unmatched information that blocks switching into the requested
strategy type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_move_positions_history}
\alias{get_account_move_positions_history}
\title{Get account move positions history}
\usage{
get_account_move_positions_history(
  block_td_id = NULL,
  client_id = NULL,
  begin_ts = NULL,
  end_ts = NULL,
  limit = NULL,
  state = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{block_td_id}{Character or `NULL`. OKX block trade identifier.}

\item{client_id}{Character or `NULL`. Client-supplied identifier.}

\item{begin_ts}{Character or `NULL`. Inclusive start timestamp in
milliseconds.}

\item{end_ts}{Character or `NULL`. Inclusive end timestamp in milliseconds.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{state}{Character or `NULL`. Transfer state filter, `"filled"` or
`"pending"`.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with top-level move-position metadata; nested `legs`
  are JSON-encoded.
}
\description{
Retrieve move-position requests from the last three days.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_books}
\alias{get_market_books}
\title{Get order book}
\usage{
get_market_books(inst_id, sz = NULL, config = NULL, tz = .okx_default_tz)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT"`.}

\item{sz}{Integer or `NULL`. Order book depth. If `NULL`, OKX uses its
endpoint default.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with JSON-encoded `asks` and `bids` columns plus `ts`.
}
\description{
Retrieve the current order book for an instrument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_amend_order}
\alias{post_trade_amend_order}
\title{Amend a Trade Order}
\usage{
post_trade_amend_order(
  inst_id,
  ord_id = NULL,
  cl_ord_id = NULL,
  req_id = NULL,
  new_sz = NULL,
  new_px = NULL,
  cxl_on_fail = NULL,
  new_px_usd = NULL,
  new_px_vol = NULL,
  px_amend_type = NULL,
  attach_algo_ords = NULL,
  speed_bump = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Instrument ID.}

\item{ord_id}{Order ID. Optional if `cl_ord_id` is supplied.}

\item{cl_ord_id}{Client order ID. Optional if `ord_id` is supplied.}

\item{req_id}{Optional client amendment request ID.}

\item{new_sz}{Optional new total order size.}

\item{new_px}{Optional new price.}

\item{cxl_on_fail}{Optional logical. Whether to cancel the order if the
amendment fails.}

\item{new_px_usd}{Optional new option order USD price.}

\item{new_px_vol}{Optional new option order implied volatility price.}

\item{px_amend_type}{Optional price amendment mode.}

\item{attach_algo_ords}{Optional attached TP/SL amendment list.}

\item{speed_bump}{Optional event-contract speed bump.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` describing the amendment request result.
}
\description{
Submit an amendment request for an incomplete order.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_instruments}
\alias{get_copy_trade_instruments}
\title{Get copy trading instruments}
\usage{
get_copy_trade_instruments(
  inst_type = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per instrument and an `enabled` flag.
}
\description{
Retrieve instruments currently available for copy trading.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_fills}
\alias{get_trade_fills}
\title{Get trade fills}
\usage{
get_trade_fills(
  inst_type = NULL,
  inst_family = NULL,
  inst_id = NULL,
  ord_id = NULL,
  sub_type = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{inst_family}{Character or `NULL`. Instrument family filter.}

\item{inst_id}{Character or `NULL`. Instrument ID filter.}

\item{ord_id}{Character or `NULL`. Order ID filter.}

\item{sub_type}{Character or `NULL`. Transaction subtype filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with fill rows.
}
\description{
Retrieve recently filled transaction details from the last 3 days.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_settings}
\alias{get_copy_trade_settings}
\title{Get copy trading settings}
\usage{
get_copy_trade_settings(
  unique_code,
  inst_type = NULL,
  config,
  tz = .okx_default_tz,
  uniqueCode = unique_code,
  instType = inst_type
)
}
\arguments{
\item{unique_code}{Character. Lead trader unique code.}

\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}

\item{uniqueCode}{Deprecated alias for `unique_code`.}

\item{instType}{Deprecated alias for `inst_type`.}
}
\value{
A `data.frame` with fields like `copyMode` and `copyState`.
}
\description{
Retrieve your account's copy trading configuration.
}
\details{
Wraps `/api/v5/copytrading/copy-settings`. Returns one row with the
current copy mode and copy state for the given `unique_code`.
}
\note{
Since okxr 0.1.2
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
get_copy_trade_settings(unique_code = "1129E65755274C36", config = cfg)
}

}
\seealso{
[get_copy_trade_my_leaders()], [get_copy_trade_current_subpos()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_post_generator.R
\docType{data}
\name{.posts}
\alias{.posts}
\title{POST request function list}
\format{
An object of class \code{list} of length 4.
}
\usage{
.posts
}
\description{
A list of POST endpoint functions generated from \code{.api_POST_specs}.
Each function has the form \code{(body_list, tz, config)} and returns a parsed result.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_candles}
\alias{get_market_candles}
\title{Get recent market candles}
\usage{
get_market_candles(
  inst_id,
  bar,
  limit = 100L,
  config = NULL,
  tz = .okx_default_tz,
  standardize_names = TRUE
)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT"`, `"ETH-USDT-SWAP"`.}

\item{bar}{Character. Candlestick granularity, e.g. `"1m"`, `"5m"`, `"1H"`, `"1D"`.}

\item{limit}{Integer. Number of bars to retrieve. Default `100L`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}

\item{standardize_names}{Logical. If `TRUE` (default), renames columns to
`timestamp`, `open`, `high`, `low`, `close`, `volume`, `volQuote`.}
}
\value{
A `data.frame` with columns including `timestamp`, `open`, `high`, `low`,
`close`, `volume`, and `volQuote`. Timestamps are `POSIXct` in `tz`.
}
\description{
Retrieve the latest candlestick data for a given instrument and bar size.
}
\details{
Wraps `/api/v5/market/candles`. Returns up to `limit` bars, sorted by
timestamp. Candlestick fields can be standardized to common OHLCV names via
`standardize_names = TRUE`.
}
\note{
Since okxr 0.1.1
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
get_market_candles("BTC-USDT", bar = "5m", limit = 50, config = cfg)
}

}
\seealso{
[get_market_history_candles()], [get_public_mark_price()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_batch_orders}
\alias{post_trade_batch_orders}
\title{Place Multiple Trade Orders}
\usage{
post_trade_batch_orders(orders, config, tz = .okx_default_tz)
}
\arguments{
\item{orders}{List of order specs.
See \code{post_trade_order()}.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with one row per submitted order result.
}
\description{
Submit multiple trade orders in one request.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_constants.R
\name{set_okxr_options}
\alias{set_okxr_options}
\title{Set or get okxr options}
\usage{
set_okxr_options(raw_data = NULL, timeout = NULL)
}
\arguments{
\item{raw_data}{Logical. If `TRUE`, return raw API `data`.
If `NULL`, the current value is left unchanged.}

\item{timeout}{Numeric. HTTP request timeout in seconds. If `NULL`, the
current value is left unchanged.}
}
\value{
An invisible named list with the current package options:
`raw_data` (logical) and `timeout` (numeric seconds).
This return value can be used to inspect the effective option state.
}
\description{
Convenience wrapper to set global options for okxr, such as whether to
return raw data instead of parsed data.
}
\examples{
old <- getOption("okxr.raw_data")
old_timeout <- getOption("okxr.timeout")
set_okxr_options(raw_data = TRUE)
set_okxr_options(timeout = 5)
options(okxr.raw_data = old, okxr.timeout = old_timeout)

set_okxr_options()  # check current values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_public_lead_traders}
\alias{get_copy_trade_public_lead_traders}
\title{Get public lead trader ranks}
\usage{
get_copy_trade_public_lead_traders(
  inst_type = NULL,
  sort_type = NULL,
  state = NULL,
  min_lead_days = NULL,
  min_assets = NULL,
  max_assets = NULL,
  min_aum = NULL,
  max_aum = NULL,
  data_ver = NULL,
  page = NULL,
  limit = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{sort_type}{Character or `NULL`. Rank sort selector.}

\item{state}{Character or `NULL`. Lead trader state filter.}

\item{min_lead_days}{Character or `NULL`. Minimum lead-days selector.}

\item{min_assets}{Character or `NULL`. Minimum assets filter.}

\item{max_assets}{Character or `NULL`. Maximum assets filter.}

\item{min_aum}{Character or `NULL`. Minimum assets-under-management filter.}

\item{max_aum}{Character or `NULL`. Maximum assets-under-management filter.}

\item{data_ver}{Character or `NULL`. Data version selector used for pagination.}

\item{page}{Character or `NULL`. Page number.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with top-level ranking metadata and a JSON-string
  `ranks` column for nested leader rows.
}
\description{
Retrieve ranked public lead trader summaries.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_tickers}
\alias{get_market_tickers}
\title{Get market tickers}
\usage{
get_market_tickers(
  inst_type,
  uly = NULL,
  inst_family = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type, e.g. `"SPOT"`, `"SWAP"`,
`"FUTURES"`, or `"OPTION"`.}

\item{uly}{Character or `NULL`. Underlying. Optional filter for derivatives.}

\item{inst_family}{Character or `NULL`. Instrument family. Optional filter
for derivatives and options.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per ticker.
}
\description{
Retrieve ticker snapshots for all instruments under an instrument type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_easy_convert_currency_list}
\alias{get_trade_easy_convert_currency_list}
\title{Get easy convert currency list}
\usage{
get_trade_easy_convert_currency_list(
  source = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{source}{Character or `NULL`. Source account type filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with source and target currency metadata.
}
\description{
Retrieve currencies available for easy convert.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_request_helpers.R
\docType{data}
\name{.okx_default_tz}
\alias{.okx_default_tz}
\title{Internal OKX request helpers}
\format{
An object of class \code{character} of length 1.
}
\usage{
.okx_default_tz
}
\description{
Shared helpers for request validation, query construction, and package-wide
defaults used by user-facing wrappers.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_public_subpositions_history}
\alias{get_copy_trade_public_subpositions_history}
\title{Get public historical copy trading subpositions}
\usage{
get_copy_trade_public_subpositions_history(
  unique_code,
  inst_type = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{unique_code}{Character. Lead trader unique code.}

\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per historical public subposition.
}
\description{
Retrieve public historical subpositions for a lead trader.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_move_positions}
\alias{post_account_move_positions}
\title{Move Positions Between Accounts}
\usage{
post_account_move_positions(
  from_acct,
  to_acct,
  legs,
  client_id,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{from_acct}{Source account name.}

\item{to_acct}{Destination account name.}

\item{legs}{List of move-position leg objects in the documented OKX shape.}

\item{client_id}{Client-supplied request ID.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` describing the move-position request result.
}
\description{
Move positions between accounts under the same master account.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/okxr-package.R
\docType{package}
\name{okxr-package}
\alias{okxr}
\alias{okxr-package}
\title{okxr: R Interface to the OKX REST API}
\description{
`okxr` provides lightweight wrappers for selected OKX REST API endpoints,
including market data, account information, asset metadata, order-book trade
queries, trading actions, and copy-trading endpoints.
}
\details{
Public market and public reference endpoints can be called without
credentials. Private account, asset, trade, and copy-trading endpoints require
an OKX API credential list with `api_key`, `secret_key`, and `passphrase`
entries.

If `config$demo` is `TRUE`, signed requests include OKX's simulated trading
header. Request timeout defaults to 10 seconds and can be set globally with
`set_okxr_options(timeout = 15)` or per request with `config$timeout`.

By default, wrappers return parsed `data.table` objects. Use
`set_okxr_options(raw_data = TRUE)` to return raw API `data` payloads instead.

Network failures, request timeouts, OKX error responses, or empty API `data`
payloads may return `NULL` with a warning. Live API examples are intentionally
non-running because they require credentials, network access, and may have
account-specific side effects.
}
\seealso{
[set_okxr_options()], [get_market_candles()], [post_trade_order()]
}
\author{
\strong{Maintainer}: Oliver Zhou \email{oliver.yxzhou@gmail.com}

Authors:
\itemize{
  \item Lily Li \email{lilyli.qc.hk@gmail.com}
}

}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_deposit_withdraw_status}
\alias{get_asset_deposit_withdraw_status}
\title{Get deposit or withdrawal status}
\usage{
get_asset_deposit_withdraw_status(
  wd_id = NULL,
  tx_id = NULL,
  ccy = NULL,
  to = NULL,
  chain = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{wd_id}{Character or `NULL`. Withdrawal ID.}

\item{tx_id}{Character or `NULL`. Deposit transaction hash.}

\item{ccy}{Character or `NULL`. Currency filter used with \code{tx_id}.}

\item{to}{Character or `NULL`. Destination address used with \code{tx_id}.}

\item{chain}{Character or `NULL`. Chain identifier used with \code{tx_id}.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with detailed deposit/withdraw status rows.
}
\description{
Retrieve detailed status for a deposit or withdrawal.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_time}
\alias{get_public_time}
\title{Get OKX system time}
\usage{
get_public_time(config = NULL, tz = .okx_default_tz)
}
\arguments{
\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with system time.
}
\description{
Retrieve OKX system time.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_delivery_exercise_history}
\alias{get_public_delivery_exercise_history}
\title{Get delivery or exercise history}
\usage{
get_public_delivery_exercise_history(
  inst_type,
  inst_family,
  after = NULL,
  before = NULL,
  limit = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. `"FUTURES"` or `"OPTION"`.}

\item{inst_family}{Character. Instrument family.}

\item{after}{Character or `NULL`. Pagination cursor for older rows.}

\item{before}{Character or `NULL`. Pagination cursor for newer rows.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `ts` and JSON-encoded `details`.
}
\description{
Retrieve futures delivery records or option exercise records.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_instruments}
\alias{get_public_instruments}
\title{Get instrument metadata}
\usage{
get_public_instruments(
  inst_id = NULL,
  inst_type = "SWAP",
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character or `NULL`. Specific instrument ID to query. Use
`NULL` to fetch all instruments of `inst_type`.}

\item{inst_type}{Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
`"SWAP"` (default), `"FUTURES"`, `"OPTION"`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with instrument metadata (e.g., `instType`, `instId`, `uly`,
`baseCcy`, `quoteCcy`, `settleCcy`, `ctVal`, `ctMult`, `tickSz`, `lotSz`,
`minSz`, `expTime`, `lever`, `state`, …).
}
\description{
Retrieve metadata for instruments of a given type.
}
\details{
Wraps `/api/v5/public/instruments`. Returns one row per instrument,
including contract specifications, tick size, lot size, expiry, and state.
}
\note{
Since okxr 0.1.2
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
# Get metadata for all SWAP instruments
df <- get_public_instruments(inst_type = "SWAP", config = cfg)

# Get metadata for one instrument
get_public_instruments("ETH-USDT-SWAP", inst_type = "SWAP", config = cfg)
}

}
\seealso{
[get_public_mark_price()]

Other okxr-market: 
\code{\link{get_market_candles}()},
\code{\link{get_market_history_candles}()},
\code{\link{get_public_mark_price}()}
}
\concept{okxr-market}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_order}
\alias{post_trade_order}
\title{Place a Trade Order}
\usage{
post_trade_order(
  inst_id,
  td_mode,
  side,
  ord_type,
  sz,
  pos_side = NULL,
  px = NULL,
  reduce_only = NULL,
  tgt_ccy = NULL,
  cl_ord_id = NULL,
  tag = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Instrument ID (e.g., \code{"BTC-USDT"}).}

\item{td_mode}{Trade mode: \code{"cross"} or \code{"isolated"}.}

\item{side}{Order side: \code{"buy"} or \code{"sell"}.}

\item{ord_type}{Order type: \code{"limit"}, \code{"market"}, etc.}

\item{sz}{Size of the order (quantity to buy/sell).}

\item{pos_side}{Optional. Position side: \code{"long"} or \code{"short"}.}

\item{px}{Optional. Price (required for limit orders).}

\item{reduce_only}{Optional. Logical flag to indicate a reduce-only order.}

\item{tgt_ccy}{Optional. Quote currency (e.g., \code{"base"}, \code{"quote"}).}

\item{cl_ord_id}{Optional. Custom client order ID (auto-generated if NULL).}

\item{tag}{Optional. Tag used for identifying the strategy or bot.}

\item{config}{A list with API credentials: \code{api_key}, \code{secret_key}, \code{passphrase}.}

\item{tz}{Timezone for parsing any timestamps (default: \code{"Asia/Hong_Kong"}).}
}
\value{
A \code{data.frame} containing fields like order ID, client order ID, and timestamp.
}
\description{
Submits a trade order to the OKX exchange.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_public_weekly_pnl}
\alias{get_copy_trade_public_weekly_pnl}
\title{Get public copy trading weekly pnl}
\usage{
get_copy_trade_public_weekly_pnl(
  unique_code,
  inst_type = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{unique_code}{Character. Lead trader unique code.}

\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with weekly pnl windows and ratios.
}
\description{
Retrieve public weekly pnl series for a lead trader.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_subaccount_balances}
\alias{get_account_subaccount_balances}
\title{Get sub-account trading balances}
\usage{
get_account_subaccount_balances(
  sub_acct,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{sub_acct}{Character. Sub-account name.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with account-level balance fields; nested per-currency
  `details` are JSON-encoded.
}
\description{
Retrieve account-level trading balances for a sub-account from the master
account.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_auth.R
\name{.get_headers}
\alias{.get_headers}
\title{Create OKX API request headers}
\usage{
.get_headers(config, httr_method, httr_path, body_json = "")
}
\arguments{
\item{config}{List with `api_key`, `secret_key`, and `passphrase`.}

\item{httr_method}{HTTP method as a string (e.g., "GET" or "POST").}

\item{httr_path}{Path portion of the API endpoint.}

\item{body_json}{Optional JSON string of the request body.}
}
\value{
A `httr::add_headers` object containing the signed headers.
}
\description{
Generate the required signed headers for an OKX REST request.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_cancel_algos}
\alias{post_trade_cancel_algos}
\title{Cancel Multiple Algo Orders}
\usage{
post_trade_cancel_algos(orders, config, tz = .okx_default_tz)
}
\arguments{
\item{orders}{List of cancellation specification lists containing `inst_id`
plus either `algo_id` or `algo_cl_ord_id`.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with one row per algo cancellation result.
}
\description{
Cancel up to 10 unfilled algo orders in one request.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_amend_algos}
\alias{post_trade_amend_algos}
\title{Amend an Algo Order}
\usage{
post_trade_amend_algos(
  inst_id,
  algo_id = NULL,
  algo_cl_ord_id = NULL,
  cxl_on_fail = NULL,
  req_id = NULL,
  new_sz = NULL,
  new_tp_trigger_px = NULL,
  new_tp_ord_px = NULL,
  new_sl_trigger_px = NULL,
  new_sl_ord_px = NULL,
  new_tp_trigger_px_type = NULL,
  new_sl_trigger_px_type = NULL,
  new_trigger_px = NULL,
  new_ord_px = NULL,
  new_trigger_px_type = NULL,
  attach_algo_ords = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Instrument ID.}

\item{algo_id}{Algo order ID. Optional if `algo_cl_ord_id` is supplied.}

\item{algo_cl_ord_id}{Client-supplied algo ID. Optional if `algo_id` is supplied.}

\item{cxl_on_fail}{Optional logical. Whether to cancel the order if the
amendment fails.}

\item{req_id}{Optional client amendment request ID.}

\item{new_sz}{Optional new quantity after amendment.}

\item{new_tp_trigger_px}{Optional new take-profit trigger price.}

\item{new_tp_ord_px}{Optional new take-profit order price.}

\item{new_sl_trigger_px}{Optional new stop-loss trigger price.}

\item{new_sl_ord_px}{Optional new stop-loss order price.}

\item{new_tp_trigger_px_type}{Optional new take-profit trigger price type.}

\item{new_sl_trigger_px_type}{Optional new stop-loss trigger price type.}

\item{new_trigger_px}{Optional new trigger price for trigger orders.}

\item{new_ord_px}{Optional new order price for trigger orders.}

\item{new_trigger_px_type}{Optional new trigger price type for trigger orders.}

\item{attach_algo_ords}{Optional attached TP/SL amendment list.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` describing the algo amendment result.
}
\description{
Amend a supported unfilled algo order.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_mark_price}
\alias{get_public_mark_price}
\title{Get current mark price}
\usage{
get_public_mark_price(
  inst_id,
  inst_type = "SWAP",
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT"`, `"ETH-USDT-SWAP"`.}

\item{inst_type}{Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
`"SWAP"` (default), `"FUTURES"`, `"OPTION"`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with columns `timestamp`, `instId`, `markPx`.
}
\description{
Retrieve the current mark price for a given instrument.
}
\details{
Wraps `/api/v5/public/mark-price`. Useful for margin calculations and PnL
estimation. Returns a single row with the latest mark price and timestamp.
}
\note{
Since okxr 0.1.1
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
get_public_mark_price("BTC-USDT", inst_type = "SWAP", config = cfg)
}

}
\seealso{
[get_public_instruments()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_historical_subpos}
\alias{get_copy_trade_historical_subpos}
\title{Get historical copy trading subpositions}
\usage{
get_copy_trade_historical_subpos(
  inst_type = NULL,
  inst_id = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{inst_id}{Character or `NULL`. Instrument ID filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with fields like `instId` and `uniqueCode`.
}
\description{
Retrieve your historical copy trading subpositions.
}
\details{
Wraps `/api/v5/copytrading/subpositions-history`. Returns one row per
closed or historical subposition.
}
\note{
Since okxr 0.1.2
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
hist <- get_copy_trade_historical_subpos(config = cfg)
head(hist)
}

}
\seealso{
[get_copy_trade_current_subpos()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_asset.R
\name{post_asset_withdrawal}
\alias{post_asset_withdrawal}
\title{Submit an Asset Withdrawal}
\usage{
post_asset_withdrawal(
  ccy,
  amt,
  dest,
  to_addr,
  chain = NULL,
  to_addr_type = NULL,
  area_code = NULL,
  rcvr_info = NULL,
  client_id = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{ccy}{Currency to withdraw.}

\item{amt}{Withdrawal amount.}

\item{dest}{Destination type code from the OKX API.}

\item{to_addr}{Destination wallet address.}

\item{chain}{Optional chain identifier, such as `"USDT-ERC20"`.}

\item{to_addr_type}{Optional destination address type code.}

\item{area_code}{Optional phone area code when required by OKX.}

\item{rcvr_info}{Optional named list in the documented `rcvrInfo` shape.}

\item{client_id}{Optional client-supplied withdrawal request ID.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{API credential list.}
}
\value{
A `data.frame` describing the submitted withdrawal request.
}
\description{
Submit a withdrawal request from the OKX funding account.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_interest_rate_loan_quota}
\alias{get_public_interest_rate_loan_quota}
\title{Get interest rate and loan quota}
\usage{
get_public_interest_rate_loan_quota(
  ccy = NULL,
  vip_level = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter.}

\item{vip_level}{Character or `NULL`. VIP level filter when supported by OKX.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` where nested basic, VIP, regular, and custom quota
  tables are JSON-encoded string columns.
}
\description{
Retrieve public borrowing-rate and loan-quota tables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_positions_history}
\alias{get_account_positions_history}
\title{Get account position history}
\usage{
get_account_positions_history(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with columns such as `instId`, `posId`, `posSide`, `pos`,
`lever`, `realizedPnl`, `fee`, plus timestamp fields (`cTime`, `uTime`).
}
\description{
Retrieve historical records of closed or adjusted positions.
}
\details{
Wraps `/api/v5/account/positions-history`. Includes closed positions and
their realized PnL. Returns one row per historical record.
}
\note{
Since okxr 0.1.1
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
hist <- get_account_positions_history(config = cfg)
tail(hist)
}

}
\seealso{
[get_account_positions()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_account_level_switch_preset}
\alias{post_account_account_level_switch_preset}
\title{Preset Account Level Switch}
\usage{
post_account_account_level_switch_preset(
  acct_lv,
  lever = NULL,
  risk_offset_type = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{acct_lv}{Target account level.}

\item{lever}{Optional leverage preset.}

\item{risk_offset_type}{Optional deprecated risk offset type field.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` describing the stored preset values.
}
\description{
Preset required values before switching account mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_position_tiers}
\alias{get_public_position_tiers}
\title{Get public position tiers}
\usage{
get_public_position_tiers(
  inst_type,
  td_mode,
  inst_family = NULL,
  inst_id = NULL,
  ccy = NULL,
  tier = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type.}

\item{td_mode}{Character. Trade mode.}

\item{inst_family}{Character or `NULL`. Instrument family.}

\item{inst_id}{Character or `NULL`. Instrument ID(s).}

\item{ccy}{Character or `NULL`. Margin currency.}

\item{tier}{Character or `NULL`. Tier filter.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with public position tier information.
}
\description{
Retrieve public tier, margin, and maximum leverage information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_position_mode}
\alias{post_account_set_position_mode}
\title{Set Account Position Mode}
\usage{
post_account_set_position_mode(
  pos_mode,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{pos_mode}{Position mode. Use `long_short_mode` or `net_mode`.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied position mode.
}
\description{
Set the account position mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_option_instrument_family_trades}
\alias{get_market_option_instrument_family_trades}
\title{Get option trades by instrument family}
\usage{
get_market_option_instrument_family_trades(
  inst_family,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_family}{Character. Instrument family, e.g. `"BTC-USD"`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with recent option trades under the requested
  instrument family, including `instId`, `tradeId`, `px`, `sz`, `side`, and
  `ts`.
}
\description{
Retrieve recent option trades for all instruments under the same instrument
family.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_position_margin_balance}
\alias{post_account_position_margin_balance}
\title{Adjust Position Margin Balance}
\usage{
post_account_position_margin_balance(
  inst_id,
  pos_side,
  type,
  amt,
  ccy = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{inst_id}{Instrument ID.}

\item{pos_side}{Position side.}

\item{type}{Margin adjustment type, typically `"add"` or `"reduce"`.}

\item{amt}{Amount to add or reduce.}

\item{ccy}{Optional currency for isolated margin orders.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` describing the applied margin adjustment.
}
\description{
Increase or reduce margin for an existing position.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_fee_type}
\alias{post_account_set_fee_type}
\title{Set Account Fee Type}
\usage{
post_account_set_fee_type(fee_type, tz = .okx_default_tz, config)
}
\arguments{
\item{fee_type}{Fee type string, typically `"0"` or `"1"`.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied fee type.
}
\description{
Configure the fee charging mode for spot trading.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_mass_cancel}
\alias{post_trade_mass_cancel}
\title{Mass Cancel MMP Orders}
\usage{
post_trade_mass_cancel(
  inst_type,
  inst_family,
  lock_interval = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Instrument type. Currently `OPTION`.}

\item{inst_family}{Instrument family.}

\item{lock_interval}{Optional lock interval in milliseconds.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with the request result.
}
\description{
Cancel all MMP pending orders for an options instrument family.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_platform_24_volume}
\alias{get_market_platform_24_volume}
\title{Get platform 24-hour volume}
\usage{
get_market_platform_24_volume(
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with `volUsd`, `volCny`, and `ts`.
}
\description{
Retrieve total platform order-book trading volume over the last 24 hours.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_option_trades}
\alias{get_public_option_trades}
\title{Get public option trades}
\usage{
get_public_option_trades(
  inst_id = NULL,
  inst_family = NULL,
  opt_type = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character or `NULL`. Specific option instrument ID.}

\item{inst_family}{Character or `NULL`. Option instrument family, e.g.
`"BTC-USD"`. Either `inst_id` or `inst_family` should be supplied.}

\item{opt_type}{Character or `NULL`. Option type filter: `"C"` for call or
`"P"` for put.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with recent option trade rows, including option
  instrument identifiers, trade price and size, option side/type, forward,
  index and mark prices, implied volatility, and trade time.
}
\description{
Retrieve recent public option trades filtered by instrument ID or instrument
family.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_collateral_assets}
\alias{post_account_set_collateral_assets}
\title{Set Account Collateral Assets}
\usage{
post_account_set_collateral_assets(
  type,
  collateral_enabled,
  ccy_list = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{type}{Type of update, typically `"all"` or `"custom"`.}

\item{collateral_enabled}{Logical. Whether the selected assets should be
collateral-enabled.}

\item{ccy_list}{Optional character vector of currencies. Required when
`type = "custom"`.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied collateral asset setting.
}
\description{
Configure whether all or selected assets are treated as collateral.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_greeks}
\alias{post_account_set_greeks}
\title{Set Account Greeks Display Type}
\usage{
post_account_set_greeks(
  greeks_type,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{greeks_type}{Greeks display type, typically `"PA"` or `"BS"`.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied Greeks display type.
}
\description{
Configure whether Greeks are displayed in PA or BS mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_config}
\alias{get_account_config}
\title{Get account configuration}
\usage{
get_account_config(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with columns like `uid`, `mainUid`, `acctLv`, `posMode`,
`autoLoan`, etc.
}
\description{
Retrieve account-level configuration information.
}
\details{
Wraps `/api/v5/account/config`. Includes account ID, mode, and position
mode flags. Returns one row.
}
\note{
Since okxr 0.1.2
}
\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
cfg_info <- get_account_config(config = cfg)
cfg_info
}

}
\seealso{
[get_account_balance()], [get_account_leverage_info()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_position_risk}
\alias{get_account_position_risk}
\title{Get account and position risk snapshot}
\usage{
get_account_position_risk(
  inst_type = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter. One of
`"MARGIN"`, `"SWAP"`, `"FUTURES"`, or `"OPTION"`.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per OKX risk snapshot. Nested balance and
  position payloads are returned as JSON strings in `balData` and `posData`.
}
\description{
Retrieve account-level adjusted equity together with same-snapshot balance
and position risk payloads.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_spot_manual_borrow_repay}
\alias{post_account_spot_manual_borrow_repay}
\title{Submit Spot Manual Borrow or Repay}
\usage{
post_account_spot_manual_borrow_repay(
  ccy,
  side,
  amt,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{ccy}{Currency.}

\item{side}{Action side, typically `"borrow"` or `"repay"`.}

\item{amt}{Amount.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` describing the executed borrow/repay request.
}
\description{
Manually borrow or repay under spot mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_auto_loan}
\alias{post_account_set_auto_loan}
\title{Set Account Auto Loan}
\usage{
post_account_set_auto_loan(
  auto_loan = TRUE,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{auto_loan}{Logical. Whether auto loan should be enabled.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied auto-loan setting.
}
\description{
Enable or disable automatic borrowing.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_cancel_all_after}
\alias{post_trade_cancel_all_after}
\title{Set Cancel-All-After}
\usage{
post_trade_cancel_all_after(
  time_out,
  tag = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{time_out}{Character or numeric. Countdown in seconds. `0` disables it.}

\item{tag}{Optional cancel-all-after tag scope.}

\item{config}{A list with API credentials.}

\item{tz}{Timezone for parsing response timestamps.}
}
\value{
A `data.frame` with the configured trigger time and tag.
}
\description{
Set or disable the cancel-all-after countdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_block_tickers}
\alias{get_market_block_tickers}
\title{Get block tickers}
\usage{
get_market_block_tickers(
  inst_type,
  inst_family = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type.}

\item{inst_family}{Character or `NULL`. Instrument family filter.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per block ticker.
}
\description{
Retrieve the latest 24-hour block-trading volume for instruments under an
instrument type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_adjust_leverage_info}
\alias{get_account_adjust_leverage_info}
\title{Get account leverage adjustment estimate}
\usage{
get_account_adjust_leverage_info(
  inst_type,
  mgn_mode,
  lever,
  inst_id = NULL,
  ccy = NULL,
  pos_side = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type: `"MARGIN"`, `"SWAP"`, or `"FUTURES"`.}

\item{mgn_mode}{Character. Margin mode: `"cross"` or `"isolated"`.}

\item{lever}{Character or numeric. Target leverage.}

\item{inst_id}{Character or `NULL`. Instrument ID.}

\item{ccy}{Character or `NULL`. Margin currency.}

\item{pos_side}{Character or `NULL`. Position side.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with estimated leverage-adjustment metrics.
}
\description{
Estimate account effects under a target leverage.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_insurance_fund}
\alias{get_public_insurance_fund}
\title{Get security fund balance information}
\usage{
get_public_insurance_fund(
  inst_type,
  type = NULL,
  inst_family = NULL,
  ccy = NULL,
  before = NULL,
  after = NULL,
  limit = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type, such as `"MARGIN"`, `"SWAP"`,
`"FUTURES"`, or `"OPTION"`.}

\item{type}{Character or `NULL`. Fund update type filter.}

\item{inst_family}{Character or `NULL`. Instrument family filter for
derivatives.}

\item{ccy}{Character or `NULL`. Currency filter for margin data.}

\item{before}{Character or `NULL`. Pagination cursor for newer rows.}

\item{after}{Character or `NULL`. Pagination cursor for older rows.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with top-level fund totals; nested per-currency detail
  rows are JSON-encoded in `details`.
}
\description{
Retrieve public insurance-fund or security-fund balance information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_asset.R
\name{post_asset_transfer}
\alias{post_asset_transfer}
\title{Transfer Assets}
\usage{
post_asset_transfer(
  ccy,
  amt,
  from,
  to,
  type = "0",
  sub_acct = NULL,
  loan_trans = NULL,
  omit_pos_risk = NULL,
  client_id = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{ccy}{Currency to transfer.}

\item{amt}{Transfer amount.}

\item{from}{Source account code, such as `"6"` for funding or `"18"` for
trading.}

\item{to}{Destination account code.}

\item{type}{Optional transfer type code. Defaults to `"0"` for an internal
transfer within the same account.}

\item{sub_acct}{Optional sub-account name when the transfer type requires it.}

\item{loan_trans}{Optional logical. Whether the transfer should be treated
as a loan transfer.}

\item{omit_pos_risk}{Optional logical. Whether to omit position risk checks
where supported by OKX.}

\item{client_id}{Optional client-supplied transfer request ID.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{API credential list.}
}
\value{
A `data.frame` describing the submitted transfer request.
}
\description{
Transfer assets between funding, trading, and related accounts.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_orders_history_archive}
\alias{get_trade_orders_history_archive}
\title{Get archived trade orders history}
\usage{
get_trade_orders_history_archive(
  inst_type,
  inst_family = NULL,
  inst_id = NULL,
  ord_type = NULL,
  state = NULL,
  category = NULL,
  after = NULL,
  before = NULL,
  begin = NULL,
  end = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type.}

\item{inst_family}{Character or `NULL`. Instrument family filter.}

\item{inst_id}{Character or `NULL`. Instrument ID filter.}

\item{ord_type}{Character or `NULL`. Order type filter.}

\item{state}{Character or `NULL`. Order state filter.}

\item{category}{Character or `NULL`. Order category filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{begin}{Character or `NULL`. Begin timestamp in milliseconds.}

\item{end}{Character or `NULL`. End timestamp in milliseconds.}

\item{limit}{Integer or `NULL`. Number of results to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with archived order rows.
}
\description{
Retrieve completed orders from the last 3 months.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_close_position}
\alias{post_trade_close_position}
\title{Close a Position}
\usage{
post_trade_close_position(
  inst_id,
  mgn_mode,
  pos_side,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{inst_id}{Instrument ID (e.g., \code{"BTC-USDT"}).}

\item{mgn_mode}{Margin mode: \code{"cross"} or \code{"isolated"}.}

\item{pos_side}{Position side to close: \code{"long"} or \code{"short"}.}

\item{tz}{Timezone for parsing any timestamps (default: \code{"Asia/Hong_Kong"}).}

\item{config}{A list with API credentials: \code{api_key}, \code{secret_key}, \code{passphrase}.}
}
\value{
A \code{data.frame} with close position confirmation details.
}
\description{
Submits a request to close a position for a given instrument and position side.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_index_components}
\alias{get_market_index_components}
\title{Get index components}
\usage{
get_market_index_components(
  index,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{index}{Character. Index identifier, e.g. `"BTC-USD"`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with `index`, `last`, `ts`, and JSON-encoded
  `components`.
}
\description{
Retrieve component-exchange information for an index.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_opt_summary}
\alias{get_public_opt_summary}
\title{Get option summary}
\usage{
get_public_opt_summary(
  inst_family,
  exp_time = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_family}{Character. Option instrument family.}

\item{exp_time}{Character or `NULL`. Expiry date in `YYMMDD`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with option greeks, volatility, forward price, and
  timestamp fields.
}
\description{
Retrieve option market summary data for an instrument family.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_economic_calendar}
\alias{get_public_economic_calendar}
\title{Get economic calendar}
\usage{
get_public_economic_calendar(
  region = NULL,
  importance = NULL,
  before = NULL,
  after = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{region}{Character or `NULL`. Region filter.}

\item{importance}{Character or `NULL`. Importance level filter.}

\item{before}{Character or `NULL`. Pagination cursor for newer rows.}

\item{after}{Character or `NULL`. Pagination cursor for older rows.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with calendar event fields and timestamps.
}
\description{
Retrieve macro-economic calendar records. OKX requires authentication for
this endpoint.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_bills_history_archive}
\alias{get_account_bills_history_archive}
\title{Get archived account bill export links}
\usage{
get_account_bills_history_archive(
  year,
  quarter,
  type = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{year}{Character or numeric. Four-digit year.}

\item{quarter}{Character. Quarter code, one of `"Q1"` to `"Q4"`.}

\item{type}{Character or `NULL`. Optional comma-separated bill type filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with file-link status rows.
}
\description{
Retrieve the generated CSV export link for historical account bills since
2021.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_collateral_assets}
\alias{get_account_collateral_assets}
\title{Get account collateral assets}
\usage{
get_account_collateral_assets(
  ccy = NULL,
  collateral_enabled = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. One currency or a comma-separated list of up
to 20 currencies.}

\item{collateral_enabled}{Logical, character, or `NULL`. Filter by collateral
status.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with currency-level collateral flags.
}
\description{
Retrieve collateral-enabled status for one or more currencies.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_mmp_config}
\alias{post_account_mmp_config}
\title{Configure MMP}
\usage{
post_account_mmp_config(
  inst_family,
  time_interval,
  frozen_interval,
  qty_limit,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{inst_family}{Instrument family.}

\item{time_interval}{Time window in milliseconds.}

\item{frozen_interval}{Frozen interval in milliseconds.}

\item{qty_limit}{Quantity limit in number of contracts.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` describing the applied MMP configuration.
}
\description{
Set market maker protection thresholds for an options instrument family.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_account_rate_limit}
\alias{get_trade_account_rate_limit}
\title{Get trade account rate limit}
\usage{
get_trade_account_rate_limit(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A one-row `data.frame` with rate-limit metrics such as
  `accRateLimit`, `fillRatio`, `mainFillRatio`, `nextAccRateLimit`, and `ts`.
}
\description{
Retrieve account rate limit information related to new and amended order
requests.
}
\details{
Wraps `/api/v5/trade/account-rate-limit`. Returns one row containing the
current account rate limit and fill-ratio metrics used by OKX.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_exchange_list}
\alias{get_asset_exchange_list}
\title{Get exchange list}
\usage{
get_asset_exchange_list(tz = .okx_default_tz)
}
\arguments{
\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with exchange identifiers and names.
}
\description{
Retrieve the public exchange list used by asset withdrawal metadata.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_asset_valuation}
\alias{get_asset_asset_valuation}
\title{Get asset valuation}
\usage{
get_asset_asset_valuation(
  ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Valuation currency filter.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with valuation summary rows.
}
\description{
Retrieve total account valuation for funding assets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_deposit_address}
\alias{get_asset_deposit_address}
\title{Get deposit address}
\usage{
get_asset_deposit_address(ccy, config, tz = .okx_default_tz)
}
\arguments{
\item{ccy}{Character. Currency, e.g. \code{"BTC"} or \code{"USDT"}.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string (default: \code{"Asia/Hong_Kong"}).}
}
\value{
A \code{data.frame} with deposit address rows.
}
\description{
Retrieve deposit addresses for a currency.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_asset.R
\name{post_asset_convert_estimate_quote}
\alias{post_asset_convert_estimate_quote}
\title{Estimate an Asset Convert Quote}
\usage{
post_asset_convert_estimate_quote(
  base_ccy,
  quote_ccy,
  side,
  rfq_sz,
  rfq_sz_ccy,
  cl_q_req_id = NULL,
  tag = NULL,
  convert_mode = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{base_ccy}{Base currency.}

\item{quote_ccy}{Quote currency.}

\item{side}{Quote side, such as `"buy"` or `"sell"`.}

\item{rfq_sz}{RFQ size.}

\item{rfq_sz_ccy}{Currency in which `rfq_sz` is specified.}

\item{cl_q_req_id}{Optional client quote request ID.}

\item{tag}{Optional request tag.}

\item{convert_mode}{Optional OKX convert mode.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{API credential list.}
}
\value{
A `data.frame` describing the estimated conversion quote.
}
\description{
Request a quote for an asset conversion without executing the trade.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_order_algo}
\alias{get_trade_order_algo}
\title{Get a single algo order}
\usage{
get_trade_order_algo(
  algo_id = NULL,
  algo_cl_ord_id = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{algo_id}{Character or `NULL`. Algo order ID.}

\item{algo_cl_ord_id}{Character or `NULL`. Client algo order ID.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with algo order details.
}
\description{
Retrieve a specific algo order by algo ID or client algo order ID.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_max_loan}
\alias{get_account_max_loan}
\title{Get account maximum loan}
\usage{
get_account_max_loan(
  mgn_mode,
  inst_id = NULL,
  ccy = NULL,
  mgn_ccy = NULL,
  trade_quote_ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{mgn_mode}{Character. Margin mode: `"cross"` or `"isolated"`.}

\item{inst_id}{Character or `NULL`. Instrument ID(s).}

\item{ccy}{Character or `NULL`. Currency.}

\item{mgn_ccy}{Character or `NULL`. Margin currency.}

\item{trade_quote_ccy}{Character or `NULL`. Quote currency for trading.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `instId`, `mgnMode`, `mgnCcy`, `maxLoan`, `ccy`,
  and `side`.
}
\description{
Retrieve the maximum loan for manual borrow or margin borrowing scenarios.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_copy_trade.R
\name{get_copy_trade_config}
\alias{get_copy_trade_config}
\title{Get copy trading config}
\usage{
get_copy_trade_config(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with top-level copy trading configuration fields.
  Nested `details` are returned as a JSON string column.
}
\description{
Retrieve your account-level copy trading configuration.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_subtypes}
\alias{get_account_subtypes}
\title{Get account bill subtypes}
\usage{
get_account_subtypes(type = NULL, config, tz = .okx_default_tz)
}
\arguments{
\item{type}{Character or `NULL`. Bill type filter. Multiple values may be
provided as a comma-separated string.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with bill type descriptions and JSON-encoded
  `subTypeDetails`.
}
\description{
Retrieve available account bill types and subtype descriptions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_account_level}
\alias{post_account_set_account_level}
\title{Set Account Level}
\usage{
post_account_set_account_level(
  acct_lv,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{acct_lv}{Account level string, such as `"1"`, `"2"`, `"3"`, or `"4"`.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` confirming the applied account level.
}
\description{
Switch the account mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_mmp_reset}
\alias{post_account_mmp_reset}
\title{Reset MMP Status}
\usage{
post_account_mmp_reset(
  inst_family,
  inst_type = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{inst_family}{Instrument family.}

\item{inst_type}{Optional instrument type. Defaults to `"OPTION"` on OKX.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{A list containing API credentials.}
}
\value{
A `data.frame` with the request result.
}
\description{
Reset market maker protection status for an instrument family.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_orders_algo_pending}
\alias{get_trade_orders_algo_pending}
\title{Get pending algo orders}
\usage{
get_trade_orders_algo_pending(
  ord_type,
  algo_id = NULL,
  inst_type = NULL,
  inst_id = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ord_type}{Character. Algo order type filter.}

\item{algo_id}{Character or `NULL`. Algo order ID.}

\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{inst_id}{Character or `NULL`. Instrument ID filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of results to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps.}
}
\value{
A `data.frame` with pending algo orders.
}
\description{
Retrieve untriggered algo orders.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_set_account_switch_precheck}
\alias{get_account_set_account_switch_precheck}
\title{Precheck account mode switch}
\usage{
get_account_set_account_switch_precheck(
  acct_lv,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{acct_lv}{Character or numeric. Target account mode.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with switch-precheck fields; nested margin and
  unmatched-information structures are JSON-encoded.
}
\description{
Retrieve precheck information and any unmatched requirements for switching to
a target account mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_orders_history_7d}
\alias{get_trade_orders_history_7d}
\title{Get trade orders history (last 7 days)}
\usage{
get_trade_orders_history_7d(
  inst_type = "SWAP",
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
`"SWAP"`, `"FUTURES"`, `"OPTION"`. Default `"SWAP"`.}

\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`. May also include `base_url`.}

\item{tz}{Character. Time zone for parsing timestamps (e.g. `"Asia/Hong_Kong"`).}
}
\value{
A `data.frame` with one row per historical order and columns following the OKX
schema (same layout as pending orders, plus final states). Timestamp columns
are `POSIXct` in `tz`.
}
\description{
Retrieve recent order history for an instrument type.
}
\details{
This wraps `/api/v5/trade/orders-history` and covers about 7 days of data.
Older data is available from OKX's archive endpoint.
}
\note{
Since okxr 0.1.2
}
\section{Common errors}{

- HTTP 401 Unauthorized
- HTTP 400 Bad Request for invalid `inst_type`
}

\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
hist <- get_trade_orders_history_7d(
  inst_type = "SWAP",
  config = cfg,
  tz = "Asia/Hong_Kong"
)
tail(hist)
}

}
\seealso{
[get_trade_order()], [get_trade_orders_pending()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_trade_fee}
\alias{get_account_trade_fee}
\title{Get account trade fee rates}
\usage{
get_account_trade_fee(
  inst_type,
  inst_id = NULL,
  inst_family = NULL,
  group_id = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type. One of `"SPOT"`, `"MARGIN"`,
`"SWAP"`, `"FUTURES"`, `"OPTION"`, or `"EVENTS"`.}

\item{inst_id}{Character or `NULL`. Instrument ID for spot or margin.}

\item{inst_family}{Character or `NULL`. Instrument family for futures, swaps,
or options.}

\item{group_id}{Character or `NULL`. Trading fee group ID.
Use this by itself.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with fee level, fee-rate columns, and any nested
  `feeGroup` or deprecated `fiat` details JSON-encoded as strings.
}
\description{
Retrieve the account's trade fee schedule for a specific instrument type and
optional instrument, instrument family, or trading fee group.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_history_mark_price_candles}
\alias{get_market_history_mark_price_candles}
\title{Get historical mark price candles}
\usage{
get_market_history_mark_price_candles(
  inst_id,
  bar = NULL,
  after = NULL,
  before = NULL,
  limit = 100L,
  config = NULL,
  tz = .okx_default_tz,
  standardize_names = TRUE
)
}
\arguments{
\item{inst_id}{Character. Instrument ID.}

\item{bar}{Character or `NULL`. Bar size.}

\item{after}{Character or `NULL`. Pagination cursor for older rows.}

\item{before}{Character or `NULL`. Pagination cursor for newer rows.}

\item{limit}{Integer. Number of rows to request. Default `100L`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}

\item{standardize_names}{Logical. If `TRUE` (default), renames OHLC columns
to `timestamp`, `open`, `high`, `low`, and `close`.}
}
\value{
A `data.frame` of historical mark-price candlesticks.
}
\description{
Retrieve historical candlestick data for mark price.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_trade.R
\name{get_trade_orders_pending}
\alias{get_trade_orders_pending}
\title{Get all pending trade orders}
\usage{
get_trade_orders_pending(config, tz = .okx_default_tz)
}
\arguments{
\item{config}{List. API credentials/config, typically containing
`api_key`, `secret_key`, and `passphrase`. May also include `base_url`.}

\item{tz}{Character. Time zone for parsing timestamps (e.g. `"Asia/Hong_Kong"`).}
}
\value{
A `data.frame` with one row per pending order and columns following the OKX
schema (e.g., `cTime`, `ordId`, `clOrdId`, `tag`, `instId`, `ordType`, `px`,
`sz`, `side`, `posSide`, `tdMode`, `accFillSz`, `fillPx`, `fillSz`,
`fillTime`, `avgPx`, `state`, `lever`, ...). Timestamp columns are `POSIXct`.
}
\description{
Retrieve all currently open (unfilled) orders for your OKX account.
}
\details{
Returns one row per open order. Timestamps are parsed to `POSIXct` using `tz`.
}
\note{
Since okxr 0.1.1
}
\section{Common errors}{

- HTTP 401 Unauthorized (missing/invalid credentials)
- Rate limiting: HTTP 429 / OKX throttle codes
}

\examples{
\dontrun{
cfg <- list(api_key = "xxx", secret_key = "xxx", passphrase = "xxx")
df <- get_trade_orders_pending(config = cfg, tz = "Asia/Hong_Kong")
head(df)
}

}
\seealso{
[get_trade_order()], [get_trade_orders_history_7d()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_max_avail_size}
\alias{get_account_max_avail_size}
\title{Get maximum available tradable amount}
\usage{
get_account_max_avail_size(
  inst_id,
  td_mode,
  ccy = NULL,
  reduce_only = NULL,
  px = NULL,
  trade_quote_ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. One instrument ID or a comma-separated list of up
to five IDs.}

\item{td_mode}{Character. Trade mode: `"cross"`, `"isolated"`, `"cash"`, or
`"spot_isolated"`.}

\item{ccy}{Character or `NULL`. Margin currency where applicable.}

\item{reduce_only}{Logical, character, or `NULL`. Whether to reduce position
only. Only applicable to margin endpoints that support it.}

\item{px}{Character, numeric, or `NULL`. Optional closing price, when
supported by OKX.}

\item{trade_quote_ccy}{Character or `NULL`. Quote currency used for trading
for spot instruments.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `instId`, `availBuy`, and `availSell`.
}
\description{
Retrieve the maximum available buy and sell amount for an instrument under
the requested trade mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_asset.R
\name{post_asset_cancel_withdrawal}
\alias{post_asset_cancel_withdrawal}
\title{Cancel an Asset Withdrawal}
\usage{
post_asset_cancel_withdrawal(wd_id, tz = .okx_default_tz, config)
}
\arguments{
\item{wd_id}{Withdrawal request ID.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{API credential list.}
}
\value{
A `data.frame` confirming the cancelled withdrawal ID.
}
\description{
Cancel a pending withdrawal request.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_get_generator.R
\name{.make_get_function}
\alias{.make_get_function}
\title{Create a GET request function from an API spec}
\usage{
.make_get_function(api)
}
\arguments{
\item{api}{A list containing API spec fields: \code{okx_path}, \code{query} (string or function),
\code{schema} (data.frame), and optional \code{mode} ("named" or "positional").}
}
\value{
A function with signature \code{(tz = "Asia/Hong_Kong", config, ...)} that:
\itemize{
  \item builds the query string using \code{api$query} (if a function),
  \item signs and sends the GET request,
  \item parses and returns the data using the schema-aware parser.
}
}
\description{
Converts a single entry from \code{.api_GET_specs} into a ready-to-use function
that executes the HTTP GET request, parses the response, and returns a typed data frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_configure.R
\name{post_account_set_leverage}
\alias{post_account_set_leverage}
\title{Set Account Leverage}
\usage{
post_account_set_leverage(
  inst_id,
  lever,
  mgn_mode,
  pos_side = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{inst_id}{Instrument ID (e.g., \code{"BTC-USDT"}).}

\item{lever}{Leverage level to apply (as a string or numeric, e.g., \code{"10"}).}

\item{mgn_mode}{Margin mode: \code{"cross"} or \code{"isolated"}.}

\item{pos_side}{Optional. Position side: \code{"long"} or \code{"short"}. Required for isolated mode.}

\item{tz}{Timezone used for any timestamp parsing (default: \code{"Asia/Hong_Kong"}).}

\item{config}{API credential list with keys `api_key`, `secret_key`,
and `passphrase`.}
}
\value{
A \code{data.frame} with leverage update confirmation (including instrument ID and leverage settings).
}
\description{
Sets the leverage level for a specific trading instrument and margin mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_convert_currency_pair}
\alias{get_asset_convert_currency_pair}
\title{Get convert currency pair}
\usage{
get_asset_convert_currency_pair(
  from_ccy,
  to_ccy,
  convert_mode = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{from_ccy}{Character. Currency to convert from.}

\item{to_ccy}{Character. Currency to convert to.}

\item{convert_mode}{Character or `NULL`. Convert mode selector.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with currency-pair convert metadata.
}
\description{
Retrieve convert metadata for a currency pair.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_convert_contract_coin}
\alias{get_public_convert_contract_coin}
\title{Convert between contract size and currency amount}
\usage{
get_public_convert_contract_coin(
  inst_id,
  sz,
  type = NULL,
  px = NULL,
  unit = NULL,
  op_type = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID.}

\item{sz}{Character or numeric. Quantity to convert.}

\item{type}{Character or `NULL`. Convert type: `"1"` for currency to
contract, `"2"` for contract to currency.}

\item{px}{Character, numeric, or `NULL`. Optional order price.}

\item{unit}{Character or `NULL`. Currency unit, `"coin"` or `"usds"`.}

\item{op_type}{Character or `NULL`. Order type for futures or swaps, such as
`"open"` or `"close"`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with conversion result fields `type`, `instId`, `px`,
  `sz`, and `unit`.
}
\description{
Convert the crypto value to the number of contracts, or vice versa.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_spot_borrow_repay_history}
\alias{get_account_spot_borrow_repay_history}
\title{Get spot borrow and repay history}
\usage{
get_account_spot_borrow_repay_history(
  ccy = NULL,
  type = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter.}

\item{type}{Character or `NULL`. Event type filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier rows.}

\item{before}{Character or `NULL`. Pagination cursor for newer rows.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per borrow or repay event.
}
\description{
Retrieve spot-mode borrow and repay history.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_max_size}
\alias{get_account_max_size}
\title{Get maximum order size}
\usage{
get_account_max_size(
  inst_id,
  td_mode,
  ccy = NULL,
  px = NULL,
  leverage = NULL,
  trade_quote_ccy = NULL,
  outcome = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. One instrument ID or a comma-separated list of up
to five IDs in the same instrument type.}

\item{td_mode}{Character. Trade mode: `"cross"`, `"isolated"`, `"cash"`, or
`"spot_isolated"`.}

\item{ccy}{Character or `NULL`. Margin currency where applicable.}

\item{px}{Character, numeric, or `NULL`. Optional price override.}

\item{leverage}{Character, numeric, or `NULL`. Optional leverage override.}

\item{trade_quote_ccy}{Character or `NULL`. Quote currency used for trading
for spot instruments.}

\item{outcome}{Character or `NULL`. Events market outcome, `"yes"` or `"no"`.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `instId`, `ccy`, `maxBuy`, and `maxSell`.
}
\description{
Retrieve the maximum order quantity allowed for one or more instruments under
the requested trade mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_mmp_config}
\alias{get_account_mmp_config}
\title{Get account MMP configuration}
\usage{
get_account_mmp_config(
  inst_family = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_family}{Character or `NULL`. Instrument family filter.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with MMP configuration fields.
}
\description{
Retrieve option-market-maker-protection configuration for one or more
instrument families.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_premium_history}
\alias{get_public_premium_history}
\title{Get premium history}
\usage{
get_public_premium_history(
  inst_id,
  after = NULL,
  before = NULL,
  bar = NULL,
  limit = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID.}

\item{after}{Character or `NULL`. Pagination cursor for older rows.}

\item{before}{Character or `NULL`. Pagination cursor for newer rows.}

\item{bar}{Character or `NULL`. Bar size such as `"1m"` or `"1H"`.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `instId`, `premium`, and `ts`.
}
\description{
Retrieve premium-index history for an instrument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_bills}
\alias{get_asset_bills}
\title{Get asset bills}
\usage{
get_asset_bills(
  ccy = NULL,
  type = NULL,
  client_id = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{ccy}{Character or `NULL`. Currency filter.}

\item{type}{Character or `NULL`. Bill type filter.}

\item{client_id}{Character or `NULL`. Client-supplied transfer or withdrawal ID.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of results to request.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with asset bill rows.
}
\description{
Retrieve recent funding account bills.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_instrument_tick_bands}
\alias{get_public_instrument_tick_bands}
\title{Get option instrument tick bands}
\usage{
get_public_instrument_tick_bands(
  inst_type = "OPTION",
  inst_family = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character. Instrument type. Currently `"OPTION"`.}

\item{inst_family}{Character or `NULL`. Instrument family filter.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with `instType`, `instFamily`, and JSON-encoded
  `tickBand` details.
}
\description{
Retrieve option tick-band information for one or more option instrument
families.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_market_index_tickers}
\alias{get_market_index_tickers}
\title{Get index tickers}
\usage{
get_market_index_tickers(
  quote_ccy = NULL,
  inst_id = NULL,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{quote_ccy}{Character or `NULL`. Quote currency filter.}

\item{inst_id}{Character or `NULL`. Specific index ID filter.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with one row per index ticker.
}
\description{
Retrieve the latest public index-price snapshots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_asset.R
\name{post_asset_convert_trade}
\alias{post_asset_convert_trade}
\title{Execute an Asset Convert Trade}
\usage{
post_asset_convert_trade(
  quote_id,
  base_ccy,
  quote_ccy,
  side,
  sz,
  sz_ccy,
  cl_t_req_id = NULL,
  tag = NULL,
  convert_mode = NULL,
  tz = .okx_default_tz,
  config
)
}
\arguments{
\item{quote_id}{Quote ID returned by [post_asset_convert_estimate_quote()].}

\item{base_ccy}{Base currency.}

\item{quote_ccy}{Quote currency.}

\item{side}{Trade side, such as `"buy"` or `"sell"`.}

\item{sz}{Trade size.}

\item{sz_ccy}{Currency in which `sz` is specified.}

\item{cl_t_req_id}{Optional client trade request ID.}

\item{tag}{Optional request tag.}

\item{convert_mode}{Optional OKX convert mode.}

\item{tz}{Timezone used for any timestamp parsing.}

\item{config}{API credential list.}
}
\value{
A `data.frame` describing the executed conversion trade.
}
\description{
Execute a confirmed asset conversion against a previously quoted price.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_asset.R
\name{get_asset_transfer_state}
\alias{get_asset_transfer_state}
\title{Get asset transfer state}
\usage{
get_asset_transfer_state(
  trans_id = NULL,
  client_id = NULL,
  type = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{trans_id}{Character or `NULL`. Transfer ID.}

\item{client_id}{Character or `NULL`. Client-supplied transfer ID.}

\item{type}{Character or `NULL`. Transfer type.}

\item{config}{API credentials as a list with \code{api_key}, \code{secret_key},
and \code{passphrase}.}

\item{tz}{Timezone string.}
}
\value{
A \code{data.frame} with transfer state rows.
}
\description{
Retrieve the state of a funding transfer.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_bills}
\alias{get_account_bills}
\title{Get account bills}
\usage{
get_account_bills(
  inst_type = NULL,
  ccy = NULL,
  mgn_mode = NULL,
  ct_type = NULL,
  type = NULL,
  sub_type = NULL,
  after = NULL,
  before = NULL,
  limit = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_type}{Character or `NULL`. Instrument type filter.}

\item{ccy}{Character or `NULL`. Currency filter.}

\item{mgn_mode}{Character or `NULL`. Margin mode filter.}

\item{ct_type}{Character or `NULL`. Contract type filter.}

\item{type}{Character or `NULL`. Bill type filter.}

\item{sub_type}{Character or `NULL`. Bill subtype filter.}

\item{after}{Character or `NULL`. Pagination cursor for earlier records.}

\item{before}{Character or `NULL`. Pagination cursor for newer records.}

\item{limit}{Integer or `NULL`. Number of rows to request.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with account bill rows.
}
\description{
Retrieve account bill details from the last 7 days.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_account.R
\name{get_account_interest_limits}
\alias{get_account_interest_limits}
\title{Get account borrow interest and limits}
\usage{
get_account_interest_limits(
  type = NULL,
  ccy = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{type}{Character or `NULL`. Loan type. Currently `"2"` for market loans.}

\item{ccy}{Character or `NULL`. Loan currency.}

\item{config}{List. API credentials/config.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` with account-level limit fields; nested per-currency
  `records` are JSON-encoded.
}
\description{
Retrieve account-level debt, next accrual timestamps, and nested per-currency
borrowing-limit records.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_post_trade.R
\name{post_trade_cancel_order}
\alias{post_trade_cancel_order}
\title{Cancel a Trade Order}
\usage{
post_trade_cancel_order(
  inst_id,
  ord_id = NULL,
  cl_ord_id = NULL,
  config,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Instrument ID (e.g., \code{"BTC-USDT"}).}

\item{ord_id}{Optional OKX order ID to cancel.}

\item{cl_ord_id}{Optional client order ID to cancel. Provide this or
`ord_id`.}

\item{config}{A list with API credentials: \code{api_key}, \code{secret_key}, \code{passphrase}.}

\item{tz}{Timezone for parsing any timestamps (default: \code{"Asia/Hong_Kong"}).}
}
\value{
A \code{data.frame} containing cancellation result and timestamp.
}
\description{
Submits a cancellation request for a previously placed trade order.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_funding_rate}
\alias{get_public_funding_rate}
\title{Get current funding rate}
\usage{
get_public_funding_rate(
  inst_id,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT-SWAP"`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` containing the current funding rate fields returned by OKX.
}
\description{
Retrieve the current funding rate for a perpetual swap instrument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_get_market.R
\name{get_public_funding_rate_history}
\alias{get_public_funding_rate_history}
\title{Get funding rate history}
\usage{
get_public_funding_rate_history(
  inst_id,
  before = NULL,
  after = NULL,
  limit = 400,
  config = NULL,
  tz = .okx_default_tz
)
}
\arguments{
\item{inst_id}{Character. Instrument ID, e.g. `"BTC-USDT-SWAP"`.}

\item{before}{Optional cursor for records earlier than the supplied value.}

\item{after}{Optional cursor for records later than the supplied value.}

\item{limit}{Integer. Number of records to request. Default `400`.}

\item{config}{Optional list. Public endpoint request options, such as
`timeout`; credentials are not required.}

\item{tz}{Character. Time zone for parsing timestamps. Default `"Asia/Hong_Kong"`.}
}
\value{
A `data.frame` containing funding rate history rows returned by OKX.
}
\description{
Retrieve historical funding rate entries for a perpetual swap instrument.
}
